@@ -32,12 +32,15 @@
 #include <arc/credential/VOMSAttribute.h>
 #include <arc/credential/VOMSUtil.h>
 #include <arc/credential/Credential.h>
+#include <arc/credential/CertUtil.h>
 #include <arc/credentialstore/CredentialStore.h>
 #include <arc/crypto/OpenSSL.h>
 #include <arc/FileUtils.h>
 
 #include <openssl/ui.h>
 
+using namespace ArcCredential;
+
 static int create_proxy_file(const std::string& path) {
   int f = -1;
 
@@ -89,7 +92,7 @@ static void tls_process_error(Arc::Logger& logger) {
   return;
 }
 
-#define PASS_MIN_LENGTH 6
+#define PASS_MIN_LENGTH (0)
 static int input_password(char *password, int passwdsz, bool verify,
                           const std::string& prompt_info,
                           const std::string& prompt_verify_info,
@@ -99,8 +102,8 @@ static int input_password(char *password, int passwdsz, bool verify,
   ui = UI_new();
   if (ui) {
     int ok = 0;
-    char buf[256];
-    memset(buf, 0, 256);
+    char* buf = new char[passwdsz];
+    memset(buf, 0, passwdsz);
     int ui_flags = 0;
     char *prompt1 = NULL;
     char *prompt2 = NULL;
@@ -109,17 +112,18 @@ static int input_password(char *password, int passwdsz, bool verify,
     ui_flags |= UI_INPUT_FLAG_DEFAULT_PWD;
     UI_ctrl(ui, UI_CTRL_PRINT_ERRORS, 1, 0, 0);
     ok = UI_add_input_string(ui, prompt1, ui_flags, password,
-                             PASS_MIN_LENGTH, BUFSIZ - 1);
-    if (ok >= 0 && verify)
+                             PASS_MIN_LENGTH, passwdsz - 1);
+    if (ok >= 0 && verify) {
       ok = UI_add_verify_string(ui, prompt2, ui_flags, buf,
-                                PASS_MIN_LENGTH, BUFSIZ - 1, password);
-    if (ok >= 0)
-      do
+                                PASS_MIN_LENGTH, passwdsz - 1, password);
+    }
+    if (ok >= 0) {
+      do {
         ok = UI_process(ui);
-      while (ok < 0 && UI_ctrl(ui, UI_CTRL_IS_REDOABLE, 0, 0, 0));
+      } while (ok < 0 && UI_ctrl(ui, UI_CTRL_IS_REDOABLE, 0, 0, 0));
+    }
 
-    if (ok >= 0)
-      res = strlen(password);
+    if (ok >= 0) res = strlen(password);
     if (ok == -1) {
       logger.msg(Arc::ERROR, "User interface error");
       tls_process_error(logger);
@@ -132,6 +136,7 @@ static int input_password(char *password, int passwdsz, bool verify,
       res = 0;
     }
     UI_free(ui);
+    delete[] buf;
     OPENSSL_free(prompt1);
     OPENSSL_free(prompt2);
   }
@@ -186,25 +191,6 @@ static std::string tokens_to_string(std::vector<std::string> tokens) {
   return s;
 }
 
-static void convert_period(std::string& period) {
-  std::string ret;
-  if (!period.empty() && ((period.rfind("h") != std::string::npos) || (period.rfind("H") != std::string::npos))) {
-    //If the validityPeriod is set as hours  
-    unsigned long tmp;
-    tmp = strtoll(period.c_str(), NULL, 0);
-    tmp = tmp * 3600;
-    period = Arc::tostring(tmp);
-  }
-  else if (!period.empty() && ((period.rfind("d") != std::string::npos) || (period.rfind("D") != std::string::npos))) {
-    //If the validityPeriod is set as days
-    unsigned long tmp;
-    tmp = strtoll(period.c_str(), NULL, 0);
-    tmp = tmp * 3600 * 24;
-    period = Arc::tostring(tmp);
-  }
-}
-
-
 int main(int argc, char *argv[]) {
 
   setlocale(LC_ALL, "");
@@ -588,98 +574,98 @@ int main(int argc, char *argv[]) {
   //proxy validity period
   //Set the default proxy validity lifetime to 12 hours if there is
   //no validity lifetime provided by command caller
-  if ((constraints["validityEnd"].empty()) &&
-      (constraints["validityPeriod"].empty())) {
-    if (myproxy_command == "put" || myproxy_command == "PUT" || myproxy_command == "Put")
-      constraints["validityPeriod"] = "604800";    
-      //For myproxy PUT operation, the proxy should be 7 days according to the default 
-      //definition in myproxy implementation.
-    else
-      constraints["validityPeriod"] = "43200";
+  // Set default values first
+  // TODO: Is default validityPeriod since now or since validityStart?
+  Arc::Time validityStart = now; // now by default
+  Arc::Period validityPeriod(12*60*60);
+  if (myproxy_command == "put" || myproxy_command == "PUT" || myproxy_command == "Put") {
+    //For myproxy PUT operation, the proxy should be 7 days according to the default 
+    //definition in myproxy implementation.
+    validityPeriod = 7*24*60*60;
   }
-  else if(!(constraints["validityEnd"].empty()) && (constraints["validityPeriod"].empty())){
-    constraints["validityPeriod"] = constraints["validityStart"].empty() ? (Arc::Time(constraints["validityEnd"]) - now) : (Arc::Time(constraints["validityEnd"]) - (Arc::Time(constraints["validityStart"]) > now ? Arc::Time(constraints["validityStart"]) : now));
+  // Acquire constraints. Check for valid values and conflicts.
+  if((!constraints["validityStart"].empty()) && 
+     (!constraints["validityEnd"].empty()) &&
+     (!constraints["validityPeriod"].empty())) {
+    std::cerr << Arc::IString("The start, end and period can't be set simultaneously") << std::endl;
+    return EXIT_FAILURE;
   }
-  else if((constraints["validityEnd"].empty()) && !(constraints["validityPeriod"].empty())){
-    Arc::Time start = constraints["validityStart"].empty() ? now : Arc::Time(constraints["validityStart"]);
-    if(start < now) {
-      std::cout << Arc::IString("The start time that you set: %s is before current time: %s", (std::string)start, (std::string)now) << std::endl;
-      Arc::Period prd = Arc::Period(constraints["validityPeriod"]);
-      if(start + prd < now) {
-        std::cout << Arc::IString("The start time that you set plus validityPeriod: %s is before current time: %s.\nPlease set the time constraints once again.", (std::string)(start + constraints["validityPeriod"]), (std::string)now) << std::endl;
-        return EXIT_FAILURE;
-      }
-      else {
-        std::string tmp = Arc::tostring(Arc::Period(prd.GetPeriod() - (now - start).GetPeriod()));
-        std::cout << Arc::IString("The start time that you set plus validityPeriod: %s is after current time: %s.\nThe validityPeriod will be shorten to %s.", (std::string)(start + constraints["validityPeriod"]), (std::string)now, tmp) << std::endl;
-        constraints["validityPeriod"] = tmp;
-      }
-      constraints["validityStart"] = now;
-    }
-  }
-  else {
-    Arc::Time start = constraints["validityStart"].empty() ? now : Arc::Time(constraints["validityStart"]);
-    if(start < now) {  
-      std::cout << Arc::IString("The start time that you set: %s is before current time: %s.\nThe current time will be used as start time.", (std::string)start, (std::string)now) << std::endl;
-      start = now;
-      constraints["validityStart"] = now;
-    }
-    else if(start > now)
-      std::cout << Arc::IString("The start time that you set: %s is after current time: %s.", (std::string)start, (std::string)now) << std::endl;
-
-    Arc::Time end = Arc::Time(constraints["validityEnd"]);
-    if(end > start) {
-      if((end - start) >= Arc::Period(constraints["validityPeriod"])) {
-        std::cout << Arc::IString("The end time that you set: %s is after the start time plus validityPeriod: %s.\n The validityPeriod will not be changed.\n", (std::string)end, (std::string)(start+Arc::Period(constraints["validityPeriod"]))) << std::endl;
-      }
-      else {
-        std::cout << Arc::IString("The end time that you set: %s is before the start time plus validityPeriod: %s.\nThe validityPeriod will be shorten to: %s.", (std::string)end, (std::string)(start+Arc::Period(constraints["validityPeriod"])), Arc::tostring(end - start)) << std::endl;
-        constraints["validityPeriod"] = Arc::tostring((end - start).GetPeriod()); 
-      }
-    }
-    else { 
-      std::cout << Arc::IString("The end time that you set: %s is before start time: %s.\nPlease set the time constraints once again.\n", (std::string)end, (std::string)(start)) << std::endl;
+  if(!constraints["validityStart"].empty()) {
+    validityStart = Arc::Time(constraints["validityStart"]);
+    if (validityStart == Arc::Time(Arc::Time::UNDEFINED)) {
+    std::cerr << Arc::IString("The start time that you set: %s can't be recognized.", (std::string)constraints["validityStart"]) << std::endl;
       return EXIT_FAILURE;
     }
   }
-  
-  if(constraints["validityPeriod"].empty()) {
-    if (myproxy_command == "put" || myproxy_command == "PUT" || myproxy_command == "Put")
-      constraints["validityPeriod"] = "604800";
-    else
-      constraints["validityPeriod"] = "43200";
+  if(!constraints["validityPeriod"].empty()) {
+    validityPeriod = Arc::Period(constraints["validityPeriod"]);
+    if (validityPeriod.GetPeriod() <= 0) {
+      std::cerr << Arc::IString("The period that you set: %s can't be recognized.", (std::string)constraints["validityPeriod"]) << std::endl;
+      return EXIT_FAILURE;
+    }
+  }
+  if(!constraints["validityEnd"].empty()) {
+    Arc::Time validityEnd = Arc::Time(constraints["validityEnd"]);
+    if (validityEnd == Arc::Time(Arc::Time::UNDEFINED)) {
+      std::cerr << Arc::IString("The end time that you set: %s can't be recognized.", (std::string)constraints["validityEnd"]) << std::endl;
+      return EXIT_FAILURE;
+    }
+    if(!constraints["validityPeriod"].empty()) {
+      // If period is explicitely set then start is derived from end and period
+      validityStart = validityEnd - validityPeriod;
+    } else {
+      // otherwise start - optionally - and end are set, period is derived
+      if(validityEnd < validityStart) {
+        std::cerr << Arc::IString("The end time that you set: %s is before start time:%s.", (std::string)validityEnd,(std::string)validityStart) << std::endl;
+        // error
+        return EXIT_FAILURE;
+      }
+      validityPeriod = validityEnd - validityStart;
+    }
+  }
+  // Here we have validityStart and validityPeriod defined
+  Arc::Time validityEnd = validityStart + validityPeriod;
+  // Warn user about strange times but do not prevent user from doing anything legal
+  if(validityStart < now) {
+    std::cout << Arc::IString("WARNING: The start time that you set: %s is before current time: %s", (std::string)validityStart, (std::string)now) << std::endl;
+  }
+  if(validityEnd < now) {
+    std::cout << Arc::IString("WARNING: The end time that you set: %s is before current time: %s", (std::string)validityEnd, (std::string)now) << std::endl;
   }
-  convert_period(constraints["validityPeriod"]);
- 
-  //voms AC valitity period
-  if(!constraints["vomsACvalidityPeriod"].empty())
-    convert_period(constraints["vomsACvalidityPeriod"]);
 
+  //voms AC valitity period
   //Set the default voms AC validity lifetime to 12 hours if there is
   //no validity lifetime provided by command caller
-  if (constraints["vomsACvalidityPeriod"].empty()) {
-    unsigned long period_val = Arc::Period(constraints["validityPeriod"]).GetPeriod();
-    if(period_val<=43200)
-      constraints["vomsACvalidityPeriod"] = constraints["validityPeriod"];
-    else 
-      constraints["vomsACvalidityPeriod"] = "43200";
+  Arc::Period vomsACvalidityPeriod(12*60*60);
+  if(!constraints["vomsACvalidityPeriod"].empty()) {
+    vomsACvalidityPeriod = Arc::Period(constraints["vomsACvalidityPeriod"]);
+    if (vomsACvalidityPeriod.GetPeriod() == 0) {
+      std::cerr << Arc::IString("The VOMS AC period that you set: %s can't be recognized.", (std::string)constraints["vomsACvalidityPeriod"]) << std::endl;
+      return EXIT_FAILURE;
+    }
+  } else {
+    if(validityPeriod < vomsACvalidityPeriod) vomsACvalidityPeriod = validityPeriod;
+    // It is strange that VOMS AC may be valid less than proxy itself.
+    // Maybe it would be more correct to have it valid by default from 
+    // now till validityEnd.
   }
-  std::string voms_period = constraints["vomsACvalidityPeriod"];
+  std::string voms_period = Arc::tostring(vomsACvalidityPeriod.GetPeriod());
 
   //myproxy validity period.
-  if(!constraints["myproxyvalidityPeriod"].empty())
-    convert_period(constraints["myproxyvalidityPeriod"]);
-
   //Set the default myproxy validity lifetime to 12 hours if there is
   //no validity lifetime provided by command caller
-  if (constraints["myproxyvalidityPeriod"].empty()) {
-    unsigned long period_val = Arc::Period(constraints["validityPeriod"]).GetPeriod();
-    if(period_val<=43200)
-      constraints["myproxyvalidityPeriod"] = constraints["validityPeriod"];
-    else  
-      constraints["myproxyvalidityPeriod"] = "43200";
+  Arc::Period myproxyvalidityPeriod(12*60*60);
+  if(!constraints["myproxyvalidityPeriod"].empty()) {
+    myproxyvalidityPeriod = Arc::Period(constraints["myproxyvalidityPeriod"]);
+    if (myproxyvalidityPeriod.GetPeriod() == 0) {
+      std::cerr << Arc::IString("The MyProxy period that you set: %s can't be recognized.", (std::string)constraints["myproxyvalidityPeriod"]) << std::endl;
+      return EXIT_FAILURE;
+    }
+  } else {
+    if(validityPeriod < myproxyvalidityPeriod) myproxyvalidityPeriod = validityPeriod;
+    // see vomsACvalidityPeriod
   }
-  std::string myproxy_period = constraints["myproxyvalidityPeriod"];
+  std::string myproxy_period = Arc::tostring(myproxyvalidityPeriod.GetPeriod());
 
 
   Arc::OpenSSLInit();
@@ -866,7 +852,8 @@ int main(int argc, char *argv[]) {
 
       std::string proxy_cred_str_pem;
      
-      Arc::UserConfig usercfg_tmp(Arc::initializeCredentialsType(Arc::initializeCredentialsType::SkipCredentials));
+      Arc::initializeCredentialsType cred_type(Arc::initializeCredentialsType::SkipCredentials);
+      Arc::UserConfig usercfg_tmp(cred_type);
       usercfg_tmp.CACertificatesDirectory(usercfg.CACertificatesDirectory());
 
       Arc::CredentialStore cstore(usercfg_tmp,Arc::URL("myproxy://"+myproxy_server));
@@ -895,14 +882,29 @@ int main(int argc, char *argv[]) {
     return EXIT_FAILURE;
   }
 
-  Arc::Time proxy_start = constraints["validityStart"].empty() ? now : Arc::Time(constraints["validityStart"]);
-  Arc::Period proxy_period = Arc::Period(constraints["validityPeriod"]);
+  Arc::Time proxy_start = validityStart;
+  Arc::Period proxy_period = validityPeriod;
+  if (constraints["validityStart"].empty() && constraints["validityEnd"].empty()) {
+    // If start/end is not explicitely specified then add 5 min back gap.
+    proxy_start = proxy_start - Arc::Period(300);
+    proxy_period.SetPeriod(proxy_period.GetPeriod() + 300);
+  }
 
   //Create proxy or voms proxy
   try {
-    std::cout << Arc::IString("Your identity: %s", Arc::Credential(cert_path, "", "", "").GetDN()) << std::endl;
 
     Arc::Credential signer(cert_path, key_path, "", "");
+    if (signer.GetIdentityName().empty()) {
+      std::cerr << Arc::IString("Proxy generation failed: No valid certificate found.") << std::endl;
+      return EXIT_FAILURE;
+    }
+    EVP_PKEY* pkey = signer.GetPrivKey();
+    if(!pkey) {
+      std::cerr << Arc::IString("Proxy generation failed: No valid private key found.") << std::endl;
+      return EXIT_FAILURE;
+    }
+    if(pkey) EVP_PKEY_free(pkey);
+    std::cout << Arc::IString("Your identity: %s", Arc::Credential(cert_path, "", "", "").GetIdentityName()) << std::endl;
     if (now > signer.GetEndTime()) {
       std::cerr << Arc::IString("Proxy generation failed: Certificate has expired.") << std::endl;
       return EXIT_FAILURE;
@@ -918,7 +920,7 @@ int main(int argc, char *argv[]) {
     std::string req_str;
     std::string policy;
     policy = constraints["proxyPolicy"].empty() ? constraints["proxyPolicyFile"] : constraints["proxyPolicy"];
-    Arc::Credential cred_request(proxy_start - Arc::Period(300), proxy_period.GetPeriod() + 300, keybits);
+    Arc::Credential cred_request(proxy_start, proxy_period, keybits);
     cred_request.GenerateRequest(req_str);
     cred_request.OutputPrivatekey(private_key);
     signer.OutputCertificate(signing_cert);
@@ -1058,6 +1060,7 @@ int main(int argc, char *argv[]) {
         voms_server = (*it).first;
         std::vector<std::vector<std::string> > voms_lines = (*it).second;
 
+        bool succeeded = false; //a boolean value to indicate if there is valid message returned from voms server, by using the current voms_line  
         for (std::vector<std::vector<std::string> >::iterator line_it = voms_lines.begin(); line_it != voms_lines.end(); line_it++) {
           std::vector<std::string> voms_line = *line_it;
           int count = server_command_map.count(voms_server);
@@ -1125,39 +1128,41 @@ int main(int argc, char *argv[]) {
           Arc::PayloadStreamInterface *response = NULL;
           Arc::MCC_Status status = client.process(&request, &response, true);
           if (!status) {
-            logger.msg(Arc::ERROR, (std::string)status);
+            //logger.msg(Arc::ERROR, (std::string)status);
             if (response)
               delete response;
               std::cout << Arc::IString("The VOMS server with the information:\n%s\"\ncan not be reached, please make sure it is available", tokens_to_string(voms_line)) << std::endl;
-              break;
-              //return EXIT_FAILURE;
+              continue; //There could be another voms replicated server with the same name exists
           }
           if (!response) {
             logger.msg(Arc::ERROR, "No stream response from VOMS server");
-            break;
-            //return EXIT_FAILURE;
+            continue;
           }
           Arc::XMLNode node;
-          {
-            std::string ret_str;
-            char ret_buf[1024];
-            int len = sizeof(ret_buf);
-            while(response->Get(ret_buf, len)) {
-              ret_str.append(ret_buf, len);
-              len = sizeof(ret_buf);
-            };
-            logger.msg(Arc::VERBOSE, "Returned message from VOMS server: %s", ret_str);
-            Arc::XMLNode(ret_str).Exchange(node);
+          std::string ret_str;
+          char ret_buf[1024];
+          int len = sizeof(ret_buf);
+          while(response->Get(ret_buf, len)) {
+            ret_str.append(ret_buf, len);
+            len = sizeof(ret_buf);
           };
+          logger.msg(Arc::VERBOSE, "Returned message from VOMS server: %s", ret_str);
+          Arc::XMLNode(ret_str).Exchange(node);
           if((!node) || ((bool)(node["error"]))) {
-            std::string str = node["error"]["item"]["message"];
-            std::string::size_type pos;
-            if((pos = str.find("The validity of this VOMS AC in your proxy is shortened to"))!= std::string::npos) {
-              std::string tmp = str.substr(pos + 59); 
-              std::cout << Arc::IString("The validity duration of VOMS AC is shortened from %s to %s, due to the validity constraint on voms server side.\n", voms_period, tmp);
+            if((bool)(node["error"])) {
+              std::string str = node["error"]["item"]["message"];
+              std::string::size_type pos;
+              if((pos = str.find("The validity of this VOMS AC in your proxy is shortened to"))!= std::string::npos) {
+                std::string tmp = str.substr(pos + 59); 
+                std::cout << Arc::IString("The validity duration of VOMS AC is shortened from %s to %s, due to the validity constraint on voms server side.\n", voms_period, tmp);
+              }
+              else
+              std::cout << Arc::IString("Cannot get any AC or attributes info from VOMS server: %s;\n       Returned message from VOMS server: %s\n", voms_server, str);
+              break; //since the voms servers with the same name should be looked as the same for robust reason, the other voms server should that can be reached could returned the same message. So we exists the loop, even if there are other backup voms server exist.
             }
             else
-              std::cout << Arc::IString("Cannot get any AC or attributes info from VOMS server: %s;\n       Returned message from VOMS server: %s\n", voms_server, str);
+              std::cout << Arc::IString("Returned message from VOMS server %s is: s%\n", voms_server, ret_str);
+              break; 
           }
 
           //Put the return attribute certificate into proxy certificate as the extension part
@@ -1179,7 +1184,7 @@ int main(int argc, char *argv[]) {
           if (command == "list") {
             //logger.msg(Arc::INFO, "The attribute information from voms server: %s is list as following:\n%s",
             //           voms_server, decodedac);
-            std::cout << Arc::IString("The attribute information from VOMS server: %s is list as following", voms_server) << std::endl << decodedac;
+            std::cout << Arc::IString("The attribute information from VOMS server: %s is list as following:", voms_server) << std::endl << decodedac << std::endl;
             if (response)
               delete response;
             return EXIT_SUCCESS;
@@ -1189,7 +1194,12 @@ int main(int argc, char *argv[]) {
             delete response;
 
           Arc::addVOMSAC(aclist, acorder, decodedac);
+          succeeded = true; break;
         }//end of the scanning of multiple vomses lines with the same name
+        if(succeeded==false) {
+          if(voms_lines.size() > 1) 
+            std::cout << Arc::IString("There are %d servers with the same name: %s in your vomses file, but all of them can not been reached, or can return valid message. But proxy without voms AC extension will still be generated.", voms_lines.size(), voms_server) << std::endl; 
+        }
       }
 
       //Put the returned attribute certificate into proxy certificate
@@ -1199,10 +1209,22 @@ int main(int argc, char *argv[]) {
         cred_request.AddExtension("order", acorder);
     }
 
-    if (!use_gsi_proxy)
-      cred_request.SetProxyPolicy("rfc", policy.empty() ? "inheritAll" : "anylanguage", policy, -1);
-    else
+    if (!use_gsi_proxy) {
+      if(!policy.empty()) {
+        cred_request.SetProxyPolicy("rfc", "anylanguage", policy, -1);
+      } else if(CERT_IS_LIMITED_PROXY(signer.GetType())) {
+        // Gross hack for globus. If Globus marks own proxy as limited
+        // it expects every derived proxy to be limited or at least
+        // independent. Independent proxies has little sense in Grid
+        // world. So here we make our proxy globus-limited to allow
+        // it to be used with globus code.
+        cred_request.SetProxyPolicy("rfc", "limited", policy, -1);
+      } else {
+        cred_request.SetProxyPolicy("rfc", "inheritAll", policy, -1);
+      }
+    } else {
       cred_request.SetProxyPolicy("gsi2", "", "", -1);
+    }
 
     std::string proxy_cert;
     if (!signer.SignRequest(&cred_request, proxy_cert))
@@ -1273,7 +1295,7 @@ int main(int argc, char *argv[]) {
       if(!retrievable_by_cert.empty()) {
         myproxyopt["retriever_trusted"] = retrievable_by_cert;
       }
-      if(!cstore.Store(myproxyopt,proxy_cred_str_pem,true,proxy_start - Arc::Period(300),proxy_period.GetPeriod() + 300))
+      if(!cstore.Store(myproxyopt,proxy_cred_str_pem,true,proxy_start,proxy_period))
         throw std::invalid_argument("Failed to delegate proxy to MyProxy service");
 
       remove_proxy_file(proxy_path);
