@@ -22,6 +22,7 @@
 #include <arc/data/DataHandle.h>
 #include <arc/data/DataMover.h>
 #include <arc/data/URLMap.h>
+#include <arc/loader/FinderLoader.h>
 #include <arc/OptionParser.h>
 
 static Arc::Logger logger(Arc::Logger::getRootLogger(), "arccp");
@@ -55,6 +56,7 @@ static void progress(FILE *o, const char*, unsigned int,
 
 bool arcregister(const Arc::URL& source_url,
                  const Arc::URL& destination_url,
+                 const std::list<std::string>& locations,
                  Arc::UserConfig& usercfg,
                  bool secure,
                  bool passive,
@@ -72,12 +74,12 @@ bool arcregister(const Arc::URL& source_url,
       destination_url.Protocol() == "urllist") {
     std::list<Arc::URL> sources = Arc::ReadURLList(source_url);
     std::list<Arc::URL> destinations = Arc::ReadURLList(destination_url);
-    if (sources.size() == 0) {
+    if (sources.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of sources from file %s",
                  source_url.Path());
       return false;
     }
-    if (destinations.size() == 0) {
+    if (destinations.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of destinations from file %s",
                  destination_url.Path());
       return false;
@@ -92,13 +94,13 @@ bool arcregister(const Arc::URL& source_url,
                                        destination = destinations.begin();
          (source != sources.end()) && (destination != destinations.end());
          source++, destination++)
-      if(!arcregister(*source, *destination, usercfg, secure, passive,
+      if(!arcregister(*source, *destination, locations, usercfg, secure, passive,
                       force_meta, timeout)) r = false;
     return r;
   }
   if (source_url.Protocol() == "urllist") {
     std::list<Arc::URL> sources = Arc::ReadURLList(source_url);
-    if (sources.size() == 0) {
+    if (sources.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of sources from file %s",
                  source_url.Path());
       return false;
@@ -106,13 +108,13 @@ bool arcregister(const Arc::URL& source_url,
     bool r = true;
     for (std::list<Arc::URL>::iterator source = sources.begin();
          source != sources.end(); source++)
-      if(!arcregister(*source, destination_url, usercfg, secure, passive,
+      if(!arcregister(*source, destination_url, locations, usercfg, secure, passive,
                       force_meta, timeout)) r = false;
     return r;
   }
   if (destination_url.Protocol() == "urllist") {
     std::list<Arc::URL> destinations = Arc::ReadURLList(destination_url);
-    if (destinations.size() == 0) {
+    if (destinations.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of destinations from file %s",
                  destination_url.Path());
       return false;
@@ -120,7 +122,7 @@ bool arcregister(const Arc::URL& source_url,
     bool r = true;
     for (std::list<Arc::URL>::iterator destination = destinations.begin();
          destination != destinations.end(); destination++)
-      if(!arcregister(source_url, *destination, usercfg, secure, passive,
+      if(!arcregister(source_url, *destination, locations, usercfg, secure, passive,
                       force_meta, timeout)) r = false;
     return r;
   }
@@ -152,6 +154,11 @@ bool arcregister(const Arc::URL& source_url,
                " and destination must be indexing service");
     return false;
   }
+  if (!locations.empty()) {
+    std::string meta(destination->GetURL().Protocol()+"://"+destination->GetURL().Host());
+    for (std::list<std::string>::const_iterator i = locations.begin(); i != locations.end(); ++i)
+      destination->AddLocation(*i, meta);
+  }
   // Obtain meta-information about source
   Arc::FileInfo fileinfo;
   Arc::DataPoint::DataPointInfoType verb = (Arc::DataPoint::DataPointInfoType)Arc::DataPoint::INFO_TYPE_CONTENT;
@@ -201,6 +208,7 @@ bool arcregister(const Arc::URL& source_url,
 
 bool arccp(const Arc::URL& source_url_,
            const Arc::URL& destination_url_,
+           const std::list<std::string>& locations,
            const std::string& cache_dir,
            Arc::UserConfig& usercfg,
            bool secure,
@@ -220,8 +228,6 @@ bool arccp(const Arc::URL& source_url_,
     logger.msg(Arc::ERROR, "Invalid URL: %s", destination_url.str());
     return false;
   }
-  std::string cache_path;
-  std::string id = "<ngcp>";
 
   if (timeout <= 0)
     timeout = 300; // 5 minute default
@@ -231,12 +237,12 @@ bool arccp(const Arc::URL& source_url_,
       destination_url.Protocol() == "urllist") {
     std::list<Arc::URL> sources = Arc::ReadURLList(source_url);
     std::list<Arc::URL> destinations = Arc::ReadURLList(destination_url);
-    if (sources.size() == 0) {
+    if (sources.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of sources from file %s",
                  source_url.Path());
       return false;
     }
-    if (destinations.size() == 0) {
+    if (destinations.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of destinations from file %s",
                  destination_url.Path());
       return false;
@@ -251,13 +257,13 @@ bool arccp(const Arc::URL& source_url_,
                                        destination = destinations.begin();
          (source != sources.end()) && (destination != destinations.end());
          source++, destination++)
-      if(!arccp(*source, *destination, cache_dir, usercfg, secure, passive,
+      if(!arccp(*source, *destination, locations, cache_dir, usercfg, secure, passive,
                 force_meta, recursion, tries, verbose, timeout)) r = false;
     return r;
   }
   if (source_url.Protocol() == "urllist") {
     std::list<Arc::URL> sources = Arc::ReadURLList(source_url);
-    if (sources.size() == 0) {
+    if (sources.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of sources from file %s",
                  source_url.Path());
       return false;
@@ -265,14 +271,14 @@ bool arccp(const Arc::URL& source_url_,
     bool r = true;
     for (std::list<Arc::URL>::iterator source = sources.begin();
          source != sources.end(); source++)
-      if(!arccp(*source, destination_url, cache_dir, usercfg, secure,
+      if(!arccp(*source, destination_url, locations, cache_dir, usercfg, secure,
                 passive, force_meta, recursion, tries, verbose, timeout))
         r = false;
     return r;
   }
   if (destination_url.Protocol() == "urllist") {
     std::list<Arc::URL> destinations = Arc::ReadURLList(destination_url);
-    if (destinations.size() == 0) {
+    if (destinations.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of destinations from file %s",
                  destination_url.Path());
       return false;
@@ -280,7 +286,7 @@ bool arccp(const Arc::URL& source_url_,
     bool r = true;
     for (std::list<Arc::URL>::iterator destination = destinations.begin();
          destination != destinations.end(); destination++)
-      if(!arccp(source_url, *destination, cache_dir, usercfg, secure,
+      if(!arccp(source_url, *destination, locations, cache_dir, usercfg, secure,
                 passive, force_meta, recursion, tries, verbose, timeout))
         r = false;
     return r;
@@ -369,6 +375,11 @@ bool arccp(const Arc::URL& source_url_,
           logger.msg(Arc::INFO, "Unsupported destination url: %s", d_url.str());
           continue;
         }
+        if (!locations.empty() && destination->IsIndex()) {
+          std::string meta(destination->GetURL().Protocol()+"://"+destination->GetURL().Host());
+          for (std::list<std::string>::const_iterator i = locations.begin(); i != locations.end(); ++i)
+            destination->AddLocation(*i, meta);
+        }
         Arc::DataMover mover;
         mover.secure(secure);
         mover.passive(passive);
@@ -421,7 +432,7 @@ bool arccp(const Arc::URL& source_url_,
           d_url += i->GetName();
           s_url += "/";
           d_url += "/";
-          if(!arccp(s_url, d_url, cache_dir, usercfg, secure, passive,
+          if(!arccp(s_url, d_url, locations, cache_dir, usercfg, secure, passive,
                     force_meta, recursion - 1, tries, verbose, timeout))
             r = false;
         }
@@ -446,6 +457,11 @@ bool arccp(const Arc::URL& source_url_,
                destination_url.str());
     return false;
   }
+  if (!locations.empty() && destination->IsIndex()) {
+    std::string meta(destination->GetURL().Protocol()+"://"+destination->GetURL().Host());
+    for (std::list<std::string>::const_iterator i = locations.begin(); i != locations.end(); ++i)
+      destination->AddLocation(*i, meta);
+  }
   Arc::DataMover mover;
   mover.secure(secure);
   mover.passive(passive);
@@ -469,10 +485,7 @@ bool arccp(const Arc::URL& source_url_,
   if (cache)
     cache.Release();
   if (!res.Passed()) {
-    if (!res.GetDesc().empty())
-      logger.msg(Arc::ERROR, "Transfer FAILED: %s - %s", std::string(res), res.GetDesc());
-    else
-      logger.msg(Arc::ERROR, "Transfer FAILED: %s", std::string(res));
+    logger.msg(Arc::ERROR, "Transfer FAILED: %s", std::string(res));
     if (res.Retryable())
       logger.msg(Arc::ERROR, "This seems like a temporary error, please try again later");
     return false;
@@ -556,6 +569,19 @@ int main(int argc, char **argv) {
                     istring("number of retries before failing file transfer"),
                     istring("number"), retries);
 
+  std::list<std::string> locations;
+  options.AddOption('L', "location",
+                    istring("physical location to write to when destination is an indexing service."
+                            " Must be specified for indexing services which do not automatically"
+                            " generate physical locations. Can be specified multiple times -"
+                            " locations will be tried in order until one succeeds."),
+                    istring("URL"), locations);
+
+  bool show_plugins = false;
+  options.AddOption('P', "listplugins",
+                    istring("list the available plugins (protocols supported)"),
+                    show_plugins);
+
   int timeout = 20;
   options.AddOption('t', "timeout", istring("timeout in seconds (default 20)"),
                     istring("seconds"), timeout);
@@ -585,6 +611,23 @@ int main(int argc, char **argv) {
   if (!debug.empty())
     Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(debug));
 
+  if (show_plugins) {
+    std::list<Arc::ModuleDesc> modules;
+    Arc::PluginsFactory pf(Arc::BaseConfig().MakeConfig(Arc::Config()).Parent());
+    pf.scan(Arc::FinderLoader::GetLibrariesList(), modules);
+    Arc::PluginsFactory::FilterByKind("HED:DMC", modules);
+
+    std::cout << Arc::IString("Protocol plugins available:") << std::endl;
+    for (std::list<Arc::ModuleDesc>::iterator itMod = modules.begin();
+         itMod != modules.end(); itMod++) {
+      for (std::list<Arc::PluginDesc>::iterator itPlug = itMod->plugins.begin();
+           itPlug != itMod->plugins.end(); itPlug++) {
+        std::cout << "  " << itPlug->name << " - " << itPlug->description << std::endl;
+      }
+    }
+    return 0;
+  }
+
   // credentials will be initialised later if necessary
   Arc::UserConfig usercfg(conffile, Arc::initializeCredentialsType::SkipCredentials);
   if (!usercfg) {
@@ -617,10 +660,10 @@ int main(int argc, char **argv) {
   if (source == "-") source = "stdio:///stdin";
   if (destination == "-") destination = "stdio:///stdout";
   if (nocopy) {
-    if(!arcregister(source, destination, usercfg, secure, passive, force, timeout))
+    if(!arcregister(source, destination, locations, usercfg, secure, passive, force, timeout))
       return 1;
   } else {
-    if(!arccp(source, destination, cache_path, usercfg, secure, passive, force,
+    if(!arccp(source, destination, locations, cache_path, usercfg, secure, passive, force,
           recursion, retries + 1, verbose, timeout))
       return 1;
   }
