@@ -19,6 +19,9 @@
 
 namespace Arc {
 
+  // Characters to be escaped in LDAP filter according to RFC4515
+  static const std::string filter_esc("&|=!><~*/()");
+
   class ThreadArgARC0 {
   public:
     TargetGenerator *mom;
@@ -104,8 +107,8 @@ namespace Arc {
       }
     }
 
-    if (serviceType == COMPUTING && mom.AddService(flavour, url) ||
-        serviceType == INDEX     && mom.AddIndexServer(flavour, url)) {
+    if ((serviceType == COMPUTING && mom.AddService(flavour, url)) ||
+        (serviceType == INDEX     && mom.AddIndexServer(flavour, url))) {
       ThreadArgARC0 *arg = CreateThreadArg(mom, true);
       if (!CreateThreadFunction((serviceType == COMPUTING ?
                                  &InterrogateTarget : &QueryIndex),
@@ -133,8 +136,8 @@ namespace Arc {
       }
     }
 
-    if (serviceType == COMPUTING && mom.AddService(flavour, url) ||
-        serviceType == INDEX     && mom.AddIndexServer(flavour, url)) {
+    if ((serviceType == COMPUTING && mom.AddService(flavour, url)) ||
+        (serviceType == INDEX     && mom.AddIndexServer(flavour, url))) {
       ThreadArgARC0 *arg = CreateThreadArg(mom, false);
       if (!CreateThreadFunction((serviceType == COMPUTING ?
                                  &InterrogateTarget : &QueryIndex),
@@ -235,12 +238,14 @@ namespace Arc {
     TargetGenerator& mom = *thrarg->mom;
     const UserConfig& usercfg = *thrarg->usercfg;
 
+    std::string emptycadir;
+    std::string emptycafile;
     //Create credential object in order to get the user DN
     Credential credential(!usercfg.ProxyPath().empty() ? usercfg.ProxyPath() :
                                                          usercfg.CertificatePath(),
                           !usercfg.ProxyPath().empty() ? usercfg.ProxyPath() :
                                                          usercfg.KeyPath(),
-                          usercfg.CACertificatesDirectory(), "");
+                          emptycadir, emptycafile);
 
     //Query GRIS for all relevant information
     URL url = thrarg->url;
@@ -250,10 +255,10 @@ namespace Arc {
       url.ChangeLDAPFilter("(|(objectclass=nordugrid-cluster)"
                            "(objectclass=nordugrid-queue)"
                            "(nordugrid-authuser-sn=" +
-                           credential.GetIdentityName() + "))");
+                           escape_chars(credential.GetIdentityName(),filter_esc,'\\',false,escape_hex) + "))");
     else
       url.ChangeLDAPFilter("(|(nordugrid-job-globalowner=" +
-                           credential.GetIdentityName() + "))");
+                           escape_chars(credential.GetIdentityName(),filter_esc,'\\',false,escape_hex) + "))");
 
     DataHandle handler(url, usercfg);
     DataBuffer buffer;
@@ -611,7 +616,7 @@ namespace Arc {
 
         URL infoEndpoint(url);
         infoEndpoint.ChangeLDAPFilter("(nordugrid-job-globalid=" +
-                                      (std::string)(*it)["nordugrid-job-globalid"] + ")");
+                                      escape_chars((std::string)(*it)["nordugrid-job-globalid"],filter_esc,'\\',false,escape_hex) + ")");
         infoEndpoint.ChangeLDAPScope(URL::subtree);
         j.InfoEndpoint = infoEndpoint;
 
