@@ -22,7 +22,7 @@ namespace Arc {
 
     virtual void GetJobInformation();
     virtual bool GetJob(const Job& job, const std::string& downloaddir,
-                        const bool usejobname, const bool force);
+                        bool usejobname, bool force);
     virtual bool CleanJob(const Job& job);
     virtual bool CancelJob(const Job& job);
     virtual bool RenewJob(const Job& job);
