@@ -96,8 +96,8 @@ namespace Arc {
       }
     }
 
-    if (serviceType == COMPUTING && mom.AddService(flavour, url) ||
-        serviceType == INDEX     && mom.AddIndexServer(flavour, url)) {
+    if ((serviceType == COMPUTING && mom.AddService(flavour, url)) ||
+        (serviceType == INDEX     && mom.AddIndexServer(flavour, url))) {
       ThreadArgCREAM *arg = CreateThreadArg(mom, true);
       if (!CreateThreadFunction((serviceType == COMPUTING ?
                                  &InterrogateTarget : &QueryIndex),
@@ -126,8 +126,8 @@ namespace Arc {
       }
     }
 
-    if (serviceType == COMPUTING && mom.AddService(flavour, url) ||
-        serviceType == INDEX     && mom.AddIndexServer(flavour, url)) {
+    if ((serviceType == COMPUTING && mom.AddService(flavour, url)) ||
+        (serviceType == INDEX     && mom.AddIndexServer(flavour, url))) {
       ThreadArgCREAM *arg = CreateThreadArg(mom, false);
       if (!CreateThreadFunction((serviceType == COMPUTING ?
                                  &InterrogateTarget : &QueryIndex),
@@ -453,6 +453,8 @@ namespace Arc {
         target.ManagerProductName = (std::string)CE["GlueCEInfoLRMSType"];
       if (CE["GlueCEInfoLRMSVersion"])
         target.ManagerProductVersion = (std::string)CE["GlueCEInfoLRMSVersion"];
+      if (CE["GlueCEInfoJobManager"])
+        target.MappingQueue = (std::string)CE["GlueCEInfoJobManager"];
       if (Cluster["GlueClusterName"]) {
         target.ServiceName = (std::string)Cluster["GlueClusterName"];
       }
