@@ -8,7 +8,6 @@
 #include <map>
 
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
 
 #include <arc/StringConv.h>
@@ -238,7 +237,9 @@ namespace Arc {
         }
       }
 
-      jobdescs.back().OtherAttributes["nordugrid:xrsl;clientxrsl"] = source;
+      std::stringstream ss;
+      ss << **it;
+      jobdescs.back().OtherAttributes["nordugrid:xrsl;clientxrsl"] = ss.str();
     }
 
     if(jobdescs.empty()) {
@@ -541,9 +542,21 @@ namespace Arc {
         for (std::list<std::list<std::string> >::iterator it = ll.begin();
              it != ll.end(); it++) {
           std::list<std::string>::iterator it2 = it->begin();
+
+          // Check whether the first string exists in the list
+          if (it2 == it->end()) {
+            return false;
+          }
+
           FileType file;
           file.Name = *it2++;
-          long fileSize;
+
+          // Check whether the second string exists in the list
+          if (it2 == it->end()) {
+            return false;
+          }
+
+          long fileSize = -1;
           // The second string in the list (it2) might either be a URL or file size.
           if (!it2->empty() && !stringto(*it2, fileSize)) {
             URL turl(*it2);
@@ -580,6 +593,7 @@ namespace Arc {
             file.Source.push_back(turl);
           }
           else {
+            if (fileSize != -1) file.FileSize = fileSize;
             file.Source.push_back(URL(file.Name));
           }
           file.KeepData = false;
@@ -635,8 +649,20 @@ namespace Arc {
         for (std::list<std::list<std::string> >::iterator it = ll.begin();
              it != ll.end(); it++) {
           std::list<std::string>::iterator it2 = it->begin();
+
+          // Check whether the first string exists in the list
+          if (it2 == it->end()) {
+            return false;
+          }
+
           FileType file;
           file.Name = *it2++;
+          
+          // Check whether the second string exists in the list
+          if (it2 == it->end()) {
+            return false;
+          }
+
           URL turl(*it2);
           // The second string in the list (it2) might be a URL or empty
           if (!it2->empty() && turl.Protocol() != "file") {
@@ -853,14 +879,9 @@ namespace Arc {
         if (!SingleValue(c, runtime))
           return false;
 
-        /* Since OR expressions in XRSL is spilt into several alternative
-         * JobDescriptions the requirement for RTE must be all (AND).
-         */
-        j.Resources.RunTimeEnvironment.setRequirement(true);
         j.Resources.RunTimeEnvironment.add(Software(runtime), convertOperator(c->Op()));
         for (std::list<JobDescription>::iterator it = j.GetAlternatives().begin();
              it != j.GetAlternatives().end(); it++) {
-          it->Resources.RunTimeEnvironment.setRequirement(true);
           it->Resources.RunTimeEnvironment.add(Software(runtime), convertOperator(c->Op()));
         }
 
@@ -1355,7 +1376,6 @@ namespace Arc {
     }
 
     if (!j.Files.empty() || !j.Application.Executable.Name.empty() || !j.Application.Input.empty()) {
-      struct stat fileStat;
       RSLList *l = NULL;
       for (std::list<FileType>::const_iterator it = j.Files.begin();
            it != j.Files.end(); it++) {
@@ -1364,16 +1384,12 @@ namespace Arc {
         }
         RSLList *s = new RSLList;
         s->Add(new RSLLiteral(it->Name));
-        if (it->Source.front().Protocol() == "file") {
-          if (stat(it->Source.front().Path().c_str(), &fileStat) == 0)
-            s->Add(new RSLLiteral(tostring(fileStat.st_size)));
-          else {
-            logger.msg(ERROR, "Cannot stat local input file %s", it->Source.front().Path());
-            delete s;
-            if (l)
-              delete l;
-            return "";
+        if (it->Source.front().Protocol() == "file" && it->FileSize != -1) {
+          std::string fsizechecksum = tostring(it->FileSize);
+          if (!it->Checksum.empty()) {
+            fsizechecksum += "."+it->Checksum;
           }
+          s->Add(new RSLLiteral(fsizechecksum));
         }
         else
           s->Add(new RSLLiteral(it->Source.front().fullstr()));
