@@ -357,7 +357,7 @@ void ARCJSDLParserTest::TestFilesUploadUpload() {
 
 void ARCJSDLParserTest::TestURIOptionsInput() {
   std::string jsdl = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
   "<JobDescription>"
     "<Application>"
       "<posix-jsdl:POSIXApplication>"
@@ -386,7 +386,7 @@ void ARCJSDLParserTest::TestURIOptionsInput() {
 
 void ARCJSDLParserTest::TestURIOptionsOutput() {
   std::string jsdl = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
   "<JobDescription>"
     "<Application>"
       "<posix-jsdl:POSIXApplication>"
@@ -433,7 +433,7 @@ void ARCJSDLParserTest::TestURIOptionsOutput() {
 
 void ARCJSDLParserTest::TestQueue() {
   std::string jsdl = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
 "<JobDescription>"
 "<Application>"
 "<posix-jsdl:POSIXApplication>"
@@ -460,7 +460,7 @@ void ARCJSDLParserTest::TestQueue() {
   CPPUNIT_ASSERT_EQUAL(0, (int)OUTJOBS.front().GetAlternatives().size());
 
   jsdl = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
 "<JobDescription>"
 "<Application>"
 "<posix-jsdl:POSIXApplication>"
@@ -494,7 +494,7 @@ void ARCJSDLParserTest::TestQueue() {
 
 void ARCJSDLParserTest::TestDryRun() {
   std::string jsdl = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
 "<JobDescription>"
 "<Application>"
 "<Executable>"
@@ -519,7 +519,7 @@ void ARCJSDLParserTest::TestDryRun() {
   CPPUNIT_ASSERT_EQUAL(0, (int)OUTJOBS.front().GetAlternatives().size());
 
   jsdl = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
 "<JobDescription>"
 "<Application>"
 "<Executable>"
@@ -550,7 +550,7 @@ void ARCJSDLParserTest::TestPOSIXCompliance() {
   MESSAGE = "Error: The parser does not comply with the JDSL POSIX specification.";
 
   const std::string posixJSDLStr = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
 "<JobDescription>"
 "<Application>"
 "<posix-jsdl:POSIXApplication>"
@@ -565,7 +565,7 @@ void ARCJSDLParserTest::TestPOSIXCompliance() {
 "<posix-jsdl:Environment name=\"var2\">value2</posix-jsdl:Environment>"
 "<posix-jsdl:Environment name=\"var3\">value3</posix-jsdl:Environment>"
 "<posix-jsdl:WallTimeLimit>50</posix-jsdl:WallTimeLimit>"
-"<posix-jsdl:MemoryLimit>100</posix-jsdl:MemoryLimit>"
+"<posix-jsdl:MemoryLimit>104857600</posix-jsdl:MemoryLimit>"
 "<posix-jsdl:CPUTimeLimit>110</posix-jsdl:CPUTimeLimit>"
 "<posix-jsdl:ProcessCountLimit>2</posix-jsdl:ProcessCountLimit>"
 "<posix-jsdl:VirtualMemoryLimit>500</posix-jsdl:VirtualMemoryLimit>"
@@ -630,7 +630,7 @@ void ARCJSDLParserTest::TestPOSIXCompliance() {
   CPPUNIT_ASSERT_EQUAL_MESSAGE("POSIX compliance failure", (std::string)"value3", (std::string)pApp["Environment"][2]);
 
   CPPUNIT_ASSERT_EQUAL_MESSAGE("POSIX compliance failure", INJOB.Resources.TotalWallTime.range.max, Arc::stringto<int>(pApp["WallTimeLimit"]));
-  CPPUNIT_ASSERT_EQUAL_MESSAGE("POSIX compliance failure", INJOB.Resources.IndividualPhysicalMemory.max, Arc::stringto<int>(pApp["MemoryLimit"]));
+  CPPUNIT_ASSERT_EQUAL_MESSAGE("POSIX compliance failure", INJOB.Resources.IndividualPhysicalMemory.max, (int)(Arc::stringto<long long>(pApp["MemoryLimit"])/(1024*1024)));
   CPPUNIT_ASSERT_EQUAL_MESSAGE("POSIX compliance failure", INJOB.Resources.TotalCPUTime.range.max, Arc::stringto<int>(pApp["CPUTimeLimit"]));
   CPPUNIT_ASSERT_EQUAL_MESSAGE("POSIX compliance failure", INJOB.Resources.SlotRequirement.NumberOfSlots.max, Arc::stringto<int>(pApp["ProcessCountLimit"]));
   CPPUNIT_ASSERT_EQUAL_MESSAGE("POSIX compliance failure", INJOB.Resources.IndividualVirtualMemory.max, Arc::stringto<int>(pApp["VirtualMemoryLimit"]));
@@ -643,7 +643,7 @@ void ARCJSDLParserTest::TestHPCCompliance() {
   MESSAGE = "Error: The parser does not comply with the JSDL HPC Profile Application Extension specification.";
 
   const std::string hpcJSDLStr = "<?xml version=\"1.0\"?>"
-"<JobDefinition>"
+"<JobDefinition xmlns=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\">"
 "<JobDescription>"
 "<Application>"
 "<HPC-jsdl:HPCProfileApplication>"
