@@ -22,10 +22,6 @@
 #include <arc/message/MCC.h>
 #include <arc/message/PayloadRaw.h>
 
-#ifdef WIN32
-#include <arc/win32.h>
-#endif
-
 #include "DataPointARC.h"
 
 namespace Arc {
@@ -58,7 +54,7 @@ namespace Arc {
     try{
       status = client.process(&request, &response);
     }
-    catch(std::exception e){
+    catch(std::exception& e){
       logger.msg(WARNING, bartender_url.Path(), "did not work");
     }
 
@@ -649,6 +645,6 @@ namespace Arc {
 } // namespace Arc
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-  { "arc", "HED:DMC", 0, &Arc::DataPointARC::Instance },
-  { NULL, NULL, 0, NULL }
+  { "arc", "HED:DMC", "Chelonia Storage Element", 0, &Arc::DataPointARC::Instance },
+  { NULL, NULL, NULL, 0, NULL }
 };
