@@ -9,10 +9,16 @@
 #include <arc/Thread.h>
 #include <arc/data/DataPointDirect.h>
 #include <arc/data/DataHandle.h>
-#include <arc/data/CheckSum.h>
+#include <arc/CheckSum.h>
 
 namespace Arc {
 
+  /**
+   * Provides an interface to the Chelonia storage system developed by ARC.
+   *
+   * This class is a loadable module and cannot be used directly. The DataHandle
+   * class loads modules at runtime and should be used instead of this.
+   */
   class DataPointARC
     : public DataPointDirect {
   private:
