@@ -46,6 +46,7 @@ namespace Arc {
     // Report chunk transferred. It may be _different_
     // from one obtained through Get().
     void Claim(uint64_t start, uint64_t length);
+    void Claim(uint64_t length);
     // Report chunk not transferred. It must be
     // _same_ as one obtained by Get().
     void Unclaim(uint64_t start, uint64_t length);
@@ -122,9 +123,9 @@ namespace Arc {
     }
   };
 
-  ChunkControl::ChunkControl(uint64_t /* size */) {
+  ChunkControl::ChunkControl(uint64_t size) {
     chunk_t chunk = {
-      0, UINT64_MAX
+      0, size
     };
     chunks_.push_back(chunk);
   }
@@ -132,8 +133,7 @@ namespace Arc {
   ChunkControl::~ChunkControl() {}
 
   bool ChunkControl::Get(uint64_t& start, uint64_t& length) {
-    if (length == 0)
-      return false;
+    if (length == 0) return false;
     lock_.lock();
     std::list<chunk_t>::iterator c = chunks_.begin();
     if (c == chunks_.end()) {
@@ -145,28 +145,25 @@ namespace Arc {
     if (l <= length) {
       length = l;
       chunks_.erase(c);
-    }
-    else
+    } else {
       c->start += length;
+    }
     lock_.unlock();
     return true;
   }
 
   void ChunkControl::Claim(uint64_t start, uint64_t length) {
-    if (length == 0)
-      return;
+    if (length == 0) return;
     uint64_t end = start + length;
     lock_.lock();
     for (std::list<chunk_t>::iterator c = chunks_.begin();
          c != chunks_.end();) {
-      if (end <= c->start)
-        break;
+      if (end <= c->start) break;
       if ((start <= c->start) && (end >= c->end)) {
         start = c->end;
         length = end - start;
         c = chunks_.erase(c);
-        if (length > 0)
-          continue;
+        if (length > 0) continue;
         break;
       }
       if ((start > c->start) && (end < c->end)) {
@@ -187,8 +184,7 @@ namespace Arc {
         start = start_;
         length = end - start;
         ++c;
-        if (length > 0)
-          continue;
+        if (length > 0) continue;
         break;
       }
       ++c;
@@ -196,16 +192,18 @@ namespace Arc {
     lock_.unlock();
   }
 
+  void ChunkControl::Claim(uint64_t start) {
+    Claim(start, UINT64_MAX - start);
+  }
+
   void ChunkControl::Unclaim(uint64_t start, uint64_t length) {
-    if (length == 0)
-      return;
+    if (length == 0) return;
     uint64_t end = start + length;
     lock_.lock();
     for (std::list<chunk_t>::iterator c = chunks_.begin();
          c != chunks_.end(); ++c) {
       if ((end >= c->start) && (end <= c->end)) {
-        if (start < c->start)
-          c->start = start;
+        if (start < c->start) c->start = start;
         lock_.unlock();
         return;
       }
@@ -222,8 +220,7 @@ namespace Arc {
               }
               c_ = chunks_.erase(c_);
             }
-            else
-              break;
+            else break;
         }
         lock_.unlock();
         return;
@@ -242,8 +239,7 @@ namespace Arc {
               }
               c_ = chunks_.erase(c_);
             }
-            else
-              break;
+            else break;
         }
         lock_.unlock();
         return;
@@ -257,6 +253,10 @@ namespace Arc {
         return;
       }
     }
+    chunk_t chunk = {
+      start, end
+    };
+    chunks_.push_back(chunk);
     lock_.unlock();
   }
 
@@ -505,12 +505,10 @@ namespace Arc {
   }
 
   DataStatus DataPointHTTP::StartReading(DataBuffer& buffer) {
-    if (transfers_started.get() != 0)
-      return DataStatus::ReadStartError;
+    if (transfers_started.get() != 0) return DataStatus::ReadStartError;
     int transfer_streams = 1;
     DataPointHTTP::buffer = &buffer;
-    if (chunks)
-      delete chunks;
+    if (chunks) delete chunks;
     chunks = new ChunkControl;
     MCCConfig cfg;
     usercfg.ApplyToConfig(cfg);
@@ -639,14 +637,18 @@ namespace Arc {
       unsigned int transfer_size = 0;
       int transfer_handle = -1;
       // get first buffer
-      if (!point.buffer->for_read(transfer_handle, transfer_size, true))
+      if (!point.buffer->for_read(transfer_handle, transfer_size, true)) {
         // No transfer buffer - must be failure or close initiated externally
         break;
+      }
       uint64_t transfer_offset = 0;
-      uint64_t chunk_length = transfer_size;
-      if (!(point.chunks->Get(transfer_offset, chunk_length)))
+      uint64_t chunk_length = 1024*1024;
+      if(transfer_size > chunk_length) chunk_length = transfer_size;
+      if (!(point.chunks->Get(transfer_offset, chunk_length))) {
         // No more chunks to transfer - quit this thread.
+        point.buffer->is_read(transfer_handle, 0, 0);
         break;
+      }
       uint64_t transfer_end = transfer_offset + chunk_length - 1;
       // Read chunk
       HTTPClientInfo transfer_info;
@@ -656,6 +658,10 @@ namespace Arc {
                                      transfer_end, &request, &transfer_info,
                                      &inbuf);
       if (!r) {
+        // Return buffer
+        point.buffer->is_read(transfer_handle, 0, 0);
+        point.chunks->Unclaim(transfer_offset, chunk_length);
+        if (inbuf) delete inbuf;
         // Failed to transfer chunk - retry.
         // 10 times in a row seems to be reasonable number
         // TODO: mark failure?
@@ -664,10 +670,6 @@ namespace Arc {
           transfer_failure = true;
           break;
         }
-        // Return buffer
-        point.buffer->is_read(transfer_handle, 0, 0);
-        point.chunks->Unclaim(transfer_offset, chunk_length);
-        if (inbuf) delete inbuf;
         // Recreate connection
         delete client;
         client = NULL;
@@ -679,9 +681,7 @@ namespace Arc {
       if (transfer_info.code == 416) { // EOF
         point.buffer->is_read(transfer_handle, 0, 0);
         point.chunks->Unclaim(transfer_offset, chunk_length);
-        if (inbuf)
-          delete inbuf;
-        // TODO: report file size to chunk control
+        if (inbuf) delete inbuf;
         break;
       }
       if((transfer_info.code == 301) || // permanent redirection
@@ -725,16 +725,52 @@ namespace Arc {
         transfer_failure = true;
         break;
       }
+      PayloadStreamInterface* instream = NULL;
+      try {
+        instream = dynamic_cast<PayloadStreamInterface*>(dynamic_cast<MessagePayload*>(inbuf));
+      } catch(std::exception& e) {
+        transfer_failure = true;
+        break;
+      }
+      if(!instream) {
+        transfer_failure = true;
+        break;
+      }
       // pick up usefull information from HTTP header
       point.created = transfer_info.lastModified;
       retries = 0;
+      // Exclude chunks after EOF. Normally that is not needed.
+      // But Apache if asked about out of file range gets confused
+      // and sends *whole* file instead of 416.
+      if(inbuf && (inbuf->Size() > 0)) point.chunks->Claim(inbuf->Size());
       bool whole = (inbuf && (((transfer_info.size == inbuf->Size() &&
                                (inbuf->BufferPos(0) == 0))) ||
                     inbuf->Size() == -1));
-      // Temporary solution - copy data between buffers
       point.transfer_lock.lock();
       point.chunks->Unclaim(transfer_offset, chunk_length);
       uint64_t transfer_pos = 0;
+      for(;;) {
+        if (transfer_handle == -1) {
+          point.transfer_lock.unlock();
+          if (!point.buffer->for_read(transfer_handle, transfer_size, true)) {
+            // No transfer buffer - must be failure or close initiated
+            // externally
+            point.transfer_lock.lock();
+            break;
+          }
+          point.transfer_lock.lock();
+        }
+        int l = transfer_size;
+        uint64_t pos = instream->Pos();
+        if(!instream->Get((*point.buffer)[transfer_handle],l)) {
+          break;
+        }
+        point.buffer->is_read(transfer_handle, l, pos);
+        point.chunks->Claim(pos, l);
+        transfer_handle = -1;
+        transfer_pos = pos + l;
+      }
+      /*
       for (unsigned int n = 0;; ++n) {
         if (!inbuf) break;
         char *buf = inbuf->Buffer(n);
@@ -759,8 +795,7 @@ namespace Arc {
             point.transfer_lock.lock();
           }
           unsigned int l = length;
-          if (l > transfer_size)
-            l = transfer_size;
+          if (l > transfer_size) l = transfer_size;
           char *buf_ = (*point.buffer)[transfer_handle];
           memcpy(buf_, buf, l);
           point.buffer->is_read(transfer_handle, l, pos);
@@ -771,17 +806,14 @@ namespace Arc {
           transfer_handle = -1;
         }
       }
-      if (transfer_handle != -1)
-        point.buffer->is_read(transfer_handle, 0, 0);
-      if (inbuf)
-        delete inbuf;
+      */
+      if (transfer_handle != -1) point.buffer->is_read(transfer_handle, 0, 0);
+      if (inbuf) delete inbuf;
       // If server returned chunk which is not overlaping requested one - seems
       // like server has nothing to say any more.
-      if (transfer_pos <= transfer_offset)
-        whole = true;
+      if (transfer_pos <= transfer_offset) whole = true;
       point.transfer_lock.unlock();
-      if (whole)
-        break;
+      if (whole) break;
     }
     point.transfer_lock.lock();
     --(point.transfers_tofinish);
