@@ -11,6 +11,14 @@ namespace Arc {
   class Logger;
   class URL;
 
+  /**
+   * The LCG File Catalog (LFC) is a replica catalog developed by CERN. It
+   * consists of a hierarchical namespace of grid files and each filename
+   * can be associated with one or more physical locations.
+   *
+   * This class is a loadable module and cannot be used directly. The DataHandle
+   * class loads modules at runtime and should be used instead of this.
+   */
   class DataPointLFC
     : public DataPointIndex {
   public:
@@ -34,6 +42,8 @@ namespace Arc {
     std::string path_for_guid;
   private:
     std::string ResolveGUIDToLFN();
+    /// Returns true if serrno is a temporary error and can potentially be retried
+    bool IsTempError() const;
     DataStatus ListFiles(std::list<FileInfo>& files, DataPointInfoType verb, bool listdir);
 
   };
