@@ -23,6 +23,29 @@ namespace Arc {
 
   Logger DataPointSRM::logger(Logger::getRootLogger(), "DataPoint.SRM");
 
+  static std::string canonic_srm_url(const URL& url) {
+    std::string sfn_path = url.HTTPOption("SFN");
+    std::string canonic_url;
+    if (!sfn_path.empty()) {
+      while (sfn_path[0] == '/') sfn_path.erase(0,1);
+      canonic_url = url.Protocol() + "://" + url.Host() + "/" + uri_encode(sfn_path, false);
+    } else {
+      // if SFN option is not used, treat everything in the path including
+      // options as part of the path and encode it
+      canonic_url = url.Protocol() + "://" + url.Host() + uri_encode(url.Path(), false);
+      std::string extrapath;
+      for (std::map<std::string, std::string>::const_iterator
+           it = url.HTTPOptions().begin(); it != url.HTTPOptions().end(); it++) {
+        if (it == url.HTTPOptions().begin()) extrapath += '?';
+        else extrapath += '&';
+        extrapath += it->first;
+        if (!it->second.empty()) extrapath += '=' + it->second;
+      }
+      canonic_url += uri_encode(extrapath, false);
+    }
+    return canonic_url;
+  }
+
   DataPointSRM::DataPointSRM(const URL& url, const UserConfig& usercfg)
     : DataPointDirect(url, usercfg),
       srm_request(NULL),
@@ -58,11 +81,7 @@ namespace Arc {
       return DataStatus::CheckError;
     }
 
-    std::string canonic_url;
-    if (!url.HTTPOption("SFN").empty())
-      canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
-    else
-      canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
+    std::string canonic_url = canonic_srm_url(url);
 
     SRMClientRequest srm_request_tmp(canonic_url);
     
@@ -125,11 +144,7 @@ namespace Arc {
     }
 
     // take out options in srm url and encode path
-    std::string canonic_url;
-    if (!url.HTTPOption("SFN").empty())
-      canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
-    else
-      canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
+    std::string canonic_url = canonic_srm_url(url);
 
     SRMClientRequest srm_request_tmp(canonic_url);
 
@@ -191,11 +206,7 @@ namespace Arc {
         }
 
         // take out options in srm url and encode path
-        std::string canonic_url;
-        if (!url.HTTPOption("SFN").empty())
-          canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
-        else
-          canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
+        std::string canonic_url = canonic_srm_url(url);
 
         delete srm_request;
         srm_request = new SRMClientRequest(canonic_url);
@@ -253,11 +264,7 @@ namespace Arc {
       }
 
       // take out options in srm url and encode path
-      std::string canonic_url;
-      if (!url.HTTPOption("SFN").empty())
-        canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
-      else
-        canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
+      std::string canonic_url = canonic_srm_url(url);
 
       delete srm_request;
 
@@ -426,11 +433,7 @@ namespace Arc {
       }
 
       // take out options in srm url and encode path
-      std::string canonic_url;
-      if (!url.HTTPOption("SFN").empty())
-        canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
-      else
-        canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
+      std::string canonic_url = canonic_srm_url(url);
 
       delete srm_request;
 
@@ -458,20 +461,25 @@ namespace Arc {
         else {
           logger.msg(VERBOSE, "Using space token description %s", space_token);
           // get token from SRM that matches description
+          // errors with space tokens now cause the transfer to fail - see bug 2061
           std::list<std::string> tokens;
           if (client->getSpaceTokens(tokens, space_token) != SRM_OK) {
-            // not critical so log a warning
-            logger.msg(WARNING, "Warning: Error looking up space tokens matching description %s. Will copy without using token", space_token);
+            logger.msg(ERROR, "Error looking up space tokens matching description %s", space_token);
+            delete client;
+            delete srm_request;
+            srm_request = NULL;
+            return DataStatus::WritePrepareError;
           }
-          else if (tokens.empty()) {
-            // not critical so log a warning
-            logger.msg(WARNING, "Warning: No space tokens found matching description! Will copy without using token");
-          }
-          else {
-            // take the first one in the list
-            logger.msg(VERBOSE, "Using space token %s", tokens.front());
-            srm_request->space_token(tokens.front());
+          if (tokens.empty()) {
+            logger.msg(ERROR, "No space tokens found matching description %s", space_token);
+            delete client;
+            delete srm_request;
+            srm_request = NULL;
+            return DataStatus::WritePrepareError;
           }
+          // take the first one in the list
+          logger.msg(VERBOSE, "Using space token %s", tokens.front());
+          srm_request->space_token(tokens.front());
         }
       }
       srm_request->request_timeout(stage_timeout);
@@ -587,6 +595,9 @@ namespace Arc {
         // call abort if failure, or releasePut on success
         if (error || srm_request->status() == SRM_REQUEST_SHOULD_ABORT) {
            client->abort(*srm_request);
+           // according to the spec the SURL may or may not exist after abort
+           // so silence error messages from trying to delete
+           srm_request->error_loglevel(VERBOSE);
            client->remove(*srm_request);
         }
         else {
@@ -640,6 +651,9 @@ namespace Arc {
               }
           } else {
             client->abort(*srm_request);
+            // according to the spec the SURL may or may not exist after abort
+            // so silence error messages from trying to delete
+            srm_request->error_loglevel(VERBOSE);
             client->remove(*srm_request);
           }
         }
@@ -677,14 +691,7 @@ namespace Arc {
       return DataStatus::ListError;
     }
     
-    std::string canonic_url;
-    std::string sfn_path = url.HTTPOption("SFN");
-    if (!sfn_path.empty()) {
-      while (sfn_path[0] == '/') sfn_path.erase(0,1);
-      canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(sfn_path, false);
-    } else {
-      canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
-    }
+    std::string canonic_url = canonic_srm_url(url);
 
     SRMClientRequest srm_request_tmp(canonic_url);
     logger.msg(VERBOSE, "ListFiles: looking for metadata: %s", CurrentLocation().str());
@@ -826,7 +833,7 @@ namespace Arc {
       transport_protocols.push_back("httpg");
       transport_protocols.push_back("ftp");
     } else {
-      Arc::tokenize(option_protocols, transport_protocols, ",");
+      tokenize(option_protocols, transport_protocols, ",");
     }
   }
 
