@@ -148,7 +148,7 @@ namespace Arc {
     request.NewChild("SRMv2:srmPing").NewChild("srmPingRequest");
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -190,7 +190,7 @@ namespace Arc {
       req.NewChild("userSpaceTokenDescription") = description;
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -225,7 +225,7 @@ namespace Arc {
       req.NewChild("userRequestDescription") = description;
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -273,7 +273,7 @@ namespace Arc {
       protocols.NewChild("stringArray") = *prot;
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK) {
       creq.finished_error();
       return status;
@@ -336,9 +336,12 @@ namespace Arc {
     else if (statuscode != SRM_SUCCESS) {
       // any other return code is a failure
       logger.msg(ERROR, explanation);
+      if (res["arrayOfFileStatuses"]["statusArray"]["status"]["explanation"])
+        logger.msg(ERROR, res["arrayOfFileStatuses"]["statusArray"]["status"]["explanation"]);
+      SRMStatusCode file_statuscode = GetStatus(res["arrayOfFileStatuses"]["statusArray"]["status"], explanation);
       creq.finished_error();
       delete response;
-      if (statuscode == SRM_INTERNAL_ERROR)
+      if (statuscode == SRM_INTERNAL_ERROR || file_statuscode == SRM_FILE_UNAVAILABLE || file_statuscode == SRM_FILE_BUSY)
         return SRM_ERROR_TEMPORARY;
       return SRM_ERROR_PERMANENT;
     }
@@ -363,7 +366,7 @@ namespace Arc {
     req.NewChild("requestToken") = creq.request_token();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK) {
       creq.finished_abort();
       return status;
@@ -387,9 +390,12 @@ namespace Arc {
     else if (statuscode != SRM_SUCCESS) {
       // error
       logger.msg(ERROR, explanation);
+      if (res["arrayOfFileStatuses"]["statusArray"]["status"]["explanation"])
+        logger.msg(ERROR, res["arrayOfFileStatuses"]["statusArray"]["status"]["explanation"]);
+      SRMStatusCode file_statuscode = GetStatus(res["arrayOfFileStatuses"]["statusArray"]["status"], explanation);
       creq.finished_error();
       delete response;
-      if (statuscode == SRM_INTERNAL_ERROR)
+      if (statuscode == SRM_INTERNAL_ERROR || file_statuscode == SRM_FILE_UNAVAILABLE || file_statuscode == SRM_FILE_BUSY)
         return SRM_ERROR_TEMPORARY;
       return SRM_ERROR_PERMANENT;
     }
@@ -434,7 +440,7 @@ namespace Arc {
     }
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK) {
       creq.finished_error();
       return status;
@@ -538,7 +544,7 @@ namespace Arc {
     req.NewChild("requestToken") = creq.request_token();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK) {
       creq.finished_abort();
       return status;
@@ -678,7 +684,7 @@ namespace Arc {
       req.NewChild("targetSpaceToken") = creq.space_token();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK) {
       creq.finished_error();
       return status;
@@ -790,7 +796,7 @@ namespace Arc {
     req.NewChild("requestToken") = creq.request_token();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK) {
       creq.finished_abort();
       return status;
@@ -832,6 +838,9 @@ namespace Arc {
         creq.finished_error();
         return mkdirres;
       }
+      if (res["arrayOfFileStatuses"]["statusArray"]["status"])
+        logger.msg(ERROR, statusexplanation);
+
       logger.msg(ERROR, explanation);
       creq.finished_error();
       delete response;
@@ -854,15 +863,13 @@ namespace Arc {
 
   SRMReturnCode SRM22Client::info(SRMClientRequest& creq,
                                   std::list<struct SRMFileMetaData>& metadata,
-                                  const int recursive,
-                                  bool report_error) {
-    return info(creq, metadata, recursive, report_error, 0, 0);
+                                  const int recursive) {
+    return info(creq, metadata, recursive, 0, 0);
   }
 
   SRMReturnCode SRM22Client::info(SRMClientRequest& creq,
                                   std::list<struct SRMFileMetaData>& metadata,
                                   const int recursive,
-                                  bool report_error,
                                   const int offset,
                                   const int count) {
     // only one SURL requested at a time
@@ -882,7 +889,7 @@ namespace Arc {
       req.NewChild("fullDetailedList") = "true";
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -920,7 +927,7 @@ namespace Arc {
 
         delete response;
         response = NULL;
-        status = process(&request, &response);
+        status = process("", &request, &response);
         if (status != SRM_OK)
           return status;
 
@@ -935,10 +942,10 @@ namespace Arc {
         if (statuscode != SRM_SUCCESS &&
             statuscode != SRM_REQUEST_QUEUED &&
             statuscode != SRM_REQUEST_INPROGRESS) {
-          logger.msg(report_error ? ERROR : VERBOSE, "%s", explanation);
+          logger.msg(creq.error_loglevel(), "%s", explanation);
           // check if individual file status gives more info
           if (res["details"]["pathDetailArray"]["status"]["explanation"])
-            logger.msg(report_error ? ERROR : VERBOSE, "%s",
+            logger.msg(creq.error_loglevel(), "%s",
                        (std::string)res["details"]["pathDetailArray"]
                        ["status"]["explanation"]);
           delete response;
@@ -950,7 +957,7 @@ namespace Arc {
 
       // check for timeout
       if (request_time >= creq.request_timeout()) {
-        logger.msg(ERROR, "Ls request timed out after %i seconds",
+        logger.msg(creq.error_loglevel(), "Ls request timed out after %i seconds",
                    creq.request_timeout());
         abort(creq);
         delete response;
@@ -958,10 +965,10 @@ namespace Arc {
       }
     }
     else {
-      logger.msg(report_error ? ERROR : VERBOSE, "%s", explanation);
+      logger.msg(creq.error_loglevel(), "%s", explanation);
       // check if individual file status gives more info
       if (res["details"]["pathDetailArray"]["status"]["explanation"])
-        logger.msg(report_error ? ERROR : VERBOSE, "%s",
+        logger.msg(creq.error_loglevel(), "%s",
                    (std::string)res["details"]["pathDetailArray"]
                    ["status"]["explanation"]);
       delete response;
@@ -1024,7 +1031,7 @@ namespace Arc {
           list_req.long_list(creq.long_list());
           int list_offset = max_files_list * list_no;
           int list_count = max_files_list;
-          SRMReturnCode res = info(list_req, list_metadata, 0, true,
+          SRMReturnCode res = info(list_req, list_metadata, 0,
                                    list_offset, list_count);
           if (res != SRM_OK) {
             delete response;
@@ -1241,7 +1248,7 @@ namespace Arc {
     req.NewChild("requestToken") = creq.request_token();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -1281,7 +1288,7 @@ namespace Arc {
     req.NewChild("arrayOfSURLs").NewChild("urlArray") = creq.surls().front();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -1318,7 +1325,7 @@ namespace Arc {
     req.NewChild("requestToken") = creq.request_token();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -1347,12 +1354,14 @@ namespace Arc {
 
     // call info() to find out if we are dealing with a file or directory
     SRMClientRequest inforeq(creq.surls());
+    inforeq.error_loglevel(creq.error_loglevel());
     std::list<struct SRMFileMetaData> metadata;
 
     // set recursion to -1, meaning don't list entries in a dir
     SRMReturnCode res = info(inforeq, metadata, -1);
     if (res != SRM_OK) {
-      logger.msg(ERROR, "Failed to find metadata info on file %s",
+      logger.msg(creq.error_loglevel(),
+                 "Failed to find metadata info on %s for determining file or directory delete",
                  inforeq.surls().front());
       return res;
     }
@@ -1380,7 +1389,7 @@ namespace Arc {
     req.NewChild("arrayOfSURLs").NewChild("urlArray") = creq.surls().front();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -1390,7 +1399,7 @@ namespace Arc {
     SRMStatusCode statuscode = GetStatus(res["returnStatus"], explanation);
 
     if (statuscode != SRM_SUCCESS) {
-      logger.msg(ERROR, "%s", explanation);
+      logger.msg(creq.error_loglevel(), "%s", explanation);
       delete response;
       if (statuscode == SRM_INTERNAL_ERROR)
         return SRM_ERROR_TEMPORARY;
@@ -1410,7 +1419,7 @@ namespace Arc {
     req.NewChild("SURL") = creq.surls().front();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -1420,7 +1429,7 @@ namespace Arc {
     SRMStatusCode statuscode = GetStatus(res["returnStatus"], explanation);
 
     if (statuscode != SRM_SUCCESS) {
-      logger.msg(ERROR, "%s", explanation);
+      logger.msg(creq.error_loglevel(), "%s", explanation);
       delete response;
       if (statuscode == SRM_INTERNAL_ERROR)
         return SRM_ERROR_TEMPORARY;
@@ -1445,7 +1454,7 @@ namespace Arc {
       req.NewChild("targetSpaceToken") = creq.space_token();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
@@ -1488,7 +1497,7 @@ namespace Arc {
 
         delete response;
         response = NULL;
-        status = process(&request, &response);
+        status = process("", &request, &response);
         if (status != SRM_OK)
           return status;
 
@@ -1546,10 +1555,12 @@ namespace Arc {
       std::string dirname = surl.substr(0, slashpos);
       // list dir to see if it exists
       SRMClientRequest listreq(dirname);
+      // don't report errors
+      listreq.error_loglevel(VERBOSE);
       std::list<struct SRMFileMetaData> metadata;
       if (keeplisting) {
         logger.msg(VERBOSE, "Checking for existence of %s", dirname);
-        if (info(listreq, metadata, -1, false) == SRM_OK) {
+        if (info(listreq, metadata, -1) == SRM_OK) {
           if (metadata.front().fileType == SRM_FILE) {
             logger.msg(ERROR, "File already exists: %s", dirname);
             return SRM_ERROR_PERMANENT;
@@ -1567,7 +1578,7 @@ namespace Arc {
       req.NewChild("SURL") = dirname;
 
       PayloadSOAP *response = NULL;
-      SRMReturnCode status = process(&request, &response);
+      SRMReturnCode status = process("", &request, &response);
       if (status != SRM_OK)
         return status;
 
@@ -1605,7 +1616,7 @@ namespace Arc {
     req.NewChild("arrayOfSURLs").NewChild("urlArray") = creq.surls().front();
 
     PayloadSOAP *response = NULL;
-    SRMReturnCode status = process(&request, &response);
+    SRMReturnCode status = process("", &request, &response);
     if (status != SRM_OK)
       return status;
 
