@@ -11,6 +11,7 @@
 #include <arc/client/Broker.h>
 #include <arc/client/ExecutionTarget.h>
 #include <arc/client/Job.h>
+#include <arc/credential/Credential.h>
 #include <arc/loader/FinderLoader.h>
 
 namespace Arc {
@@ -32,6 +33,10 @@ namespace Arc {
     PossibleTargets.clear();
     job = &jobdesc;
 
+    Credential credential(usercfg);
+    std::string proxyIssuerCA = credential.GetCAName();
+    logger.msg(DEBUG, "Your issuer CA's DN: %s.", proxyIssuerCA);
+
     for (std::list<ExecutionTarget>::iterator target = targets.begin();
          target != targets.end(); target++) {
       logger.msg(VERBOSE, "Performing matchmaking against target (%s).", target->url.str());
@@ -52,6 +57,12 @@ namespace Arc {
         }
       }
 
+      if ( !(target->TrustedCA.empty()) && (find(target->TrustedCA.begin(), target->TrustedCA.end(), proxyIssuerCA)
+              == target->TrustedCA.end()) ){
+          logger.msg(VERBOSE, "Your issuer CA's DN (%s) is not supported on (%s) target.",proxyIssuerCA, target->url.str());
+          continue;
+      }
+
       std::map<std::string, std::string>::const_iterator itAtt;
       if ((itAtt = job->OtherAttributes.find("nordugrid:broker;reject_queue")) != job->OtherAttributes.end()) {
         if (target->ComputingShareName.empty()) {
@@ -273,67 +284,73 @@ namespace Arc {
           }
         }
 
-      if (job->Resources.DiskSpaceRequirement.SessionDiskSpace != -1) {
-        if (target->MaxDiskSpace != -1) {     // Example: 5656
-          if (target->MaxDiskSpace < job->Resources.DiskSpaceRequirement.SessionDiskSpace) {
-            logger.msg(VERBOSE, "Matchmaking, MaxDiskSpace problem, ExecutionTarget: %d (MaxDiskSpace) JobDescription: %d (SessionDiskSpace)", target->MaxDiskSpace, job->Resources.DiskSpaceRequirement.SessionDiskSpace);
+      if (job->Resources.DiskSpaceRequirement.SessionDiskSpace > -1) {
+        if (target->MaxDiskSpace > -1) {     // Example: 5656
+          if (target->MaxDiskSpace*1024 < job->Resources.DiskSpaceRequirement.SessionDiskSpace) {
+            logger.msg(VERBOSE, "Matchmaking, MaxDiskSpace problem, ExecutionTarget: %d MB (MaxDiskSpace); JobDescription: %d MB (SessionDiskSpace)", target->MaxDiskSpace*1024, job->Resources.DiskSpaceRequirement.SessionDiskSpace);
             continue;
           }
         }
-        else if (target->WorkingAreaTotal != -1) {     // Example: 5656
-          if (target->WorkingAreaTotal < job->Resources.DiskSpaceRequirement.SessionDiskSpace) {
-            logger.msg(VERBOSE, "Matchmaking, WorkingAreaTotal problem, ExecutionTarget: %d (WorkingAreaTotal) JobDescription: %d (SessionDiskSpace)", target->WorkingAreaTotal, job->Resources.DiskSpaceRequirement.SessionDiskSpace);
+
+        if (target->WorkingAreaFree > -1) {     // Example: 5656
+          if (target->WorkingAreaFree*1024 < job->Resources.DiskSpaceRequirement.SessionDiskSpace) {
+            logger.msg(VERBOSE, "Matchmaking, WorkingAreaFree problem, ExecutionTarget: %d MB (WorkingAreaFree); JobDescription: %d MB (SessionDiskSpace)", target->WorkingAreaFree*1024, job->Resources.DiskSpaceRequirement.SessionDiskSpace);
             continue;
           }
         }
-        else {
-          logger.msg(VERBOSE, "Matchmaking, ExecutionTarget:  %s, MaxDiskSpace and WorkingAreaTotal are not defined", target->url.str());
+        
+        if (target->MaxDiskSpace <= -1 && target->WorkingAreaFree <= -1) {
+          logger.msg(VERBOSE, "Matchmaking, ExecutionTarget:  %s, MaxDiskSpace and WorkingAreaFree are not defined", target->url.str());
           continue;
         }
       }
 
-      if (job->Resources.DiskSpaceRequirement.DiskSpace != -1 && job->Resources.DiskSpaceRequirement.CacheDiskSpace != -1) {
-        if (target->MaxDiskSpace != -1) {     // Example: 5656
-          if (target->MaxDiskSpace < job->Resources.DiskSpaceRequirement.DiskSpace - job->Resources.DiskSpaceRequirement.CacheDiskSpace) {
-            logger.msg(VERBOSE, "Matchmaking, MaxDiskSpace >= DiskSpace - CacheDiskSpace problem, ExecutionTarget: %d (MaxDiskSpace) JobDescription: %d (DiskSpace) - %d (CacheDiskSpace)", target->MaxDiskSpace, job->Resources.DiskSpaceRequirement.DiskSpace.max, job->Resources.DiskSpaceRequirement.CacheDiskSpace);
+      if (job->Resources.DiskSpaceRequirement.DiskSpace.max > -1 && job->Resources.DiskSpaceRequirement.CacheDiskSpace > -1) {
+        if (target->MaxDiskSpace > -1) {     // Example: 5656
+          if (target->MaxDiskSpace*1024 < job->Resources.DiskSpaceRequirement.DiskSpace.max - job->Resources.DiskSpaceRequirement.CacheDiskSpace) {
+            logger.msg(VERBOSE, "Matchmaking, MaxDiskSpace*1024 >= DiskSpace - CacheDiskSpace problem, ExecutionTarget: %d MB (MaxDiskSpace); JobDescription: %d MB (DiskSpace), %d MB (CacheDiskSpace)", target->MaxDiskSpace*1024, job->Resources.DiskSpaceRequirement.DiskSpace.max, job->Resources.DiskSpaceRequirement.CacheDiskSpace);
             continue;
           }
         }
-        else if (target->WorkingAreaTotal != -1) {     // Example: 5656
-          if (target->WorkingAreaTotal < job->Resources.DiskSpaceRequirement.DiskSpace - job->Resources.DiskSpaceRequirement.CacheDiskSpace) {
-            logger.msg(VERBOSE, "Matchmaking, WorkingAreaTotal >= DiskSpace - CacheDiskSpace problem, ExecutionTarget: %d (MaxDiskSpace) JobDescription: %d (DiskSpace) - %d (CacheDiskSpace)", target->WorkingAreaTotal, job->Resources.DiskSpaceRequirement.DiskSpace.max, job->Resources.DiskSpaceRequirement.CacheDiskSpace);
+
+        if (target->WorkingAreaFree > -1) {     // Example: 5656
+          if (target->WorkingAreaFree*1024 < job->Resources.DiskSpaceRequirement.DiskSpace.max - job->Resources.DiskSpaceRequirement.CacheDiskSpace) {
+            logger.msg(VERBOSE, "Matchmaking, WorkingAreaFree*1024 >= DiskSpace - CacheDiskSpace problem, ExecutionTarget: %d MB (MaxDiskSpace); JobDescription: %d MB (DiskSpace), %d MB (CacheDiskSpace)", target->WorkingAreaFree*1024, job->Resources.DiskSpaceRequirement.DiskSpace.max, job->Resources.DiskSpaceRequirement.CacheDiskSpace);
             continue;
           }
         }
-        else {
-          logger.msg(VERBOSE, "Matchmaking, ExecutionTarget:  %s, MaxDiskSpace and WorkingAreaTotal are not defined", target->url.str());
+        
+        if (target->MaxDiskSpace <= -1 && target->WorkingAreaFree <= -1) {
+          logger.msg(VERBOSE, "Matchmaking, ExecutionTarget:  %s, MaxDiskSpace and WorkingAreaFree are not defined", target->url.str());
           continue;
         }
       }
 
-      if (job->Resources.DiskSpaceRequirement.DiskSpace != -1) {
-        if (target->MaxDiskSpace != -1) {     // Example: 5656
-          if (target->MaxDiskSpace < job->Resources.DiskSpaceRequirement.DiskSpace) {
-            logger.msg(VERBOSE, "Matchmaking, MaxDiskSpace problem, ExecutionTarget: %d (MaxDiskSpace) JobDescription: %d (DiskSpace)", target->MaxDiskSpace, job->Resources.DiskSpaceRequirement.DiskSpace.max);
+      if (job->Resources.DiskSpaceRequirement.DiskSpace.max > -1) {
+        if (target->MaxDiskSpace > -1) {     // Example: 5656
+          if (target->MaxDiskSpace*1024 < job->Resources.DiskSpaceRequirement.DiskSpace.max) {
+            logger.msg(VERBOSE, "Matchmaking, MaxDiskSpace problem, ExecutionTarget: %d MB (MaxDiskSpace); JobDescription: %d MB (DiskSpace)", target->MaxDiskSpace*1024, job->Resources.DiskSpaceRequirement.DiskSpace.max);
             continue;
           }
         }
-        else if (target->WorkingAreaTotal != -1) {     // Example: 5656
-          if (target->WorkingAreaTotal < job->Resources.DiskSpaceRequirement.DiskSpace) {
-            logger.msg(VERBOSE, "Matchmaking, WorkingAreaTotal problem, ExecutionTarget: %d (WorkingAreaTotal) JobDescription: %d (DiskSpace)", target->WorkingAreaTotal, job->Resources.DiskSpaceRequirement.DiskSpace.max);
+        
+        if (target->WorkingAreaFree > -1) {     // Example: 5656
+          if (target->WorkingAreaFree*1024 < job->Resources.DiskSpaceRequirement.DiskSpace.max) {
+            logger.msg(VERBOSE, "Matchmaking, WorkingAreaFree problem, ExecutionTarget: %d MB (WorkingAreaFree); JobDescription: %d MB (DiskSpace)", target->WorkingAreaFree*1024, job->Resources.DiskSpaceRequirement.DiskSpace.max);
             continue;
           }
         }
-        else {
-          logger.msg(VERBOSE, "Matchmaking, ExecutionTarget:  %s, MaxDiskSpace and WorkingAreaTotal are not defined", target->url.str());
+        
+        if (target->WorkingAreaFree <= -1 && target->MaxDiskSpace <= -1) {
+          logger.msg(VERBOSE, "Matchmaking, ExecutionTarget:  %s, MaxDiskSpace and WorkingAreaFree are not defined", target->url.str());
           continue;
         }
       }
 
-      if (job->Resources.DiskSpaceRequirement.CacheDiskSpace != -1) {
-        if (target->CacheTotal != -1) {     // Example: 5656
-          if (target->CacheTotal < job->Resources.DiskSpaceRequirement.CacheDiskSpace) {
-            logger.msg(VERBOSE, "Matchmaking, CacheTotal problem, ExecutionTarget: %d (CacheTotal) JobDescription: %d (CacheDiskSpace)", target->CacheTotal, job->Resources.DiskSpaceRequirement.CacheDiskSpace);
+      if (job->Resources.DiskSpaceRequirement.CacheDiskSpace > -1) {
+        if (target->CacheTotal > -1) {     // Example: 5656
+          if (target->CacheTotal*1024 < job->Resources.DiskSpaceRequirement.CacheDiskSpace) {
+            logger.msg(VERBOSE, "Matchmaking, CacheTotal problem, ExecutionTarget: %d MB (CacheTotal); JobDescription: %d MB (CacheDiskSpace)", target->CacheTotal*1024, job->Resources.DiskSpaceRequirement.CacheDiskSpace);
             continue;
           }
         }
@@ -452,13 +469,20 @@ namespace Arc {
       JobDescription jobdescription;
       // Return if test jobdescription is not defined
       if (!((*it)->GetTestJob(usercfg, testid, jobdescription))) {
-        logger.msg(ERROR, "Illegal testjob-id given");
-        return false;
-      }
-      if ((*it)->Submit(usercfg, jobdescription, job)) {
-        current = it;
-        RegisterJobsubmission();
-        return true;
+        std::ostringstream ids;
+        int i = 0;
+        while ((*it)->GetTestJob(usercfg, ++i, jobdescription)) {
+          if ( i-1 == 0 ) ids << i;
+          else ids << ", " << i;
+        }
+        if ( i-1 == 0 ) logger.msg(Arc::INFO, "For this middleware there are no testjobs defined.");
+        else logger.msg(Arc::INFO, "For this middleware only %s testjobs are defined.", ids.str());
+      } else {
+        if ((*it)->Submit(usercfg, jobdescription, job)) {
+          current = it;
+          RegisterJobsubmission();
+          return true;
+        }
       }
     }
 
