@@ -1,6 +1,7 @@
 #include "TestACCControl.h"
 
 bool* BrokerTestACCControl::TargetSortingDone = NULL;
+bool* BrokerTestACCControl::TargetSortingDoneSortTargets = NULL;
 std::list<Arc::ExecutionTarget*>* BrokerTestACCControl::PossibleTargets = NULL;
 
 bool JobDescriptionParserTestACCControl::parseStatus = false;
@@ -26,3 +27,5 @@ Arc::Job* SubmitterTestACCControl::submitJob = NULL;
 Arc::Job* SubmitterTestACCControl::migrateJob = NULL;
 
 Arc::TargetGenerator* TargetRetrieverTestACCControl::tg = NULL;
+std::list<Arc::ExecutionTarget>* TargetRetrieverTestACCControl::foundTargets = NULL;
+std::list<Arc::Job>* TargetRetrieverTestACCControl::foundJobs = NULL;
