@@ -12,6 +12,7 @@
 class BrokerTestACCControl {
   public:
     static bool* TargetSortingDone;
+    static bool* TargetSortingDoneSortTargets;
     static std::list<Arc::ExecutionTarget*>* PossibleTargets;
 };
 
@@ -49,6 +50,8 @@ class SubmitterTestACCControl {
 class TargetRetrieverTestACCControl {
   public:
     static Arc::TargetGenerator* tg;
+    static std::list<Arc::ExecutionTarget>* foundTargets;
+    static std::list<Arc::Job>* foundJobs;
 };
 
 #endif // __ARC_TESTACCCONTROL_H__
