@@ -2,6 +2,15 @@
 
 #include <arc/Thread.h>
 
+#ifdef WIN32
+#ifndef uid_t
+#define uid_t int
+#endif
+#ifndef gid_t
+#define gid_t int
+#endif
+#endif
+
 namespace Arc {
 
   /// Utility functions for handling files and directories.
