@@ -9,6 +9,9 @@
 
 #ifdef HAVE_GLIBMM_OPTIONCONTEXT_SET_SUMMARY
 #include <glibmm/optioncontext.h>
+#ifndef HAVE_GLIBMM_OPTIONCONTEXT_GET_HELP
+#include <arc/IString.h>
+#endif
 #else
 #include <getopt.h>
 #include <arc/IString.h>
@@ -244,7 +247,7 @@ namespace Arc {
 
     try {
       ctx.parse(argc, argv);
-    } catch (Glib::OptionError err) {
+    } catch (Glib::OptionError& err) {
       std::cout << err.what() << std::endl;
       exit(1);
     }
@@ -252,7 +255,7 @@ namespace Arc {
 #ifdef HAVE_GLIBMM_OPTIONCONTEXT_GET_HELP
       std::cout << ctx.get_help() << std::endl;
 #else
-      std::cout << "Use -? to get usage description" << std::endl;
+      std::cout << IString("Use -? to get usage description") << std::endl;
 #endif
       exit(0);
     }
