@@ -88,9 +88,9 @@ namespace Arc {
     Pid(void):p_(0) { }; 
     Pid(Glib::Pid p):p_(p) { }; 
     ~Pid(void) { if(p_) Glib::spawn_close_pid(p_); };
-    operator Glib::Pid(void) { return p_; };
-    Glib::Pid pid(void) { return p_; };
-    Glib::Pid operator=(Glib::Pid p) { return (p_=p); };
+    operator Glib::Pid(void) const { return p_; };
+    Glib::Pid pid(void) const { return p_; };
+    Glib::Pid& operator=(const Glib::Pid& p) { return (p_=p); };
   };
 
   class RunInitializerArgument {
@@ -199,10 +199,8 @@ namespace Arc {
           // returning false non-stop was observed. So here we 
           // are trying to prevent that.
           if((++storm_count) >= 10) {
-std::cout<<"--- "<<"!dispatched sleep"<<std::endl;
             sleep(1);
             storm_count = 0;
-std::cout<<"--- "<<"!dispatched sleep ends"<<std::endl;
           };
         } else {
           storm_count = 0;
@@ -302,6 +300,7 @@ std::cout<<"--- "<<"!dispatched sleep ends"<<std::endl;
       initializer_func_(NULL),
       initializer_arg_(NULL),
       kicker_func_(NULL),
+      kicker_arg_(NULL),
       started_(false),
       running_(false),
       abandoned_(false),
@@ -327,6 +326,7 @@ std::cout<<"--- "<<"!dispatched sleep ends"<<std::endl;
       initializer_func_(NULL),
       initializer_arg_(NULL),
       kicker_func_(NULL),
+      kicker_arg_(NULL),
       started_(false),
       running_(false),
       abandoned_(false),
@@ -343,6 +343,7 @@ std::cout<<"--- "<<"!dispatched sleep ends"<<std::endl;
       CloseStderr();
       CloseStdin();
       RunPump::Instance().Remove(this);
+      delete pid_;
     };
   }
 
