@@ -1,3 +1,5 @@
+// TODO: test for operations under different account
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -18,6 +20,7 @@ class FileUtilsTest
   CPPUNIT_TEST(TestFileCreateAndRead);
   CPPUNIT_TEST(TestMakeAndDeleteDir);
   CPPUNIT_TEST(TestTmpDirCreate);
+  CPPUNIT_TEST(TestTmpFileCreate);
   CPPUNIT_TEST_SUITE_END();
 
 public:
@@ -30,6 +33,7 @@ public:
   void TestFileCreateAndRead();
   void TestMakeAndDeleteDir();
   void TestTmpDirCreate();
+  void TestTmpFileCreate();
 
 private:
   bool _createFile(const std::string& filename, const std::string& text = "a");
@@ -117,6 +121,7 @@ void FileUtilsTest::TestMakeAndDeleteDir() {
   CPPUNIT_ASSERT(stat(testroot.c_str(), &st) == 0);
   CPPUNIT_ASSERT(_createFile(testroot + "/file1"));
   CPPUNIT_ASSERT(Arc::DirCreate(std::string(testroot + "/dir1"), S_IRUSR | S_IWUSR | S_IXUSR));
+  CPPUNIT_ASSERT(Arc::DirCreate(std::string(testroot + "/dir1"), S_IRUSR | S_IWUSR | S_IXUSR));
   CPPUNIT_ASSERT(stat(std::string(testroot + "/dir1").c_str(), &st) == 0);
   CPPUNIT_ASSERT(S_ISDIR(st.st_mode));
   CPPUNIT_ASSERT(_createFile(testroot + "/dir1/file2"));
@@ -143,6 +148,17 @@ void FileUtilsTest::TestTmpDirCreate() {
   CPPUNIT_ASSERT(stat(path.c_str(), &st) != 0);
 }
 
+void FileUtilsTest::TestTmpFileCreate() {
+  std::string path;
+  CPPUNIT_ASSERT(Arc::TmpFileCreate(path,"TEST"));
+  struct stat st;
+  CPPUNIT_ASSERT(stat(path.c_str(), &st) == 0);
+  CPPUNIT_ASSERT(S_ISREG(st.st_mode));
+  CPPUNIT_ASSERT_EQUAL(4,(int)st.st_size);
+  CPPUNIT_ASSERT(Arc::FileDelete(path));
+  CPPUNIT_ASSERT(stat(path.c_str(), &st) != 0);
+}
+
 bool FileUtilsTest::_createFile(const std::string& filename, const std::string& text) {
 
   FILE *pFile;
