@@ -91,13 +91,15 @@ int verify_cert_chain(X509* cert, STACK_OF(X509)** certchain, cert_verify_contex
   }
 
   //Replace the trusted certificate chain after verification passed, the
-  //trusted ca certificate is added
-  if(*certchain) { sk_X509_pop_free(*certchain, X509_free); }
-  *certchain = sk_X509_new_null();
-
-  if(store_ctx != NULL) for (i=0; i < sk_X509_num(store_ctx->chain); i++) {
-    X509* tmp = NULL; tmp = X509_dup(sk_X509_value(store_ctx->chain,i));
-    sk_X509_insert(*certchain, tmp, i);
+  //trusted ca certificate is added but top cert is excluded if already
+  //stored separately.
+  if(store_ctx != NULL) {
+    if(*certchain) { sk_X509_pop_free(*certchain, X509_free); }
+    *certchain = sk_X509_new_null();
+    for (i=(cert)?1:0; i < sk_X509_num(store_ctx->chain); i++) {
+      X509* tmp = NULL; tmp = X509_dup(sk_X509_value(store_ctx->chain,i));
+      sk_X509_insert(*certchain, tmp, i);
+    }
   }
 
   retval = 1;
@@ -281,18 +283,29 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
       return (0);
     }
 
+    /* Limited proxy does not mean it can't be followed by unlimited proxy
+       because proxy chain is checked recusrsively according to RFC3820.
+       Unless specific limitation put on proxy means it can't be used
+       for creating another proxy. But for that purpose proxy depth
+       is used.
+       In general it is task of authorization to check ALL proxies in 
+       the chain for valid policies. */
+    /*
     if(CERT_IS_LIMITED_PROXY(vctx->cert_type) &&
        !(CERT_IS_LIMITED_PROXY(type) || CERT_IS_INDEPENDENT_PROXY(type))) {
       logger.msg(Arc::ERROR,"Can't sign a non-limited, non-independent proxy with a limited proxy");
       store_ctx->error = X509_V_ERR_CERT_SIGNATURE_FAILURE;
       return (0);
     }
+    */
 
     vctx->proxy_depth++;
-    if(vctx->max_proxy_depth!=-1 && vctx->max_proxy_depth < vctx->proxy_depth) {
+    if((vctx->max_proxy_depth!=-1) &&
+       (vctx->max_proxy_depth < vctx->proxy_depth)) {
       logger.msg(Arc::ERROR,"The proxy depth %i is out of maximum limit %i",vctx->proxy_depth,vctx->max_proxy_depth);
       return (0);
     }
+    // vctx stores previous certificate type
     vctx->cert_type=type;
   }
 	
