@@ -18,20 +18,6 @@
 
 #include <arc/credential/CertUtil.h>
 
-/**Credential class covers the functionality about general processing about certificate/key
- * files, including:
- * 1. cerficate/key parsing, information extracting (such as subject name,
- * issuer name, lifetime, etc.), chain verifying, extension processing about proxy certinfo,
- * extension processing about other general certificate extension (such as voms attributes,
- * it should be the extension-specific code itself to create, parse and verify the extension,
- * not the Credential class. For voms, it is some code about writing and parsing voms-implementing
- * Attibute Certificate/ RFC3281, the voms-attibute is then be looked as a binary part and
- * embeded into extension of X509 certificate/proxy certificate);
- * 2. certificate request, extension emeding and certificate signing, for both proxy certificate
- * and EEC (end entity certificate) certificate
- * The Credential class support PEM, DER PKCS12 credential.
- */
-
 namespace Arc {
   // An exception class for the Credential class.
   /** This is an exception class that is used to handle runtime errors
@@ -51,6 +37,19 @@ typedef enum {CRED_PEM, CRED_DER, CRED_PKCS, CRED_UNKNOWN} Credformat;
 /**Logger to be used by all modules of credentials library*/
 extern Logger CredentialLogger;
 
+/**Credential class covers the functionality about general processing about certificate/key
+ * files, including:
+ * 1. cerficate/key parsing, information extracting (such as subject name,
+ * issuer name, lifetime, etc.), chain verifying, extension processing about proxy certinfo,
+ * extension processing about other general certificate extension (such as voms attributes,
+ * it should be the extension-specific code itself to create, parse and verify the extension,
+ * not the Credential class. For voms, it is some code about writing and parsing voms-implementing
+ * Attibute Certificate/ RFC3281, the voms-attibute is then be looked as a binary part and
+ * embeded into extension of X509 certificate/proxy certificate);
+ * 2. certificate request, extension emeding and certificate signing, for both proxy certificate
+ * and EEC (end entity certificate) certificate
+ * The Credential class support PEM, DER PKCS12 credential.
+ */
 class Credential {
   public:
     /**Default constructor, only acts as a container for inquiring certificate request,
@@ -180,7 +179,7 @@ class Credential {
     * will not after the ealier value from issuer's end
     * time and the "start" parameter plus "lifetime" paremeter
     */
-    bool SetProxyPeriod(X509* tosign, X509* issuer, Time& start, Period& lifetime);
+    bool SetProxyPeriod(X509* tosign, X509* issuer, const Time& start, const Period& lifetime);
 
     /**Assistant method for signing the proxy request, the method will duplicate some information
     *(subject and extension) from signing certificate
@@ -237,6 +236,11 @@ class Credential {
     /**Get issuer of the certificate attached to this object*/
     std::string GetIssuerName(void) const;
 
+    /**Get CA of the certificate attached to this object, if the certificate 
+     *is an EEC, GetCAName get the same value as GetIssuerName
+     */
+    std::string GetCAName(void) const;
+
     /**Get the proxy policy attached to the "proxy certificate
      * information" extension of the proxy certicate
      */
@@ -311,6 +315,13 @@ class Credential {
     */
     bool AddExtension(const std::string& name, char** binary);
 
+    /**Get the specific extension (named by the parameter) in a certificate
+    * this function is only supposed to be called after certificate and key
+    * are loaded by the constructor for usual certificate 
+    *@param name, the name of the extension to get
+    */
+    std::string GetExtension(const std::string& name);
+
     /**Generate an EEC request, based on the keybits and signing
     * algorithm information inside this object
     * output the certificate request to output BIO
@@ -367,7 +378,14 @@ class Credential {
      */
     bool SignRequest(Credential* proxy, const char* filename, bool foamat = false);
 
-    bool SelfSignEECRequest(const std::string& dn, const char* filename);
+    /**Self sign a certificate. This functionality is specific for creating a CA credential
+    * by using this Credential class. 
+    * @param dn  the DN for the subject
+    * @param extfile  the configration file which includes the extension information, typically the openssl.cnf file
+    * @param extsect  the section/group name for the extension, e.g. in openssl.cnf, usr_cert and v3_ca
+    * @param certfile the certificate file, which contains the signed certificate 
+    */
+    bool SelfSignEECRequest(const std::string& dn, const char* extfile, const std::string& extsect, const char* certfile);
 
     //The following three methods is about signing an EEC certificate by implementing the same
     //functionality as a normal CA
