@@ -16,9 +16,32 @@ namespace Arc {
 
   class VOMSACInfo {
    public:
+    // Not all statuses are implemented
+    typedef enum {
+      Success = 0,
+      CAUnknown = (1<<0), // Signed by VOMS certificate of unknow CA
+      CertRevoked = (1<<1), // Signed by revoked VOMS certificate
+      LSCFailed = (1<<2), // Failed while matching VOMS attr. against LSC files
+      TrustFailed = (1<<2), // Failed matching VOMS attr. against specified trust list
+      X509ParsingFailed = (1<<3), // Failed while parsing at X509 level
+      ACParsingFailed = (1<<4), // Failed while parsing at AC level
+      InternalParsingFailed = (1<<5), // Failed while parsing internal VOMS structures
+      TimeValidFailed = (1<<6), // VOMS attributes are not valid yet or expired
+      IsCritical = (1<<7), // VOMS extension was marked as critical (unusual but not error)
+      ParsingError = (X509ParsingFailed | ACParsingFailed | InternalParsingFailed), // Mask to test if status represents any failure caused by failed parsing
+      ValidationError = (CAUnknown | CertRevoked | LSCFailed | TrustFailed | TimeValidFailed), // Mask to test if status represents any failure caused by validation rules
+      Error = (0xffff & ~IsCritical) // Mask to test if status represents any failure
+    } status_t;
     std::string voname;
+    std::string holder;
+    std::string issuer;
+    std::string target;
     std::vector<std::string> attributes;
-    Period validity;
+    Time from;
+    Time till;
+    //Period validity;
+    unsigned int status;
+    VOMSACInfo(void):from(-1),till(-1),status(0) { };
   };
 
   /** Stores definitions for making decision if VOMS server is trusted */
@@ -89,6 +112,7 @@ namespace Arc {
       VOMSTrustChain& AddChain(const VOMSTrustChain& chain);
       /** Adds empty chain of trusted DNs to list. */
       VOMSTrustChain& AddChain(void);
+      void AddElement(const std::vector<std::string>& encoded_list);
       /** Adds regular expression to list.
         During verification each signature of AC is checked against
         all stored regular expressions. DN of signing certificate
@@ -124,6 +148,7 @@ namespace Arc {
    *  			  as the grantor of this AC
    * @param lifetime	The lifetime of this AC
    */
+  /*
   int createVOMSAC(X509 *issuer, STACK_OF(X509) *issuerstack, X509 *holder,
                    EVP_PKEY *pkey, BIGNUM *serialnum,
                    std::vector<std::string> &fqan,
@@ -131,6 +156,7 @@ namespace Arc {
                    std::vector<std::string>& attributes,
                    ArcCredential::AC **ac, std::string voname,
                    std::string uri, int lifetime);
+  */
 
   /**Create AC(Attribute Certificate) with voms specific format.
    * @param codedac	  The coded AC as output of this method
@@ -191,36 +217,55 @@ namespace Arc {
    * 		    ca_cert_dir/ca_cert_file which specifies the CA certificates, 
    * 		    and the vomscert_trust_dn which specifies the trusted DN chain
    * 		    from voms server certificate to CA certificate.
-   *
    * 		    false: Not verify, which means the issuer of AC (voms server
    * 		    certificate is supposed to be trusted by default).
    * 		    In this case the parameters 'ca_cert_dir', 
    * 		    'ca_cert_file' and 'vomscert_trust_dn' will not effect, and 
-   * 		    should be set as empty.
+   * 		    may be left empty.
    * 		    This case is specifically used by 'arcproxy --info' to list 
    * 		    all of the attributes in AC, and not to need to verify if
    * 		    the AC's issuer is trusted.
+   *
+   * @param reportall If set to true fills output with all attributes
+   *                including those which failed passing test procedures.
+   *                Validity of attributes can be checked through status 
+   *                members of output items.
+   *                Combination of verify=true and reportall=true provides 
+   *                most information.
+   *
    */
   bool parseVOMSAC(X509* holder, const std::string& ca_cert_dir,
                    const std::string& ca_cert_file, 
-                   const VOMSTrustList& vomscert_trust_dn,
-                   std::vector<VOMSACInfo>& output, bool verify = true);
+                   VOMSTrustList& vomscert_trust_dn,
+                   std::vector<VOMSACInfo>& output, 
+                   bool verify = true, bool reportall = false);
 
   /**Parse the certificate. Similar to above one, but collects information
     From all certificates in a chain. */
   bool parseVOMSAC(const Credential& holder_cred,
                    const std::string& ca_cert_dir,
                    const std::string& ca_cert_file, 
-                   const VOMSTrustList& vomscert_trust_dn,
-                   std::vector<VOMSACInfo>& output, bool verify = true);
+                   VOMSTrustList& vomscert_trust_dn,
+                   std::vector<VOMSACInfo>& output,
+                   bool verify = true, bool reportall = false);
   
   /**Decode the data which is encoded by voms server. Since voms code uses some specific
   * coding method (not base64 encoding), we simply copy the method from voms code to here*/
   char *VOMSDecode(const char *data, int size, int *j);
 
-  /**Extract the needed field from the certificate
+  /**Extract the needed field from the certificate.
+   * @param u  The proxy certificate which includes the voms
+   *          specific formated AC.
+   * @param property The property that caller would get, 
+   *          including: dn, voms:vo, voms:role, voms:group 
+   * @param ca_cert_dir 
+   * @param ca_cert_file
+   * @param voms_trust_list  the dn chain that is trusted when parsing voms AC
   */
-  const std::string get_property(const Arc::Credential& u, const std::string property);
+  std::string getCredentialProperty(const Arc::Credential& u, const std::string& property, const std::string& ca_cert_dir = std::string(""), const std::string& ca_cert_file = std::string(""), const std::vector<std::string>& voms_trust_list = std::vector<std::string>());
+
+  std::string VOMSFQANToFull(const std::string& vo, const std::string& fqan);
+
 }// namespace Arc
 
 #endif /* __ARC_VOMSUTIL_H__ */
