@@ -184,7 +184,7 @@ namespace Arc {
   }
 
   int DataPointDirect::AddCheckSumObject(CheckSum *cksum) {
-    if(!cksum)
+    if(!cksum || !(*cksum))
       return -1;
     cksum->start();
     checksums.push_back(cksum);
