@@ -30,9 +30,10 @@ Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
     {
         "testplugin",         /* name */
         "TEST",               /* kind */
+        NULL,                 /* description */
         0,                    /* version */
         Test::get_instance    /* get_instance function */
     },
-    { NULL, 0, NULL }
+    { NULL, NULL, 0, NULL }
 };
 
