@@ -15,6 +15,8 @@
 
 namespace Arc {
 
+Arc::Logger Arc::PayloadHTTP::logger(Arc::Logger::getRootLogger(), "MCC.HTTP");
+
 static std::string empty_string("");
 
 static bool ParseHTTPVersion(const std::string& s,int& major,int& minor) {
@@ -287,6 +289,7 @@ bool PayloadHTTP::flush_multipart(void) {
 bool PayloadHTTP::read_header(void) {
   std::string line;
   for(;readline_chunked(line) && (!line.empty());) {
+    logger.msg(Arc::DEBUG,"< %s",line);
     std::string::size_type pos = line.find(':');
     if(pos == std::string::npos) continue;
     std::string name = line.substr(0,pos);
@@ -318,6 +321,7 @@ bool PayloadHTTP::read_header(void) {
         if(((*e) == '/') || ((*e) == 0)) {
           if(range_start <= range_end) {
             offset_=range_start;
+            end_=range_end+1;
           };
           if((*p) == '/') {
             p++; entity_size=strtoull(p,&e,10);
@@ -389,6 +393,7 @@ bool PayloadHTTP::parse_header(void) {
     length_=0;
     return true;
   };
+  logger.msg(Arc::DEBUG,"< %s",line);
   // Parse request/response line
   std::string::size_type pos2 = line.find(' ');
   if(pos2 == std::string::npos) return false;
@@ -465,6 +470,7 @@ bool PayloadHTTP::get_body(void) {
       if(new_result == NULL) { free(result); return false; };
       result=new_result;
       if(!read_multipart(result+result_size,chunk_size)) break;
+  // TODO: logical size is not always same as end of body
       result_size+=chunk_size;
     };
   };
@@ -501,8 +507,10 @@ void PayloadHTTP::Attribute(const std::string& name,const std::string& value) {
 
 PayloadHTTP::PayloadHTTP(PayloadStreamInterface& stream,bool own):
     valid_(false),fetched_(false),stream_(&stream),stream_own_(own),
-    rbody_(NULL),sbody_(NULL),body_own_(false),keep_alive_(true),
-    stream_offset_(0),head_response_(false) {
+    rbody_(NULL),sbody_(NULL),body_own_(false),
+    code_(0),length_(0),end_(0),chunked_(CHUNKED_NONE),chunk_size_(0),
+    multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
+    head_response_(false) {
   tbuf_[0]=0; tbuflen_=0;
   if(!parse_header()) return;
   // If stream_ is owned then body can be fetched later
@@ -513,7 +521,8 @@ PayloadHTTP::PayloadHTTP(PayloadStreamInterface& stream,bool own):
 PayloadHTTP::PayloadHTTP(const std::string& method,const std::string& url,PayloadStreamInterface& stream):
     valid_(true),fetched_(true),stream_(&stream),stream_own_(false),
     rbody_(NULL),sbody_(NULL),body_own_(false),uri_(url),method_(method),
-    code_(0),chunked_(CHUNKED_NONE),chunk_size_(0),multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
+    code_(0),length_(0),end_(0),chunked_(CHUNKED_NONE),chunk_size_(0),
+    multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
     head_response_(false) {
   tbuf_[0]=0; tbuflen_=0;
   version_major_=1; version_minor_=1;
@@ -522,8 +531,9 @@ PayloadHTTP::PayloadHTTP(const std::string& method,const std::string& url,Payloa
 
 PayloadHTTP::PayloadHTTP(int code,const std::string& reason,PayloadStreamInterface& stream,bool head_response):
     valid_(true),fetched_(true),stream_(&stream),stream_own_(false),
-    rbody_(NULL),sbody_(NULL),body_own_(false),code_(code),
-    reason_(reason),chunked_(CHUNKED_NONE),chunk_size_(0),multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
+    rbody_(NULL),sbody_(NULL),body_own_(false),
+    code_(code),length_(0),end_(0),reason_(reason),chunked_(CHUNKED_NONE),chunk_size_(0),
+    multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
     head_response_(head_response) {
   tbuf_[0]=0; tbuflen_=0;
   version_major_=1; version_minor_=1;
@@ -533,7 +543,8 @@ PayloadHTTP::PayloadHTTP(int code,const std::string& reason,PayloadStreamInterfa
 PayloadHTTP::PayloadHTTP(const std::string& method,const std::string& url):
     valid_(true),fetched_(true),stream_(NULL),stream_own_(false),
     rbody_(NULL),sbody_(NULL),body_own_(false),uri_(url),method_(method),
-    code_(0),chunked_(CHUNKED_NONE),chunk_size_(0),multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
+    code_(0),length_(0),end_(0),chunked_(CHUNKED_NONE),chunk_size_(0),
+    multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
     head_response_(false) {
   tbuf_[0]=0; tbuflen_=0;
   version_major_=1; version_minor_=1;
@@ -542,8 +553,9 @@ PayloadHTTP::PayloadHTTP(const std::string& method,const std::string& url):
 
 PayloadHTTP::PayloadHTTP(int code,const std::string& reason,bool head_response):
     valid_(true),fetched_(true),stream_(NULL),stream_own_(false),
-    rbody_(NULL),sbody_(NULL),body_own_(false),code_(code),
-    reason_(reason),chunked_(CHUNKED_NONE),chunk_size_(0),multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
+    rbody_(NULL),sbody_(NULL),body_own_(false),
+    code_(code),length_(0),end_(0),reason_(reason),chunked_(CHUNKED_NONE),chunk_size_(0),
+    multipart_(MULTIPART_NONE),keep_alive_(true),stream_offset_(0),
     head_response_(head_response) {
   tbuf_[0]=0; tbuflen_=0;
   version_major_=1; version_minor_=1;
@@ -551,7 +563,7 @@ PayloadHTTP::PayloadHTTP(int code,const std::string& reason,bool head_response):
 }
 
 PayloadHTTP::~PayloadHTTP(void) {
-  // allign to and of message
+  // allign to end of message
   flush_multipart();
   flush_chunked();
   if(rbody_ && body_own_) delete rbody_;
@@ -564,30 +576,30 @@ bool PayloadHTTP::Flush(void) {
   bool to_stream = (stream_ != NULL);
   if(method_.empty() && (code_ == 0)) return false;
   // Computing length of Body part
-  length_=0;
+  int64_t length = 0;
   std::string range_header;
   if((method_ != "GET") && (method_ != "HEAD")) {
     int64_t start = 0;
     if(head_response_ && (code_==HTTP_OK)) {
-      length_ = Size();
+      length = Size();
     } else {
       if(sbody_) {
         if(sbody_->Limit() > sbody_->Pos()) {
-          length_ = sbody_->Limit() - sbody_->Pos();
+          length = sbody_->Limit() - sbody_->Pos();
         };
         start=sbody_->Pos();
       } else {
         for(int n=0;;++n) {
           if(Buffer(n) == NULL) break;
-          length_+=BufferSize(n);
+          length+=BufferSize(n);
         };
         start=BufferPos(0);
       };
     };
-    if(length_ != Size()) {
+    if(length != Size()) {
       // Add range definition if Body represents part of logical buffer size
       // and adjust HTTP code accordingly
-      int64_t end = start+length_;
+      int64_t end = start+length;
       std::string length_str;
       std::string range_str;
       if(end <= Size()) {
@@ -610,7 +622,7 @@ bool PayloadHTTP::Flush(void) {
       };
       range_header="Content-Range: bytes "+range_str+"/"+length_str+"\r\n";
     };
-    range_header+="Content-Length: "+tostring(length_)+"\r\n";
+    range_header+="Content-Length: "+tostring(length)+"\r\n";
   };
   // Starting header
   if(!method_.empty()) {
@@ -650,14 +662,15 @@ bool PayloadHTTP::Flush(void) {
     header+=(a->first)+": "+(a->second)+"\r\n";
   };
   header+="\r\n";
+  logger.msg(Arc::DEBUG,"> %s",header);
   if(to_stream) {
     if(!stream_->Put(header)) return false;
-    if(length_ > 0) {
+    if(length > 0) {
       if(sbody_) {
         // stream to stream transfer
         // TODO: choose optimal buffer size
         // TODO: parallel read and write for better performance
-        int tbufsize = (length_>1024*1024)?(1024*1024):length_;
+        int tbufsize = (length>1024*1024)?(1024*1024):length;
         char* tbuf = new char[tbufsize];
         if(!tbuf) return false;
         for(;;) {
@@ -725,14 +738,26 @@ char* PayloadHTTP::Content(PayloadRawInterface::Size_t pos) {
 }
 
 PayloadRawInterface::Size_t PayloadHTTP::Size(void) const {
-  if(!((PayloadHTTP*)this)->get_body()) return 0;
+  if(!valid_) return 0;
+  PayloadRawInterface::Size_t size = 0;
+  if(size_ > 0) {
+    size = size_;
+  } else if(end_ > 0) {
+    size = end_;
+  } else if(length_ >= 0) {
+    size = offset_ + length_;
+  } else {
+    // Only do it if no other way of determining size worked.
+    if(!((PayloadHTTP*)this)->get_body()) return 0;
+    size = PayloadRaw::Size();
+  }
   if(rbody_) {
-    return PayloadRaw::Size() + (rbody_->Size());
+    return size + (rbody_->Size());
   };
   if(sbody_) {
-    return PayloadRaw::Size() + (sbody_->Size());
+    return size + (sbody_->Size());
   };
-  return PayloadRaw::Size();
+  return size;
 }
 
 char* PayloadHTTP::Insert(PayloadRawInterface::Size_t pos,PayloadRawInterface::Size_t size) {
@@ -774,6 +799,9 @@ PayloadRawInterface::Size_t PayloadHTTP::BufferSize(unsigned int num) const {
 }
 
 PayloadRawInterface::Size_t PayloadHTTP::BufferPos(unsigned int num) const {
+  if((num == 0) && (buf_.size() == 0) && (!rbody_) && (!sbody_)) {
+    return offset_;
+  }
   if(!((PayloadHTTP*)this)->get_body()) return 0;
   if(num < buf_.size()) {
     return PayloadRaw::BufferPos(num);
@@ -916,6 +944,7 @@ PayloadStreamInterface::Size_t PayloadHTTP::Pos(void) const {
 }
 
 PayloadStreamInterface::Size_t PayloadHTTP::Limit(void) const {
+  // TODO: logical size is not always same as end of body
   return Size();
 }
 
