@@ -8,6 +8,7 @@
 #include <stdint.h>
 #endif
 
+#include <arc/Logger.h>
 #include <arc/message/PayloadRaw.h>
 #include <arc/message/PayloadStream.h>
 
@@ -26,6 +27,7 @@ namespace Arc {
   for including as payload into Message passed through MCC chains. */
 class PayloadHTTP: virtual public PayloadRaw, virtual public PayloadStreamInterface {
  protected:
+  static Arc::Logger logger;
   typedef enum {
     MULTIPART_NONE = 0,
     MULTIPART_START,
@@ -62,6 +64,7 @@ class PayloadHTTP: virtual public PayloadRaw, virtual public PayloadStreamInterf
   int64_t length_;                 /** Content-length of HTTP message */
   //int offset_;                   /** Logical beginning of content computed from Content-Range */
   //int size_;                     /** Logical size of content obtained from Content-Range */
+  int64_t end_;                    /** Logical end of content computed from Content-Range */
   chunked_t chunked_;              /** chunked encoding parsing state */
   int64_t chunk_size_;
   bool keep_alive_;                /** true if conection should not be closed after response */
