@@ -2,7 +2,11 @@
 #include <config.h>
 #endif
 
+#include <utime.h>
+
+#include <arc/FileLock.h>
 #include <arc/GUID.h>
+#include <arc/Utils.h>
 
 #include "Processor.h"
 #include "DataDelivery.h"
@@ -40,12 +44,15 @@ namespace DataStaging {
        cfg(usercfg),
        source_endpoint(source_url, cfg),
        destination_endpoint(destination_url, cfg),
+       source_url_str(source_url.str()),
+       destination_url_str(destination_url.str()),
        user(uid),
        parent_job_id(jobid),
        priority(50),
        transfershare("_default"),
        sub_share(""),
        tries_left(1),
+       initial_tries(1),
        replication(false),
        force_registration(false),
        status(DTRStatus::NEW,"Created by the generator"),
@@ -110,6 +117,8 @@ namespace DataStaging {
       cfg(dtr.cfg),
       source_endpoint(source_url, cfg),
       destination_endpoint(destination_url, cfg),
+      source_url_str(source_url.str()),
+      destination_url_str(destination_url.str()),
       cache_file(dtr.cache_file),
       cache_parameters(dtr.cache_parameters),
       cache_state(dtr.cache_state),
@@ -119,6 +128,7 @@ namespace DataStaging {
       transfershare(dtr.transfershare),
       sub_share(dtr.sub_share),
       tries_left(dtr.tries_left),
+      initial_tries(dtr.initial_tries),
       replication(dtr.replication),
       force_registration(dtr.force_registration),
       mapped_source(dtr.mapped_source),
@@ -138,8 +148,8 @@ namespace DataStaging {
       lock()
   {
     // set insecure by default. Real value will come from configuration
-    source_endpoint->SetSecure(false);
-    destination_endpoint->SetSecure(false);
+    if (source_endpoint) source_endpoint->SetSecure(false);
+    if (destination_endpoint) destination_endpoint->SetSecure(false);
 
     mark_modification();
   }
@@ -203,7 +213,8 @@ namespace DataStaging {
   }
   
   void DTR::set_tries_left(unsigned int tries) {
-    tries_left = tries;
+    initial_tries = tries;
+    tries_left = initial_tries;
   }
 
   void DTR::decrease_tries_left() {
@@ -253,7 +264,6 @@ namespace DataStaging {
 
   void DTR::set_bytes_transferred(unsigned long long int bytes) {
     bytes_transferred = bytes;
-    mark_modification();
   }
 
   void DTR::set_cache_file(const std::string& filename)
@@ -282,6 +292,7 @@ namespace DataStaging {
     next_process_time.SetTime(t.GetTime(), t.GetTimeNanosec());
   }
 
+
   std::list<DTRCallback*> DTR::get_callbacks(const std::map<StagingProcesses,std::list<DTRCallback*> >& proc_callback, StagingProcesses owner) {
     std::list<DTRCallback*> l;
     lock.lock();
@@ -353,7 +364,7 @@ namespace DataStaging {
   }
   
   bool DTR::is_destined_for_delivery() const {
-  	return (status == DTRStatus::TRANSFER_WAIT);
+  	return (status == DTRStatus::TRANSFER);
   }
   
   bool DTR::came_from_pre_processor() const {
@@ -384,7 +395,7 @@ namespace DataStaging {
    	   status == DTRStatus::ERROR);
   }
   
-  void DTR::set_transfer_share(std::string share_name) {
+  void DTR::set_transfer_share(const std::string& share_name) {
   	lock.lock(); 
   	transfershare = share_name;
   	if (!sub_share.empty())
