@@ -66,7 +66,6 @@ namespace DataStaging {
   enum CacheState {
     CACHEABLE,             ///< Source should be cached
     NON_CACHEABLE,         ///< Source should not be cached
-    CACHE_RENEW,           ///< Cache file should be deleted then re-downloaded
     CACHE_ALREADY_PRESENT, ///< Source is available in cache from before
     CACHE_DOWNLOADED,      ///< Source has just been downloaded and put in cache
     CACHE_LOCKED,          ///< Cache file is locked
@@ -170,6 +169,11 @@ namespace DataStaging {
       /// Destination file
       Arc::DataHandle destination_endpoint;
       
+      /// Source file as a string
+      std::string source_url_str;
+      /// Destination file as a string
+      std::string destination_url_str;
+
       /// Endpoint of cached file.
       /* Kept as string so we don't need to duplicate DataHandle properties
        * of destination. Delivery should check if this is set and if so use
@@ -203,6 +207,9 @@ namespace DataStaging {
       /// Number of attempts left to complete this DTR
       unsigned int tries_left;
 
+      /// Initial number of attempts
+      unsigned int initial_tries;
+
       /// A flag to say whether the DTR is replicating inside the same LFN of an index service
       bool replication;
 
@@ -355,6 +362,11 @@ namespace DataStaging {
      /// Get destination handle. Return by reference since DataHandle cannot be copied
      const Arc::DataHandle& get_destination() const { return destination_endpoint; };
 
+     /// Get source as a string
+     std::string get_source_str() const { return source_url_str; };
+     /// Get destination as a string
+     std::string get_destination_str() const { return destination_url_str; };
+
      /// Get the UserConfig object associated with this DTR
      const Arc::UserConfig& get_usercfg() const { return cfg; };
 
@@ -383,7 +395,7 @@ namespace DataStaging {
      int get_priority() const { return priority; };
      
      /// Set the transfer share. sub_share is automatically added to transfershare
-     void set_transfer_share(std::string share_name);
+     void set_transfer_share(const std::string& share_name);
      /// Get the transfer share. sub_share is automatically added to transfershare
      std::string get_transfer_share() const { return transfershare; };
      
@@ -396,6 +408,8 @@ namespace DataStaging {
      void set_tries_left(unsigned int tries);
      /// Get the number of attempts remaining
      unsigned int get_tries_left() const { return tries_left; };
+     /// Get the initial number of attempts (set by set_tries_left())
+     unsigned int get_initial_tries() const { return initial_tries; }
      /// Decrease attempt number
      void decrease_tries_left();
 
