@@ -9,14 +9,14 @@
 
 namespace DataStaging {
   
-  bool DTRList::add_dtr(const DTR& DTRToAdd) {
+  DTR* DTRList::add_dtr(const DTR& DTRToAdd) {
   	DTR* dtr = new DTR(DTRToAdd);
   	Lock.lock();
   	DTRs.push_back(dtr);
   	Lock.unlock();
   	
   	// Added successfully
-  	return true;
+  	return dtr;
   }
   
   bool DTRList::delete_dtr(DTR* DTRToDelete) {
@@ -57,19 +57,49 @@ namespace DataStaging {
   	return counter;
   }
   
-  bool DTRList::filter_dtrs_by_status(DTRStatus StatusToFilter, std::list<DTR*>& FilteredList){
-  	std::list<DTR*>::iterator it;
-  	
-  	Lock.lock();
-  	for(it = DTRs.begin();it != DTRs.end(); ++it)
-  	  if((*it)->get_status() == StatusToFilter)
-  	    FilteredList.push_back(*it);
-  	Lock.unlock();
-  	    
-  	// Filtered successfully
-  	return true;
+  bool DTRList::filter_dtrs_by_status(DTRStatus::DTRStatusType StatusToFilter, std::list<DTR*>& FilteredList){
+    std::vector<DTRStatus::DTRStatusType> StatusesToFilter(1, StatusToFilter);
+    return filter_dtrs_by_statuses(StatusesToFilter, FilteredList);
   }
-  
+
+  bool DTRList::filter_dtrs_by_statuses(const std::vector<DTRStatus::DTRStatusType>& StatusesToFilter,
+                                        std::list<DTR*>& FilteredList){
+    std::list<DTR*>::iterator it;
+
+    Lock.lock();
+    for(it = DTRs.begin();it != DTRs.end(); ++it) {
+      for (std::vector<DTRStatus::DTRStatusType>::const_iterator i = StatusesToFilter.begin(); i != StatusesToFilter.end(); ++i) {
+        if((*it)->get_status().GetStatus() == *i) {
+          FilteredList.push_back(*it);
+          break;
+        }
+      }
+    }
+    Lock.unlock();
+
+    // Filtered successfully
+    return true;
+  }
+
+  bool DTRList::filter_dtrs_by_statuses(const std::vector<DTRStatus::DTRStatusType>& StatusesToFilter,
+                                        std::map<DTRStatus::DTRStatusType, std::list<DTR*> >& FilteredList) {
+    std::list<DTR*>::iterator it;
+
+    Lock.lock();
+    for(it = DTRs.begin();it != DTRs.end(); ++it) {
+      for (std::vector<DTRStatus::DTRStatusType>::const_iterator i = StatusesToFilter.begin(); i != StatusesToFilter.end(); ++i) {
+        if((*it)->get_status().GetStatus() == *i) {
+          FilteredList[*i].push_back(*it);
+          break;
+        }
+      }
+    }
+    Lock.unlock();
+
+    // Filtered successfully
+    return true;
+  }
+
   bool DTRList::filter_dtrs_by_next_receiver(StagingProcesses NextReceiver, std::list<DTR*>& FilteredList) {
   	std::list<DTR*>::iterator it;
   	
@@ -133,12 +163,31 @@ namespace DataStaging {
     return true;
   }
 
-  std::list<DTR*> DTRList::all_dtrs() {
-    std::list<DTR*> allDTRs;
+  void DTRList::caching_started(DTR* request) {
+    CachingLock.lock();
+    CachingSources.insert(request->get_source_str());
+    CachingLock.unlock();
+  }
+
+  void DTRList::caching_finished(DTR* request) {
+    CachingLock.lock();
+    CachingSources.erase(request->get_source_str());
+    CachingLock.unlock();
+  }
+
+  bool DTRList::is_being_cached(DTR* DTRToCheck) {
+
+    CachingLock.lock();
+    bool caching = (CachingSources.find(DTRToCheck->get_source_str()) != CachingSources.end());
+    CachingLock.unlock();
+    return caching;
+  }
+
+  bool DTRList::empty() {
     Lock.lock();
-    allDTRs = DTRs;
+    bool empty = DTRs.empty();
     Lock.unlock();
-    return allDTRs;
+    return empty;
   }
 
   std::list<std::string> DTRList::all_jobs() {
