@@ -1,6 +1,8 @@
 #ifndef DTRLIST_H_
 #define DTRLIST_H_
 
+#include <set>
+
 #include <arc/Thread.h>
 
 #include "DTR.h"
@@ -19,12 +21,19 @@ namespace DataStaging {
   	  /// Lock to protect list during modification
       Arc::SimpleCondition Lock;
   	  
+      /// Internal set of sources that are currently being cached
+      std::set<std::string> CachingSources;
+
+      /// Lock to protect caching sources set during modification
+      Arc::SimpleCondition CachingLock;
+
   	public:
           	
   	  /// Put a new DTR into the list.
       /** A (pointer to a) copy of the DTR is added to the list, and so
-       * DTRToAdd can be deleted after this method is called. */
-  	  bool add_dtr(const DTR& DTRToAdd);
+       * DTRToAdd can be deleted after this method is called.
+       * @return The DTR pointer added to the list */
+  	  DTR* add_dtr(const DTR& DTRToAdd);
   	  
   	  /// Remove a DTR from the list.
   	  /** The DTRToDelete object is destroyed, and hence should not be
@@ -44,8 +53,19 @@ namespace DataStaging {
        * pre-, post-processor or delivery stages.
        * @param FilteredList This list is filled with filtered DTRs
        */
-      bool filter_dtrs_by_status(DTRStatus StatusToFilter, std::list<DTR*>& FilteredList);
+      bool filter_dtrs_by_status(DTRStatus::DTRStatusType StatusToFilter, std::list<DTR*>& FilteredList);
       
+      /// Filter the queue to select DTRs with particular statuses.
+      /// @param FilteredList This list is filled with filtered DTRs
+      bool filter_dtrs_by_statuses(const std::vector<DTRStatus::DTRStatusType>& StatusesToFilter,
+                                   std::list<DTR*>& FilteredList);
+
+      /// Filter the queue to select DTRs with particular statuses.
+      /// @param FilteredList This map is filled with filtered DTRs,
+      /// one list per state.
+      bool filter_dtrs_by_statuses(const std::vector<DTRStatus::DTRStatusType>& StatusesToFilter,
+                                   std::map<DTRStatus::DTRStatusType, std::list<DTR*> >& FilteredList);
+
       /// Select DTRs that are about to go to the specified process.
       /** This selection is actually a virtual queue for pre-, post-processor
        * and delivery.
@@ -64,8 +84,17 @@ namespace DataStaging {
       /// @param FilteredList This list is filled with filtered DTRs
       bool filter_dtrs_by_job(const std::string& jobid, std::list<DTR*>& FilteredList);
       
-      /// Get the list of all DTRs
-      std::list<DTR*> all_dtrs();
+      /// Update the caching set, add a DTR (only if it is CACHEABLE).
+      void caching_started(DTR* request);
+
+      /// Update the caching set, removing a DTR.
+      void caching_finished(DTR* request);
+
+      /// Returns true if the DTR's source is currently in the caching set.
+      bool is_being_cached(DTR* DTRToCheck);
+
+      /// Returns true if there are no DTRs in the list
+      bool empty();
 
       /// Get the list of all job IDs
       std::list<std::string> all_jobs();
