@@ -86,12 +86,15 @@ namespace DataStaging {
         return true;
       }
     }
-    // DTR is not in the active transfer list - log a warning
+    // DTR is not in the active transfer list, probably because it just finished
     dtr_list_lock.unlock();
     request->get_logger()->msg(Arc::WARNING, "DTR %s requested cancel but no active transfer",
                    request->get_id());
-    request->set_status(DTRStatus::TRANSFERRED);
-    request->push(SCHEDULER);
+    // if request is already TRANSFERRED, no need to push to Scheduler again
+    if (request->get_status() != DTRStatus::TRANSFERRED) {
+      request->set_status(DTRStatus::TRANSFERRED);
+      request->push(SCHEDULER);
+    }
     return true;
   }
 
@@ -140,10 +143,13 @@ namespace DataStaging {
           d = dtr_list.erase(d);
           dtr_list_lock.unlock();
 
-          dp->dtr->set_status(DTRStatus::TRANSFERRED);
-          dp->dtr->push(SCHEDULER);
           // deleting delivery_pair_t kills the spawned process
+          // Do this before passing back to Scheduler to avoid race condition
+          // of DTR being deleted before Comm object has finished with it
+          DTR* tmp = dp->dtr;
           delete dp;
+          tmp->set_status(DTRStatus::TRANSFERRED);
+          tmp->push(SCHEDULER);
           continue;
         }
         if((status.commstatus == DataDeliveryComm::CommExited) ||
