@@ -44,7 +44,8 @@ namespace DataStaging {
     // connect to service and make a new transfer request
     Arc::MCCConfig cfg;
     if (dtr.host_cert_for_remote_delivery()) {
-      Arc::UserConfig host_cfg(Arc::initializeCredentialsType(Arc::initializeCredentialsType::TryCredentials));
+      Arc::initializeCredentialsType cred_type(Arc::initializeCredentialsType::TryCredentials);
+      Arc::UserConfig host_cfg(cred_type);
       host_cfg.ProxyPath(""); // to force using cert/key files instead of non-existent proxy
       host_cfg.ApplyToConfig(cfg);
     } else {
@@ -60,7 +61,7 @@ namespace DataStaging {
 
     Arc::XMLNode dtrnode = request.NewChild("DataDeliveryStart").NewChild("DTR");
 
-    dtrnode.NewChild("ID") = dtr.get_id();
+    dtrnode.NewChild("ID") = dtr_full_id;
     dtrnode.NewChild("Source") = surl;
     dtrnode.NewChild("Destination") = durl;
     if (dtr.get_source()->CheckCheckSum()) dtrnode.NewChild("CheckSum") = dtr.get_source()->GetCheckSum();
@@ -78,7 +79,8 @@ namespace DataStaging {
     // delegate credentials
     Arc::XMLNode op = request.Child(0);
     if (!SetupDelegation(op, dtr.get_usercfg())) {
-      logger_->msg(Arc::ERROR, "DTR %s: Failed to set up credential delegation", dtr_id);
+      logger_->msg(Arc::ERROR, "DTR %s: Failed to set up credential delegation with %s",
+                   dtr_id, dtr.get_delivery_endpoint().str());
       return;
     }
 
