@@ -74,8 +74,12 @@ namespace DataStaging {
     bool is_in_cache = false;
     bool is_locked = false;
     bool use_remote = true;
+    // check for forced re-download option
+    bool renew = (request->get_source()->GetURL().Option("cache") == "renew");
+    if (renew) request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Forcing re-download of file %s", request->get_short_id(), canonic_url);
+
     for (;;) {
-      if (!cache.Start(canonic_url, is_in_cache, is_locked, use_remote)) {
+      if (!cache.Start(canonic_url, is_in_cache, is_locked, use_remote, renew)) {
         if (is_locked) {
           request->get_logger()->msg(Arc::WARNING, "DTR %s: Cached file is locked - should retry", request->get_short_id());
           request->set_cache_state(CACHE_LOCKED);
@@ -105,14 +109,6 @@ namespace DataStaging {
       }
       request->set_cache_file(cache.File(canonic_url));
       if (is_in_cache) {
-        // check for forced re-download option
-        std::string cache_option = request->get_source()->GetURL().Option("cache");
-        if (cache_option == "renew") {
-          request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Forcing re-download of file %s", request->get_short_id(), canonic_url);
-          cache.StopAndDelete(canonic_url);
-          use_remote = false;
-          continue;
-        }
         // just need to check permissions
         request->get_logger()->msg(Arc::INFO, "DTR %s: File %s is cached (%s) - checking permissions",
                    request->get_short_id(), canonic_url, cache.File(canonic_url));
@@ -124,7 +120,6 @@ namespace DataStaging {
           Arc::DataStatus cres = request->get_source()->Check();
           if (!cres.Passed()) {
             request->get_logger()->msg(Arc::ERROR, "DTR %s: Permission checking failed", request->get_short_id());
-            cache.Stop(canonic_url);
             request->set_cache_state(CACHE_SKIP);
             request->set_error_status(DTRErrorStatus::CACHE_ERROR,
                                   DTRErrorStatus::ERROR_DESTINATION,
@@ -155,9 +150,9 @@ namespace DataStaging {
             outdated = true;
         }
         if (outdated) {
-          cache.StopAndDelete(canonic_url);
           request->get_logger()->msg(Arc::INFO, "DTR %s: Cached file is outdated, will re-download", request->get_short_id());
           use_remote = false;
+          renew = true;
           continue;
         }
         // cached file is present and valid
@@ -547,7 +542,9 @@ namespace DataStaging {
                             DTRErrorStatus::ERROR_DESTINATION,
                             "Failed to create cache for " + request->get_source()->str());
       request->set_status(DTRStatus::CACHE_PROCESSED);
+      request->connect_logger();
       request->push(SCHEDULER);
+      return;
     }
 
     std::string canonic_url(request->get_source()->GetURL().plainstr());
@@ -576,14 +573,54 @@ namespace DataStaging {
     request->get_logger()->msg(Arc::INFO, "DTR %s: Linking/copying cached file to %s",
                                request->get_short_id(), request->get_destination()->CurrentLocation().Path());
 
-    if (!cache.Link(request->get_destination()->CurrentLocation().Path(), canonic_url, cache_copy, executable)) {
-      request->get_logger()->msg(Arc::ERROR, "DTR %s: Error linking cache file to %s.",
-                                 request->get_short_id(), request->get_destination()->CurrentLocation().Path());
-      request->set_error_status(DTRErrorStatus::CACHE_ERROR,
-                                DTRErrorStatus::ERROR_DESTINATION,
-                                "Failed to link/copy cache file to session dir");
+    bool was_downloaded = (request->get_cache_state() == CACHE_DOWNLOADED) ? true : false;
+    if (was_downloaded) {
+      // Add DN to cached permissions
+      Arc::Credential cred(request->get_usercfg().ProxyPath(),
+                           request->get_usercfg().ProxyPath(),
+                           request->get_usercfg().CACertificatesDirectory(), "");
+      cache.AddDN(canonic_url, cred.GetIdentityName(), cred.GetEndTime());
     }
-    cache.Stop(canonic_url);
+
+    bool try_again = false;
+    if (!cache.Link(request->get_destination()->CurrentLocation().Path(),
+                    canonic_url,
+                    cache_copy,
+                    executable,
+                    was_downloaded,
+                    try_again)) {
+      if (try_again) {
+        // go back to CACHE_CHECK - even if we went through a transfer this is
+        // ok because the cache file is unaffected by a failed linking
+        request->get_logger()->msg(Arc::WARNING, "DTR %s: Failed linking cache file to %s",
+                                   request->get_short_id(), request->get_destination()->CurrentLocation().Path());
+
+        if (was_downloaded) cache.Stop(canonic_url);
+        request->set_cache_state(CACHE_LOCKED);
+        request->set_status(DTRStatus::CACHE_WAIT);
+
+        // set a flat wait time with some randomness, fine-grained to minimise lock clashes
+        // this may change in future eg be taken from configuration or increase over time
+        time_t cache_wait_time = 10;
+        time_t randomness = (rand() % cache_wait_time) - (cache_wait_time/2);
+        cache_wait_time += randomness;
+        // add random number of milliseconds
+        uint32_t nano_randomness = (rand() % 1000) * 1000000;
+        Arc::Period cache_wait_period(cache_wait_time, nano_randomness);
+        request->get_logger()->msg(Arc::INFO, "DTR %s: Will wait around %is", request->get_short_id(), cache_wait_time);
+        request->set_process_time(cache_wait_period);
+
+        request->connect_logger();
+        request->push(SCHEDULER);
+        return;
+      }
+      request->get_logger()->msg(Arc::ERROR, "DTR %s: Error linking cache file to %s.",
+                                   request->get_short_id(), request->get_destination()->CurrentLocation().Path());
+      request->set_error_status(DTRErrorStatus::CACHE_ERROR,
+                                  DTRErrorStatus::ERROR_DESTINATION,
+                                  "Failed to link/copy cache file to session dir");
+    }
+    if (was_downloaded) cache.Stop(canonic_url);
     request->set_status(DTRStatus::CACHE_PROCESSED);
     request->connect_logger();
     request->push(SCHEDULER);
