@@ -50,24 +50,9 @@ namespace DataStaging {
       preferred_pattern = pattern;
   }
 
-  void Scheduler::SetTransferShares(const TransferShares& shares) {
+  void Scheduler::SetTransferSharesConf(const TransferSharesConf& share_conf) {
     if (scheduler_state == INITIATED)
-      transferShares = shares;
-  }
-
-  void Scheduler::AddSharePriority(const std::string& name, int priority) {
-    if (scheduler_state == INITIATED)
-      transferShares.set_reference_share(name, priority);
-  }
-
-  void Scheduler::SetSharePriorities(const std::map<std::string, int>& shares) {
-    if (scheduler_state == INITIATED)
-      transferShares.set_reference_shares(shares);
-  }
-
-  void Scheduler::SetShareType(TransferShares::ShareType share_type) {
-    if (scheduler_state == INITIATED)
-      transferShares.set_share_type(share_type);
+      transferSharesConf = share_conf;
   }
 
   void Scheduler::SetTransferParameters(const TransferParameters& params) {
@@ -95,12 +80,16 @@ namespace DataStaging {
       services.push_back(DTR::LOCAL_DELIVERY);
       delivery_services = services;
     }
-    DeliverySlots *= delivery_services.size();
-    DeliveryEmergencySlots *= delivery_services.size();
     Arc::CreateThreadFunction(&main_thread, this);
     return true;
   }
 
+  void Scheduler::log_to_root_logger(Arc::LogLevel level, const std::string& message) {
+    Arc::Logger::getRootLogger().addDestinations(root_destinations);
+    logger.msg(level, message);
+    Arc::Logger::getRootLogger().removeDestinations();
+  }
+
   /* Function to sort the list of the pointers to DTRs 
    * according to the priorities the DTRs have.
    * DTRs with higher priority go first to the beginning,
@@ -231,9 +220,7 @@ namespace DataStaging {
   }
 
   void Scheduler::ProcessDTRNEW(DTR* request){
-    // Fresh DTRs should receive the initial priority
-    // This is to be implemented
-    //compute_priority(request);
+
     request->get_logger()->msg(Arc::INFO, "Scheduler received new DTR %s with source: %s,"
         " destination: %s, assigned to transfer share %s with priority %d",
         request->get_id(), request->get_source()->str(), request->get_destination()->str(),
@@ -248,7 +235,15 @@ namespace DataStaging {
       // take a long time to download a big file
       request->set_timeout(3600);
       request->get_logger()->msg(Arc::VERBOSE, "DTR %s: File is cacheable, will check cache", request->get_short_id());
-      request->set_status(DTRStatus::CHECK_CACHE);
+      if (DtrList.is_being_cached(request)) {
+        Arc::Period cache_wait_period(10);
+        request->get_logger()->msg(Arc::VERBOSE, "DTR %s: File is currently being cached, will wait %is",
+                                   request->get_short_id(), cache_wait_period.GetPeriod());
+        request->set_process_time(cache_wait_period);
+        request->set_status(DTRStatus::CACHE_WAIT);
+      } else {
+        request->set_status(DTRStatus::CHECK_CACHE);
+      }
     }
   }
   
@@ -264,6 +259,14 @@ namespace DataStaging {
                                 "Timed out while waiting for cache for " + request->get_source()->str());
       request->get_logger()->msg(Arc::ERROR, "DTR %s: Timed out while waiting for cache lock", request->get_short_id());
       request->set_status(DTRStatus::CACHE_PROCESSED);
+    } else if (DtrList.is_being_cached(request)) {
+      // TODO A low priority DTR holding the cache lock can block a high priority DTR
+      // downloading the same file. Here we should cancel the low priority one to let
+      // the high priority one go through
+      Arc::Period cache_wait_period(10);
+      request->get_logger()->msg(Arc::VERBOSE, "DTR %s: File is currently being cached, will wait %is",
+                                 request->get_short_id(), cache_wait_period.GetPeriod());
+      request->set_process_time(cache_wait_period);
     } else {
       // Try to check cache again
       request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Checking cache again", request->get_short_id());
@@ -275,9 +278,9 @@ namespace DataStaging {
     // There's no need to check additionally for cache error
     // If the error has occurred -- we just proceed the normal
     // workflow as if it was not cached at all.
-    // But we should clear error flag if it was set by
-    // the pre-processor
+    // But we should clear error flag if it was set by the pre-processor
     request->reset_error_status();
+    if (request->get_cache_state() == CACHEABLE) DtrList.caching_started(request);
 
     if(request->get_cache_state() == CACHE_ALREADY_PRESENT){
       // File is on place already. After the post-processor
@@ -374,11 +377,8 @@ namespace DataStaging {
       // to limit per remote host. For now count staging transfers in this
       // share already in transfer queue and apply limit. In order not to block
       // the highest priority DTRs here we allow them to bypass the limit.
-      std::list<DTR*> DeliveryQueue;
-      DtrList.filter_dtrs_by_next_receiver(DELIVERY,DeliveryQueue);
-
       int share_queue = 0, highest_priority = 0;
-      for (std::list<DTR*>::iterator dtr = DeliveryQueue.begin(); dtr != DeliveryQueue.end(); ++dtr) {
+      for (std::list<DTR*>::iterator dtr = staged_queue.begin(); dtr != staged_queue.end(); ++dtr) {
         if ((*dtr)->get_transfer_share() == request->get_transfer_share() &&
             ((*dtr)->get_source()->IsStageable() ||
              (*dtr)->get_destination()->IsStageable())) {
@@ -386,8 +386,8 @@ namespace DataStaging {
           if ((*dtr)->get_priority() > highest_priority) highest_priority = (*dtr)->get_priority();
         }
       }
-      if (share_queue >= DeliverySlots*2 && request->get_priority() <= highest_priority) {
-        request->get_logger()->msg(Arc::INFO, "DTR %s: Large transfer queue - will wait 10s before staging", request->get_short_id());
+      if (share_queue >= DeliverySlots*4 && request->get_priority() <= highest_priority) {
+        request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Large transfer queue - will wait 10s before staging", request->get_short_id());
         request->set_process_time(10);
       }
       else {
@@ -395,6 +395,7 @@ namespace DataStaging {
         request->set_timeout(3600);
         // processor will take care of staging source or destination or both
         request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Source or destination requires staging", request->get_short_id());
+        staged_queue.push_back(request);
         request->set_status(DTRStatus::STAGE_PREPARE);
       }
     }
@@ -458,32 +459,25 @@ namespace DataStaging {
       }
     }
 
-    // Probably operations in the preprocessor may
-    // influence the priority in the delivery.
-    // Recomputation might be needed.
-    // compute_priority(request);
-    // After normal workflow the DTR has become ready for delivery
+    // After normal workflow the DTR is ready for delivery
     request->get_logger()->msg(Arc::VERBOSE, "DTR %s: DTR is ready for transfer, moving to delivery queue", request->get_short_id());
 
     // set long timeout for waiting for transfer slot
     // (setting timeouts for active transfers is done in Delivery)
     request->set_timeout(7200);
-    request->set_status(DTRStatus::TRANSFER_WAIT);
+    request->set_status(DTRStatus::TRANSFER);
   }
   
   void Scheduler::ProcessDTRTRANSFERRED(DTR* request){
-    // We don't do check if the error has happened
-    // If it has -- the post-processor will take needed
-    // steps in RELEASE_REQUEST in any case
-    // The error flag will work now as a sign to return
-    // the DTR to QUERY_REPLICA again
+    // We don't check if error has happened - if it has the post-processor
+    // will take needed steps in RELEASE_REQUEST in any case. The error flag
+    // will work now as a sign to return the DTR to QUERY_REPLICA again.
 
     // Delivery will clean up destination physical file on error
     if (request->error())
       request->get_logger()->msg(Arc::ERROR, "DTR %s: Transfer failed: %s", request->get_short_id(), request->get_error_status().GetDesc());
 
-    // Resuming normal workflow after the DTR
-    // has finished transferring
+    // Resuming normal workflow after the DTR has finished transferring
     // The next state is RELEASE_REQUEST
 
     // if cacheable and no cancellation or error, mark the DTR as CACHE_DOWNLOADED
@@ -547,6 +541,10 @@ namespace DataStaging {
   void Scheduler::ProcessDTRCACHE_PROCESSED(DTR* request){
     // Final stage within scheduler. Retries are initiated from here if necessary,
     // otherwise report success or failure to generator
+
+    // First remove from caching list
+    DtrList.caching_finished(request);
+
     if (request->cancel_requested()) {
       // Cancellation steps finished
       request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Cancellation complete", request->get_short_id());
@@ -554,14 +552,16 @@ namespace DataStaging {
     }
     else if(request->error()) {
       // If the error occurred in cache processing we send back
-      // to REPLICA_QUERIED to try the same replica again without cache.
-      // If there was a cache timeout we go back to CACHE_CHECKED. If in
+      // to REPLICA_QUERIED to try the same replica again without cache,
+      // or to CACHE_CHECKED if the file was already in cache. If there
+      // was a cache timeout we also go back to CACHE_CHECKED. If in
       // another place we are finished and report error to generator
       if (request->get_error_status().GetLastErrorState() == DTRStatus::PROCESSING_CACHE) {
         request->get_logger()->msg(Arc::ERROR, "DTR %s: Error in cache processing, will retry without caching", request->get_short_id());
-        request->set_cache_state(CACHE_SKIP);
         request->reset_error_status();
-        request->set_status(DTRStatus::REPLICA_QUERIED);
+        if (request->get_cache_state() == CACHE_ALREADY_PRESENT) request->set_status(DTRStatus::CACHE_CHECKED);
+        else request->set_status(DTRStatus::REPLICA_QUERIED);
+        request->set_cache_state(CACHE_SKIP);
         return;
       }
       else if (request->get_error_status().GetLastErrorState() == DTRStatus::CACHE_WAIT) {
@@ -579,7 +579,9 @@ namespace DataStaging {
             request->get_error_status().GetErrorStatus() == DTRErrorStatus::TRANSFER_SPEED_ERROR ||
             request->get_error_status().GetErrorStatus() == DTRErrorStatus::INTERNAL_PROCESS_ERROR) {
           if (request->get_tries_left() > 0) {
-            request->set_process_time(10);
+            // exponential back off - 10s, 40s, 90s, ...
+            request->set_process_time(10*(request->get_initial_tries()-request->get_tries_left())*
+                                         (request->get_initial_tries()-request->get_tries_left()));
             request->get_logger()->msg(Arc::INFO, "DTR %s: %i retries left, will wait until %s before next attempt",
                                        request->get_short_id(), request->get_tries_left(), request->get_process_time().str());
             // set state depending on where the error occurred
@@ -610,21 +612,19 @@ namespace DataStaging {
   }
   
   void Scheduler::ProcessDTRFINAL_STATE(DTR* request){
-  	/* The only place where the DTR is returned to the generator 
-  	 * and deleted from the global list
-  	 */
+  	// This is the only place where the DTR is returned to the generator
+  	// and deleted from the global list
+
   	// Return to the generator
     request->get_logger()->msg(Arc::INFO, "DTR %s: Returning to generator", request->get_short_id());
     request->push(GENERATOR);
-    // Decrease the corresponding transfer share
-    transferShares.decrease_transfer_share(request->get_transfer_share());
     // Delete from the global list
     DtrList.delete_dtr(request);
   }
   
   void Scheduler::map_state_and_process(DTR* request){
-    // DTRs that were requested to be cancelled are processed not here
-    if(request->cancel_requested()) map_cancel_state_and_process(request);
+    // For cancelled DTRs set the appropriate post-processor state
+    if(request->cancel_requested()) map_cancel_state(request);
     // Loop until the DTR is sent somewhere for some action to be done
     // This is more efficient because many DTRs will skip some states and
     // we don't want to have to wait for the full list to be processed before
@@ -652,24 +652,20 @@ namespace DataStaging {
         default: ; //DoNothing
       }
     }
-    if (request->is_in_final_state()) {
-      // If we came here -- we were in the final state,
-      // so the DTR is returned to the generator and deleted
-      ProcessDTRFINAL_STATE(request);
-    }
+
   }
   
-  void Scheduler::map_cancel_state_and_process(DTR* request){
+  void Scheduler::map_cancel_state(DTR* request){
     switch (request->get_status().GetStatus()) {
       case DTRStatus::NEW:
       case DTRStatus::CHECK_CACHE:
+      case DTRStatus::CACHE_WAIT:
         {
           // Nothing has yet been done to require cleanup or additional
           // activities. Return to the generator via CACHE_PROCESSED.
           request->set_status(DTRStatus::CACHE_PROCESSED);
         }
         break;
-      case DTRStatus::CACHE_WAIT:
       case DTRStatus::CACHE_CHECKED:
       case DTRStatus::RESOLVE:
         {
@@ -693,7 +689,6 @@ namespace DataStaging {
         break;
       case DTRStatus::STAGING_PREPARING_WAIT:
       case DTRStatus::STAGED_PREPARED:
-      case DTRStatus::TRANSFER_WAIT:
       case DTRStatus::TRANSFER:
         {
           // At this stage we in addition to cache work
@@ -724,275 +719,231 @@ namespace DataStaging {
     }
   }
   
+  void Scheduler::add_event(DTR* event) {
+    event_lock.lock();
+    events.push_back(event);
+    event_lock.unlock();
+  }
+
   void Scheduler::process_events(void){
-    std::list<DTR*> Events;
     
-    // Get the events
-    DtrList.filter_pending_dtrs(Events);
-    
-    std::list<DTR*>::iterator Event;
-    for(Event = Events.begin(); Event != Events.end(); Event++){
-      map_state_and_process(*Event);
-    }
-  }
+    // Get all the DTRs in a staged state
+    staged_queue.clear();
+    DtrList.filter_dtrs_by_statuses(DTRStatus::StagedStates, staged_queue);
 
-  void Scheduler::revise_pre_processor_queue()
-  {
-    std::list<DTR*> PreProcessorQueue;
-    DtrList.filter_dtrs_by_next_receiver(PRE_PROCESSOR,PreProcessorQueue);
+    Arc::Time now;
+    event_lock.lock();
 
-    if (PreProcessorQueue.empty()) return;
+    for (std::list<DTR*>::iterator event = events.begin(); event != events.end();) {
+      DTR* tmp = *event;
+      event_lock.unlock();
 
-    // Sort the queue by priority
-    PreProcessorQueue.sort(dtr_sort_predicate);
-
-    DTR* tmp;
-
-    std::list<DTR*>::iterator dtr = PreProcessorQueue.begin();
-    int highest_priority = (*dtr)->get_priority();
-
-    while (dtr != PreProcessorQueue.end()) {
-
-      tmp = *dtr;
-      // The cancellation requests break the normal workflow. A cancelled
-      // request will either go back to generator or be put into a
-      // post-processor state for clean up
-      if(tmp->cancel_requested()){
-        map_cancel_state_and_process(tmp);
-        dtr = PreProcessorQueue.erase(dtr);
-        continue;
-      }
-
-      // To avoid the situation where DTRs get blocked due to higher
-      // priority DTRs, DTRs that have passed their timeout should have their
-      // priority boosted. But this should only happen if there are higher
-      // priority DTRs, since there could be a large queue of low priority DTRs
-      // which, after having their priority boosted, would then block new
-      // high priority requests.
-      // The simple solution here is to increase priority by 1 every 5 minutes.
-      // There is plenty of scope for more intelligent solutions.
-      // TODO reset priority back to original value after pre-processing
-      if(tmp->get_timeout() < time(NULL) && tmp->get_priority() < highest_priority){
-        tmp->set_priority(tmp->get_priority() + 1);
-        tmp->set_timeout(300);
-      }
-      ++dtr;
-    }
-
-    transferShares.calculate_shares(PreProcessorSlots);
-
-    std::list<DTR*> InPreProcessor;
-    DtrList.filter_dtrs_by_owner(PRE_PROCESSOR, InPreProcessor);
-
-    // Number of the DTRs running in the pre-processor
-    int PreProcessorRunning = InPreProcessor.size();
-    if (PreProcessorRunning == PreProcessorSlots) return;
-
-    // Decrease shares for those already in the pre-processor
-    for (dtr = InPreProcessor.begin(); dtr != InPreProcessor.end(); ++dtr) {
-      transferShares.decrease_number_of_slots((*dtr)->get_transfer_share());
-    }
-
-    // Send to pre-processor according to share and place in the queue
-    while(PreProcessorRunning < PreProcessorSlots && !PreProcessorQueue.empty()){
-      tmp = PreProcessorQueue.front();
-      PreProcessorQueue.pop_front();
-      if (transferShares.can_start(tmp->get_transfer_share())) {
-        tmp->push(PRE_PROCESSOR);
-        PreProcessorRunning++;
-        transferShares.decrease_number_of_slots(tmp->get_transfer_share());
-      }
-    }
-  }
-  
-  void Scheduler::revise_post_processor_queue()
-  {
-    // There's no check for cancellation requests.
-    // The post-processor is special. Most DTRs
-    // with cancellation requests will go to the
-    // post-processor for cleanups, hold releases,
-    // etc., so the cancellation requests don't break
-    // normal workflow in the post-processor (as opposed
-    // to any other process), but instead act just as a
-    // sign that the post-processor should do additional
-    // cleanup activities.
-
-    std::list<DTR*> PostProcessorQueue;
-    DtrList.filter_dtrs_by_next_receiver(POST_PROCESSOR,PostProcessorQueue);
-
-    if (PostProcessorQueue.empty()) return;
-
-    // Sort the queue by priority
-    PostProcessorQueue.sort(dtr_sort_predicate);
-
-    DTR* tmp;
-
-    std::list<DTR*>::iterator dtr = PostProcessorQueue.begin();
-    int highest_priority = (*dtr)->get_priority();
-
-    while (dtr != PostProcessorQueue.end()) {
-
-      tmp = *dtr;
-
-      // see revise_pre_processor_queue() for explanation
-      if(tmp->get_timeout() < time(NULL) && tmp->get_priority() < highest_priority){
-        tmp->set_priority(tmp->get_priority() + 1);
-        tmp->set_timeout(300);
-      }
-      ++dtr;
-    }
-
-    transferShares.calculate_shares(PostProcessorSlots);
-
-    std::list<DTR*> InPostProcessor;
-    DtrList.filter_dtrs_by_owner(POST_PROCESSOR, InPostProcessor);
-
-    // Number of the DTRs running in the post-processor
-    int PostProcessorRunning = InPostProcessor.size();
-    if (PostProcessorRunning == PostProcessorSlots) return;
-
-    // Decrease shares for those already in the post-processor
-    for (dtr = InPostProcessor.begin(); dtr != InPostProcessor.end(); ++dtr) {
-      transferShares.decrease_number_of_slots((*dtr)->get_transfer_share());
-    }
-
-    // Send to pre-processor according to share and place in the queue
-    while(PostProcessorRunning < PostProcessorSlots && !PostProcessorQueue.empty()){
-      tmp = PostProcessorQueue.front();
-      PostProcessorQueue.pop_front();
-      if (transferShares.can_start(tmp->get_transfer_share())) {
-        tmp->push(POST_PROCESSOR);
-        PostProcessorRunning++;
-        transferShares.decrease_number_of_slots(tmp->get_transfer_share());
-      }
-    }
-  }
-  
-  void Scheduler::revise_delivery_queue()
-  {
-    std::list<DTR*> DeliveryQueue;    
-    DtrList.filter_dtrs_by_next_receiver(DELIVERY,DeliveryQueue);
-
-    // Sort the Delivery Queue according to
-    // the priorities the DTRs have.
-    DeliveryQueue.sort(dtr_sort_predicate);
-
-    DTR* tmp;
-    int highest_priority = 0;
-
-    std::list<DTR*>::iterator dtr = DeliveryQueue.begin();
-
-    while (dtr != DeliveryQueue.end()) {
-      if (dtr == DeliveryQueue.begin()) highest_priority = (*dtr)->get_priority();
-
-      tmp = *dtr;
-      // The cancellation requests break the normal workflow. A cancelled
-      // request will either go back to generator or be put into a
-      // post-processor state for clean up
-      if(tmp->cancel_requested()){
-        map_cancel_state_and_process(tmp);
-        dtr = DeliveryQueue.erase(dtr);
-        continue;
-      }
-
-      // see revise_pre_processor_queue() for explanation
-      if(tmp->get_timeout() < time(NULL) && tmp->get_priority() < highest_priority){
-        tmp->set_priority(tmp->get_priority() + 1);
-        tmp->set_timeout(300);
-      }
-      dtr++;
-    }
-
-    transferShares.calculate_shares(DeliverySlots);
-    
-    // Shares which have at least one DTR in Delivery
-    // Shares can only use emergency slots if they are not in this list
-    std::set<std::string> shares_in_delivery;
-
-    // The shares are re-calculated. Now we have to determine
-    // how many slots every share has already grabbed.
-    {
-      std::list<DTR*> shareDeliveryQueue;
-      std::list<DTR*>::iterator sharedtr;
-      DTR* sharetmp;
-
-      DtrList.filter_dtrs_by_owner(DELIVERY,shareDeliveryQueue);
-    	
-      for(sharedtr = shareDeliveryQueue.begin(); sharedtr != shareDeliveryQueue.end(); sharedtr++){
-        sharetmp = *sharedtr;
-        // First check for cancellation - send cancellation call to Delivery
-        // and don't count as an active slot
-        if (sharetmp->cancel_requested()) {
-          // check if already cancelled
-          if (sharetmp->get_status() != DTRStatus::TRANSFERRING_CANCEL) {
-            sharetmp->get_logger()->msg(Arc::INFO, "DTR %s: Cancelling active transfer", sharetmp->get_short_id());
-            delivery.cancelDTR(sharetmp);
-          }
+      if (tmp->get_process_time() <= now) {
+        map_state_and_process(tmp);
+        // If final state, the DTR is returned to the generator and deleted
+        if (tmp->is_in_final_state()) {
+          ProcessDTRFINAL_STATE(tmp);
+          event_lock.lock();
+          event = events.erase(event);
+          continue;
+        }
+        // If the event was sent on to a queue, erase it from the list
+        if (tmp->is_destined_for_pre_processor() ||
+            tmp->is_destined_for_delivery() ||
+            tmp->is_destined_for_post_processor()) {
+          event_lock.lock();
+          event = events.erase(event);
           continue;
         }
-        // Every active DTR for sure has its share represented
-        // in active shares. So we can just decrease the corresponding
-        // number
-        transferShares.decrease_number_of_slots(sharetmp->get_transfer_share());
-        shares_in_delivery.insert(sharetmp->get_transfer_share());
       }
+      event_lock.lock();
+      ++event;
     }
-    
-    // Refresh the number of DTRs running in the Delivery,
-    int DeliveryRunning = DtrList.number_of_dtrs_by_owner(DELIVERY);
+    event_lock.unlock();
+  }
 
-    // Now at the beginning of the queue we have DTRs that
-    // should be launched first. Launch them, but with respect
-    // to the transfer shares.
-    for(dtr = DeliveryQueue.begin(); dtr != DeliveryQueue.end(); dtr++){
-      tmp = *dtr;
-      // Limit reached - check if emergency slots are needed for any shares
-      // in the queue but not already running
-      if (DeliveryRunning >= DeliverySlots) {
-        if (DeliveryRunning == DeliverySlots + DeliveryEmergencySlots)
-          break;
-        if ((shares_in_delivery.find(tmp->get_transfer_share()) == shares_in_delivery.end()) &&
-            transferShares.can_start(tmp->get_transfer_share())) {
-          // choose delivery service - random for now
-          // if this is a retry, try to use a different service
-          if (!delivery_services.empty()) {
-            tmp->set_delivery_endpoint(delivery_services.at(rand() % delivery_services.size()));
+  void Scheduler::revise_queues() {
+
+    // The DTRs ready to go into a processing state
+    std::map<DTRStatus::DTRStatusType, std::list<DTR*> > DTRQueueStates;
+    DtrList.filter_dtrs_by_statuses(DTRStatus::ToProcessStates, DTRQueueStates);
+
+    // The active DTRs currently in processing states
+    std::map<DTRStatus::DTRStatusType, std::list<DTR*> > DTRRunningStates;
+    DtrList.filter_dtrs_by_statuses(DTRStatus::ProcessingStates, DTRRunningStates);
+
+    // Go through "to process" states, work out shares and push DTRs
+    for (unsigned int i = 0; i < DTRStatus::ToProcessStates.size(); ++i) {
+
+      std::list<DTR*> DTRQueue = DTRQueueStates[DTRStatus::ToProcessStates.at(i)];
+      std::list<DTR*> ActiveDTRs = DTRRunningStates[DTRStatus::ProcessingStates.at(i)];
+
+      if (DTRQueue.empty() && ActiveDTRs.empty()) continue;
+
+      // Transfer shares for this queue
+      TransferShares transferShares(transferSharesConf);
+
+      // Sort the DTR queue according to the priorities the DTRs have.
+      // Highest priority will be at the beginning of the list.
+      DTRQueue.sort(dtr_sort_predicate);
+
+      int highest_priority = 0;
+
+      // First go over the queue and check for cancellation and timeout
+      for (std::list<DTR*>::iterator dtr = DTRQueue.begin(); dtr != DTRQueue.end();) {
+
+        DTR* tmp = *dtr;
+        if (dtr == DTRQueue.begin()) highest_priority = tmp->get_priority();
+
+        // There's no check for cancellation requests for the post-processor.
+        // Most DTRs with cancellation requests will go to the post-processor
+        // for cleanups, hold releases, etc., so the cancellation requests
+        // don't break normal workflow in the post-processor (as opposed
+        // to any other process), but instead act just as a sign that the
+        // post-processor should do additional cleanup activities.
+        if (tmp->is_destined_for_pre_processor() || tmp->is_destined_for_delivery()) {
+
+          // The cancellation requests break the normal workflow. A cancelled
+          // request will either go back to generator or be put into a
+          // post-processor state for clean up.
+          if (tmp->cancel_requested()) {
+            map_cancel_state(tmp);
+            add_event(tmp);
+            dtr = DTRQueue.erase(dtr);
+            continue;
+          }
+        }
+        // To avoid the situation where DTRs get blocked due to higher
+        // priority DTRs, DTRs that have passed their timeout should have their
+        // priority boosted. But this should only happen if there are higher
+        // priority DTRs, since there could be a large queue of low priority DTRs
+        // which, after having their priority boosted, would then block new
+        // high priority requests.
+        // The simple solution here is to increase priority by 1 every 5 minutes.
+        // There is plenty of scope for more intelligent solutions.
+        // TODO reset priority back to original value once past this stage.
+        if (tmp->get_timeout() < time(NULL) && tmp->get_priority() < highest_priority) {
+          tmp->set_priority(tmp->get_priority() + 1);
+          tmp->set_timeout(300);
+        }
+        transferShares.increase_transfer_share(tmp->get_transfer_share());
+        ++dtr;
+      }
+
+      // Go over the active DTRs and add to transfer share
+      for (std::list<DTR*>::iterator dtr = ActiveDTRs.begin(); dtr != ActiveDTRs.end();) {
+
+        DTR* tmp = *dtr;
+        // If the DTR is in Delivery, check for cancellation. The pre- and
+        // post-processor DTRs don't get cancelled here but are allowed to
+        // continue processing.
+        if (tmp->get_status() == DTRStatus::TRANSFERRING && tmp->cancel_requested()) {
+          tmp->get_logger()->msg(Arc::INFO, "DTR %s: Cancelling active transfer", tmp->get_short_id());
+          delivery.cancelDTR(tmp);
+          dtr = ActiveDTRs.erase(dtr);
+          continue;
+        }
+        transferShares.increase_transfer_share((*dtr)->get_transfer_share());
+        ++dtr;
+      }
+
+      // If the queue is empty we can go straight to the next state
+      if (DTRQueue.empty()) continue;
+
+      // Slot limit for this state
+      int slot_limit = DeliverySlots;
+      if (DTRQueue.front()->is_destined_for_pre_processor()) slot_limit = PreProcessorSlots;
+      else if (DTRQueue.front()->is_destined_for_post_processor()) slot_limit = PostProcessorSlots;
+
+      // Calculate the slots available for each active share
+      transferShares.calculate_shares(slot_limit);
+
+      // Shares which have at least one DTR active and running.
+      // Shares can only use emergency slots if they are not in this list.
+      std::set<std::string> active_shares;
+      int running = ActiveDTRs.size();
+
+      // Go over the active DTRs again and decrease slots in corresponding shares
+      for (std::list<DTR*>::iterator dtr = ActiveDTRs.begin(); dtr != ActiveDTRs.end(); ++dtr) {
+        transferShares.decrease_number_of_slots((*dtr)->get_transfer_share());
+        active_shares.insert((*dtr)->get_transfer_share());
+      }
+
+      // Now at the beginning of the queue we have DTRs that should be
+      // launched first. Launch them, but with respect to the transfer shares.
+      for (std::list<DTR*>::iterator dtr = DTRQueue.begin(); dtr != DTRQueue.end(); ++dtr) {
+
+        DTR* tmp = *dtr;
+
+        // Check if there are any shares left in the queue which might need
+        // an emergency share - if not we are done
+        if (running >= slot_limit &&
+            transferShares.active_shares().size() == active_shares.size()) break;
+
+        // Are there slots left for this share?
+        bool can_start = transferShares.can_start(tmp->get_transfer_share());
+        // Check if it is possible to use an emergency share
+        if (running >= slot_limit &&
+            active_shares.find(tmp->get_transfer_share()) != active_shares.end()) {
+          can_start = false;
+        }
+
+        if (can_start) {
+
+          // If going into Delivery, choose delivery service - random for now
+          if (tmp->is_destined_for_delivery() && !delivery_services.empty()) {
+            // previous endpoint
+            Arc::URL delivery_endpoint(tmp->get_delivery_endpoint());
+            // If this is a retry, use a different service
+            if (tmp->get_tries_left() < tmp->get_initial_tries() && delivery_services.size() > 1) {
+              Arc::URL ep(delivery_endpoint);
+              // Find a random service different from the previous one, looping a
+              // limited number of times in case all delivery_services are the same url
+              for (unsigned int i = 0; ep == delivery_endpoint && i < delivery_services.size() * 10; ++i) {
+                ep = delivery_services.at(rand() % delivery_services.size());
+              }
+              delivery_endpoint = ep;
+            } else {
+              delivery_endpoint = delivery_services.at(rand() % delivery_services.size());
+            }
+            tmp->set_delivery_endpoint(delivery_endpoint);
           }
           transferShares.decrease_number_of_slots(tmp->get_transfer_share());
-          tmp->set_status(DTRStatus::TRANSFER);
-          tmp->push(DELIVERY);
-          DeliveryRunning++;
-          shares_in_delivery.insert(tmp->get_transfer_share());
+
+          // Send to processor/delivery
+          if (tmp->is_destined_for_pre_processor()) tmp->push(PRE_PROCESSOR);
+          else if (tmp->is_destined_for_delivery()) tmp->push(DELIVERY);
+          else if (tmp->is_destined_for_post_processor()) tmp->push(POST_PROCESSOR);
+
+          ++running;
+          active_shares.insert(tmp->get_transfer_share());
         }
-      }
-      else if(transferShares.can_start(tmp->get_transfer_share())){
-        // choose delivery service - random for now
-        // if this is a retry, try to use a different service
-        if (!delivery_services.empty()) {
-          tmp->set_delivery_endpoint(delivery_services.at(rand() % delivery_services.size()));
-        }
-      	transferShares.decrease_number_of_slots(tmp->get_transfer_share());
-        tmp->set_status(DTRStatus::TRANSFER);
-        tmp->push(DELIVERY);
-        DeliveryRunning++;
-        shares_in_delivery.insert(tmp->get_transfer_share());
+        // Hard limit with all emergency slots used
+        if (running == slot_limit + DeliveryEmergencySlots) break;
       }
     }
-    
   }
 
   void Scheduler::receiveDTR(DTR& request){
-    if(request.get_status() != DTRStatus::NEW) {
-       // If DTR is not NEW scheduler will pick it up itself.
-       return;
+
+    if (request.get_status() != DTRStatus::NEW) {
+      add_event(&request);
+      return;
     }
-    
+    // New DTR - first check it is valid
+    if (!request) {
+      logger.msg(Arc::ERROR, "Scheduler received invalid DTR");
+      request.set_status(DTRStatus::ERROR);
+      request.push(GENERATOR);
+      return;
+    }
+
     request.registerCallback(&processor,PRE_PROCESSOR);
     request.registerCallback(&processor,POST_PROCESSOR);
     request.registerCallback(&delivery,DELIVERY);
     /* Shares part*/
     // First, get the transfer share this dtr should belong to
-    std::string DtrTransferShare = transferShares.extract_share_info(request);
+    std::string DtrTransferShare = transferSharesConf.extract_share_info(request);
 
     // If no share information could be obtained, use default share
     if (DtrTransferShare.empty())
@@ -1000,30 +951,27 @@ namespace DataStaging {
 
     // If this share is a reference share, we have to add the sub-share
     // to the reference list
-    bool in_reference = transferShares.is_configured(DtrTransferShare);
-    int priority = transferShares.get_basic_priority(DtrTransferShare);
+    bool in_reference = transferSharesConf.is_configured(DtrTransferShare);
+    int priority = transferSharesConf.get_basic_priority(DtrTransferShare);
 
     request.set_transfer_share(DtrTransferShare);
     DtrTransferShare = request.get_transfer_share();
 
     // Now the sub-share is added to DtrTransferShare, add it to reference
-    // shares if appropriate
-    if (in_reference && !transferShares.is_configured(DtrTransferShare))
-      transferShares.set_reference_share(DtrTransferShare, priority);
-        
-    // Increase the number of DTRs belonging to this share
-    transferShares.increase_transfer_share(DtrTransferShare);
+    // shares if appropriate and update each TransferShare
+    if (in_reference && !transferSharesConf.is_configured(DtrTransferShare)) {
+      transferSharesConf.set_reference_share(DtrTransferShare, priority);
+    }
     
-    // Compute the priority this DTR receives
-    // This is the priority of the share adjusted by the priority
-    // of the parent job
-    request.set_priority(int(transferShares.get_basic_priority(DtrTransferShare) * request.get_priority() * 0.01));
+    // Compute the priority this DTR receives - this is the priority of the
+    // share adjusted by the priority of the parent job
+    request.set_priority(int(transferSharesConf.get_basic_priority(DtrTransferShare) * request.get_priority() * 0.01));
     /* Shares part ends*/               
-    
-    DtrList.add_dtr(request);
-    
-    // Accepted successfully
-    return;
+
+    DTR * new_dtr = DtrList.add_dtr(request);
+    if (new_dtr) {
+      add_event(new_dtr);
+    }
   }
 
   bool Scheduler::cancelDTRs(const std::string& jobid) {
@@ -1063,20 +1011,20 @@ namespace DataStaging {
     logger.msg(Arc::INFO, "  Delivery slots: %i", DeliverySlots);
     logger.msg(Arc::INFO, "  Emergency Delivery slots: %i", DeliveryEmergencySlots);
     logger.msg(Arc::INFO, "  Post-processor slots: %i", PostProcessorSlots);
-    logger.msg(Arc::INFO, "  Shares configuration:\n%s", transferShares.conf());
+    logger.msg(Arc::INFO, "  Shares configuration:\n%s", transferSharesConf.conf());
     for (std::vector<Arc::URL>::iterator i = delivery_services.begin(); i != delivery_services.end(); ++i) {
       if (*i == DTR::LOCAL_DELIVERY) logger.msg(Arc::INFO, "  Delivery service: LOCAL");
       else logger.msg(Arc::INFO, "  Delivery service: %s", i->str());
     }
 
-    // disconnect from root logger
-    // messages are logged to per-DTR Logger
+    // Disconnect from root logger so that messages are logged to per-DTR Logger
     Arc::Logger::getRootLogger().setThreadContext();
+    root_destinations = Arc::Logger::getRootLogger().getDestinations();
     Arc::Logger::getRootLogger().removeDestinations();
 
     // flag to say when to dump state
     bool dump = true;
-    while(scheduler_state != TO_STOP || !DtrList.all_dtrs().empty()) {
+    while(scheduler_state != TO_STOP || !DtrList.empty()) {
       // first check for cancelled jobs
       cancelled_jobs_lock.lock();
       std::list<std::string>::iterator jobid = cancelled_jobs.begin();
@@ -1094,19 +1042,8 @@ namespace DataStaging {
       // Dealing with pending events, i.e. DTRs from another processes
       process_events();
       // Revise all the internal queues and take actions
-      revise_pre_processor_queue();
-      revise_delivery_queue();
-      revise_post_processor_queue();
-      // log states for debugging
-      std::list<DTR*> DeliveryQueue;
-      DtrList.filter_dtrs_by_next_receiver(DELIVERY,DeliveryQueue);
-      // this logger is now disconnected, but in future this information will
-      // be available in other ways
-      logger.msg(Arc::DEBUG, "Pre-processor %i, DeliveryQueue %i, Delivery %i, Post-processor %i",
-                             DtrList.number_of_dtrs_by_owner(PRE_PROCESSOR),
-                             DeliveryQueue.size(),
-                             DtrList.number_of_dtrs_by_owner(DELIVERY),
-                             DtrList.number_of_dtrs_by_owner(POST_PROCESSOR));
+      revise_queues();
+
       // every 5 seconds, dump state
       if (!dumplocation.empty() && Arc::Time().GetTime() % 5 == 0) {
         if (dump) {
@@ -1118,7 +1055,7 @@ namespace DataStaging {
       Glib::usleep(50000);
     }
     DtrList.dumpState(dumplocation);
-    logger.msg(Arc::INFO, "Scheduler loop exited");
+    log_to_root_logger(Arc::INFO, "Scheduler loop exited");
     run_signal.signal();
   }
   
