@@ -34,11 +34,17 @@ class Scheduler: public DTRCallback {
      * processes it during the main loop. */
     std::list<std::string> cancelled_jobs;
 
+    /// A list of DTRs to process
+    std::list<DTR*> events;
+
+    /// List of staged DTRs. Filled each event processing loop
+    std::list<DTR*> staged_queue;
+
     /// A lock for the cancelled jobs list
     Arc::SimpleCondition cancelled_jobs_lock;
 
-    /// Transfer shares the scheduler uses
-    TransferShares transferShares;
+    /// Configuration of transfer shares
+    TransferSharesConf transferSharesConf;
 
     /// URLMap containing information on any local mappings defined in the configuration
     Arc::URLMap url_map;
@@ -70,6 +76,9 @@ class Scheduler: public DTRCallback {
     /// Logger object
     static Arc::Logger logger;
 
+    /// Root logger destinations, to use when logging non-DTR specific messages
+    std::list<Arc::LogDestination*> root_destinations;
+
     /// Flag describing scheduler state. Used to decide whether to keep running main loop.
     ProcessState scheduler_state;
 
@@ -123,23 +132,26 @@ class Scheduler: public DTRCallback {
      */ 
     //void compute_priority(DTR* request);
     
+    /// Log a message to the root logger. This sends the message to the log
+    /// destinations attached to the root logger at the point the Scheduler
+    /// was started.
+    void log_to_root_logger(Arc::LogLevel level, const std::string& message);
+
     /// Call the appropriate Process method depending on the DTR state
     void map_state_and_process(DTR* request);
     
-    /// Call the appropriate Process method when the DTR has a pending cancel request.
+    /// Maps the DTR to the appropriate state when it is cancelled.
     /** This is a separate function, since cancellation request
      * can arrive at any time, breaking the normal workflow. */
-    void map_cancel_state_and_process(DTR* request);
+    void map_cancel_state(DTR* request);
     
-    /* Functions to loop through DTRs awaiting sending to proper processes. */
+    /// Go through all DTRs waiting to go into a processing state and decide
+    /// whether to push them into that state, depending on shares and limits.
+    void revise_queues();
+
+    /// Add a new event for the Scheduler to process. Used in receiveDTR().
+    void add_event(DTR* event);
 
-    /// Go through the list of DTRs waiting to go to the pre-processor and possibly submit them
-    void revise_pre_processor_queue();
-    /// Go through the list of DTRs waiting to go to the post-processor and possibly submit them
-    void revise_post_processor_queue();
-    /// Go through the list of DTRs waiting to go to the delivery and possibly submit them
-    void revise_delivery_queue();
-    
     /// Process the pool of DTRs which have arrived from other processes
     void process_events(void);
     
@@ -188,17 +200,8 @@ class Scheduler: public DTRCallback {
     /// Set the preferred pattern
     void SetPreferredPattern(const std::string& pattern);
 
-    /// Set TransferShares
-    void SetTransferShares(const TransferShares& shares);
-
-    /// Add share
-    void AddSharePriority(const std::string& name, int priority);
-
-    /// Replace all shares
-    void SetSharePriorities(const std::map<std::string, int>& shares);
-
-    /// Set share type
-    void SetShareType(TransferShares::ShareType share_type);
+    /// Set TransferShares configuration
+    void SetTransferSharesConf(const TransferSharesConf& share_conf);
 
     /// Set transfer limits
     void SetTransferParameters(const TransferParameters& params);
