@@ -82,13 +82,13 @@ Arc::MCC_Status ARexService::ChangeActivityStatus(ARexGMConfig& config,Arc::XMLN
   std::string new_arex_state = new_state["a-rex:state"];
   // Take renewed proxy if supplied
   std::string delegation;
-  Arc::XMLNode delegated_token = new_state["deleg:DelegatedToken"];
+  Arc::XMLNode delegated_token = new_state["arcdeleg:DelegatedToken"];
   if(delegated_token) {
     if(!delegations_.DelegatedToken(delegation,delegated_token,config.GridName())) {
       // Failed to accept delegation (report as bad request)
       logger_.msg(Arc::ERROR, "ChangeActivityStatus: Failed to accept delegation");
       Arc::SOAPFault fault(out.Parent(),Arc::SOAPFault::Sender,"Failed to accept delegation");
-      InvalidRequestMessageFault(fault,"deleg:DelegatedToken","This token does not exist");
+      InvalidRequestMessageFault(fault,"arcdeleg:DelegatedToken","This token does not exist");
       out.Destroy();
       return Arc::MCC_Status();
     };
@@ -210,12 +210,12 @@ Arc::MCC_Status ARexService::ESWipeActivity(ARexGMConfig& config,Arc::XMLNode in
     ARexJob job(jobid,config,logger_);
     if(!job) {
       // There is no such job
-      logger_.msg(Arc::ERROR, "ESEMI:WipeActivity: job %s - %s", jobid, job.Failure());
+      logger_.msg(Arc::ERROR, "EMIES:WipeActivity: job %s - %s", jobid, job.Failure());
       ESUnknownActivityIDFault(item.NewChild("dummy"),job.Failure());
     } else {
       if(!job.Clean()) {
         // Probably wrong current state
-        logger_.msg(Arc::ERROR, "ESEMI:WipeActivity: job %s - %s", jobid, job.Failure());
+        logger_.msg(Arc::ERROR, "EMIES:WipeActivity: job %s - %s", jobid, job.Failure());
         // TODO: check ffor real reason
         ESInvalidActivityStateFault(item.NewChild("dummy"),job.Failure());
       } else {
