@@ -122,7 +122,7 @@ Arc::MCC_Status ARexService::ESGetActivityStatus(ARexGMConfig& config,Arc::XMLNo
     ARexJob job(jobid,config,logger_);
     if(!job) {
       // There is no such job
-      logger_.msg(Arc::ERROR, "ESEMI:GetActivityStatus: job %s - %s", jobid, job.Failure());
+      logger_.msg(Arc::ERROR, "EMIES:GetActivityStatus: job %s - %s", jobid, job.Failure());
       ESUnknownActivityIDFault(item.NewChild("dummy"),job.Failure());
     } else {
       bool job_pending = false;
@@ -145,7 +145,7 @@ Arc::MCC_Status ARexService::ESGetActivityInfo(ARexGMConfig& config,Arc::XMLNode
     ARexJob job(jobid,config,logger_);
     if(!job) {
       // There is no such job
-      logger_.msg(Arc::ERROR, "ESEMI:GetActivityStatus: job %s - %s", jobid, job.Failure());
+      logger_.msg(Arc::ERROR, "EMIES:GetActivityStatus: job %s - %s", jobid, job.Failure());
       ESUnknownActivityIDFault(item.NewChild("dummy"),job.Failure());
     } else {
       std::string glue_s;
@@ -167,7 +167,7 @@ Arc::MCC_Status ARexService::ESGetActivityInfo(ARexGMConfig& config,Arc::XMLNode
         };
       };
       if(!info) {
-        logger_.msg(Arc::ERROR, "ESEMI:GetActivityStatus: job %s - failed to retrieve Glue2 information", jobid);
+        logger_.msg(Arc::ERROR, "EMIES:GetActivityStatus: job %s - failed to retrieve Glue2 information", jobid);
         ESInternalBaseFault(item.NewChild("dummy"),"failed to retrieve Glue2 information");
       };
     };
