@@ -124,6 +124,17 @@ bool configure_serviced_users(JobUsers &users,uid_t my_uid,const std::string &my
         break;
       };
     };
+    if(command == "voms_processing") {
+      std::string voms_processing = config_next_arg(rest);
+      //Since the voms_processing could not only be used by gridmanager,
+      //but also used by other services (e.g., gridftp service),
+      //the value is set as environment variables.
+      Arc::SetEnv("VOMS_PROCESSING", voms_processing.c_str());
+    }
+    if(command == "voms_trust_chains") {
+      std::string voms_trust_chains = config_next_arg(rest);
+      Arc::SetEnv("VOMS_TRUST_CHAINS", voms_trust_chains.c_str());
+    }
     if(command == "runtimedir") {
       users.Env().runtime_config_dir(rest);
     } else if(command == "joblog") { /* where to write job inforamtion */
@@ -194,7 +205,7 @@ bool configure_serviced_users(JobUsers &users,uid_t my_uid,const std::string &my
     else if(command == "maxload") { /* maximum number of the jobs processed on frontend */
       std::string max_jobs_s = config_next_arg(rest);
       long int i;
-      int max_jobs_processing, max_jobs_processing_emergency, max_downloads = -1;
+      int max_jobs_processing = -1, max_jobs_processing_emergency = -1, max_downloads = -1;
       if(max_jobs_s.length() != 0) {
         if(!Arc::stringto(max_jobs_s,i)) {
           logger.msg(Arc::ERROR,"Wrong number in maxload: %s",max_jobs_s);
@@ -296,8 +307,8 @@ bool configure_serviced_users(JobUsers &users,uid_t my_uid,const std::string &my
     }
     else if(command == "wakeupperiod") {
       std::string wakeup_s = config_next_arg(rest);
-      unsigned int wakeup_period;
       if(wakeup_s.length() != 0) {
+        unsigned int wakeup_period;
         if(!Arc::stringto(wakeup_s,wakeup_period)) {
           logger.msg(Arc::ERROR,"Wrong number in wakeupperiod: %s",wakeup_s);
           goto exit;
@@ -335,8 +346,8 @@ bool configure_serviced_users(JobUsers &users,uid_t my_uid,const std::string &my
     }
     else if(command == "maxtransfertries") {
       std::string maxtries_s = config_next_arg(rest);
-      int max_retries = DEFAULT_MAX_RETRIES;
       if(maxtries_s.length() != 0) {
+        int max_retries = DEFAULT_MAX_RETRIES;
         if(!Arc::stringto(maxtries_s,max_retries)) {
           logger.msg(Arc::ERROR,"Wrong number in maxtransfertries"); goto exit;
         };
@@ -490,7 +501,23 @@ bool configure_serviced_users(JobUsers &users,uid_t my_uid,const std::string &my
       };
       jcfg.SetNewDataStaging(use_new_data_staging);
     }
-    else if(command == "sessiondir") {
+    else if(command == "delivery_service") {
+      std::string url = config_next_arg(rest);
+      if (!jcfg.AddDeliveryService(url)) {
+        logger.msg(Arc::ERROR, "Bad URL in delivery_service: %s", url);
+        goto exit;
+      }
+    }
+    else if(command == "local_delivery") {
+      std::string use_local = config_next_arg(rest);
+      if (use_local == "yes") {
+        if (!jcfg.AddDeliveryService("file:/local")) {
+          logger.msg(Arc::ERROR, "Could not add file:/local to delivery services");
+          goto exit;
+        }
+      }
+    }
+   else if(command == "sessiondir") {
       /* set session root directory - applied
          to all following 'control' commands */
       std::string session_root = config_next_arg(rest);
@@ -560,7 +587,7 @@ bool configure_serviced_users(JobUsers &users,uid_t my_uid,const std::string &my
               CacheConfig cache_config = CacheConfig(users.Env(),user->UnixName());
               user->SetCacheParams(cache_config);
             }
-            catch (CacheConfigException e) {
+            catch (CacheConfigException& e) {
               logger.msg(Arc::ERROR, "Error with cache configuration: %s", e.what());
               goto exit;
             }
@@ -790,6 +817,8 @@ bool configure_serviced_users(Arc::XMLNode cfg,JobUsers &users,uid_t my_uid,cons
       TCPPortRange
       UDPPortRange
     httpProxy
+    deliveryService
+    localDelivery
   */
   tmp_node = cfg["dataTransfer"];
   if(tmp_node) {
@@ -802,6 +831,7 @@ bool configure_serviced_users(Arc::XMLNode cfg,JobUsers &users,uid_t my_uid,cons
     bool use_passive_transfer = true;
     bool use_local_transfer = false;
     bool use_new_data_staging = false;
+    bool use_local_delivery = false;
     Arc::XMLNode to_node = tmp_node["timeouts"];
     if(to_node) {
       elementtoint(tmp_node,"minSpeed",min_speed,&logger);
@@ -824,6 +854,18 @@ bool configure_serviced_users(Arc::XMLNode cfg,JobUsers &users,uid_t my_uid,cons
     }
     std::string preferred_pattern = tmp_node["preferredPattern"];
     jcfg.SetPreferredPattern(preferred_pattern);
+    std::string delivery_service = tmp_node["deliveryService"];
+    if (!delivery_service.empty() && !jcfg.AddDeliveryService(delivery_service)) {
+      logger.msg(Arc::ERROR, "Bad URL in deliveryService: %s", delivery_service);
+      return false;
+    }
+    elementtobool(tmp_node,"localDelivery",use_local_delivery,&logger);
+    if (use_local_delivery) {
+      if (!jcfg.AddDeliveryService("file:/local")) {
+        logger.msg(Arc::ERROR, "Could not add file:/local to delivery services");
+        return false;
+      }
+    }
   };
   /*
   serviceMail
@@ -996,7 +1038,7 @@ bool configure_serviced_users(Arc::XMLNode cfg,JobUsers &users,uid_t my_uid,cons
       };
       userlist.push_back(username);
     };
-    if(userlist.size() == 0) {
+    if(userlist.empty()) {
       logger.msg(Arc::ERROR,"No username entries in control directory"); return false;
     };
     for(std::list<std::string>::iterator username = userlist.begin();
@@ -1033,7 +1075,7 @@ bool configure_serviced_users(Arc::XMLNode cfg,JobUsers &users,uid_t my_uid,cons
             CacheConfig cache_config(users.Env(),user->UnixName());
             user->SetCacheParams(cache_config);
           }
-          catch (CacheConfigException e) {
+          catch (CacheConfigException& e) {
             logger.msg(Arc::ERROR, "Error with cache configuration: %s", e.what());
             return false;
           }
