@@ -2,6 +2,7 @@
 #include <list>
 
 #include <arc/XMLNode.h>
+#include <arc/Thread.h>
 
 class GMEnvironment;
 class JobUsers;
@@ -12,6 +13,8 @@ namespace ARex {
 class GridManager {
  private:
   bool active_;
+  bool tostop_;
+  Arc::SimpleCondition* sleep_cond_;
   GMEnvironment* env_;
   JobUsers* users_;
   DTRGenerator* dtr_generator_;
