@@ -125,10 +125,8 @@ DTRGenerator::DTRGenerator(const JobUsers& users,
   scheduler.SetSlots(max_processing,max_processing,max_processing,max_processing_emergency);
 
   // transfer shares
-  DataStaging::TransferShares shares;
-  shares.set_reference_shares(jcfg.GetLimitedShares());
-  shares.set_share_type(jcfg.GetShareType());
-  scheduler.SetTransferShares(shares);
+  DataStaging::TransferSharesConf share_conf(jcfg.GetShareType(), jcfg.GetLimitedShares());
+  scheduler.SetTransferSharesConf(share_conf);
 
   // transfer limits
   DataStaging::TransferParameters transfer_limits;
@@ -225,24 +223,92 @@ bool DTRGenerator::queryJobFinished(JobDescription& job) {
     return false;
   }
   std::map<std::string, std::string>::iterator i = finished_jobs.find(job.get_id());
-  if (i == finished_jobs.end()) {
-    // not in active or finished - must have been picked up already
-    // or nothing to transfer
-    lock.unlock();
-    return true;
-  }
-  // add failure to job if any DTR failed
-  if (!i->second.empty())
+  if (i != finished_jobs.end() && !i->second.empty()) {
+    // add failure to job if any DTR failed
     job.AddFailure(i->second);
-  finished_jobs.erase(i);
+    finished_jobs[job.get_id()] = "";
+  }
   lock.unlock();
   return true;
 }
 
+bool DTRGenerator::hasJob(const JobDescription& job) {
+
+  // check if this job is still in the received jobs queue
+  event_lock.lock();
+  for (std::list<JobDescription>::iterator i = jobs_received.begin(); i != jobs_received.end(); ++i) {
+    if (*i == job) {
+      event_lock.unlock();
+      return true;
+    }
+  }
+  event_lock.unlock();
+
+  // check if any DTRs in this job are still active
+  lock.lock();
+  if (active_dtrs.find(job.get_id()) != active_dtrs.end()) {
+    lock.unlock();
+    return true;
+  }
+
+  // finally check finished jobs
+  std::map<std::string, std::string>::iterator i = finished_jobs.find(job.get_id());
+  if (i != finished_jobs.end()) {
+    lock.unlock();
+    return true;
+  }
+  lock.unlock();
+  // not found
+  return false;
+}
+
+void DTRGenerator::removeJob(const JobDescription& job) {
+
+  // check if this job is still in the received jobs queue
+  event_lock.lock();
+  for (std::list<JobDescription>::iterator i = jobs_received.begin(); i != jobs_received.end(); ++i) {
+    if (*i == job) {
+      event_lock.unlock();
+      logger.msg(Arc::WARNING, "%s: Trying to remove job from data staging which is still active", job.get_id());
+      return;
+    }
+  }
+  event_lock.unlock();
+
+  // check if any DTRs in this job are still active
+  lock.lock();
+  if (active_dtrs.find(job.get_id()) != active_dtrs.end()) {
+    lock.unlock();
+    logger.msg(Arc::WARNING, "%s: Trying to remove job from data staging which is still active", job.get_id());
+    return;
+  }
+
+  // finally check finished jobs
+  std::map<std::string, std::string>::iterator i = finished_jobs.find(job.get_id());
+  if (i == finished_jobs.end()) {
+    // warn if not in finished
+    lock.unlock();
+    logger.msg(Arc::WARNING, "%s: Trying remove job from data staging which does not exist", job.get_id());
+    return;
+  }
+  finished_jobs.erase(i);
+  lock.unlock();
+}
 
 bool DTRGenerator::processReceivedDTR(DataStaging::DTR& dtr) {
 
   std::string jobid(dtr.get_parent_job_id());
+  if (!dtr) {
+    logger.msg(Arc::ERROR, "%s: Invalid DTR", jobid);
+    if (dtr.get_status() != DataStaging::DTRStatus::CANCELLED) {
+      scheduler.cancelDTRs(jobid);
+      lock.lock();
+      finished_jobs[jobid] = std::string("Invalid Data Transfer Request");
+      active_dtrs.erase(jobid);
+      lock.unlock();
+    }
+    return false;
+  }
   logger.msg(Arc::DEBUG, "%s: Received DTR %s to copy file %s in state %s",
                           jobid, dtr.get_id(), dtr.get_source()->str(), dtr.get_status().str());
 
@@ -451,22 +517,15 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR& dtr) {
   // add to finished jobs (without overwriting existing error)
   finished_jobs[jobid] += "";
 
-  // log summary to DTR log
-  const std::list<Arc::LogDestination*> log_dests = Arc::Logger::getRootLogger().getDestinations();
-  Arc::Logger::getRootLogger().removeDestinations();
-
+  // log summary to DTR log and A-REX log
   if (finished_jobs[jobid].empty())
-    dtr.get_logger()->msg(Arc::INFO, "All %s %s successfully",
+    dtr.get_logger()->msg(Arc::INFO, "%s: All %s %s successfully", jobid,
                           dtr.get_source()->Local() ? "uploads":"downloads",
                           (dtr.get_status() == DataStaging::DTRStatus::CANCELLED) ? "cancelled":"finished");
   else
-    dtr.get_logger()->msg(Arc::INFO, "Some %s failed",
+    dtr.get_logger()->msg(Arc::INFO, "%s: Some %s failed", jobid,
                           dtr.get_source()->Local() ? "uploads":"downloads");
-
   lock.unlock();
-  Arc::Logger::getRootLogger().addDestinations(log_dests);
-
-  logger.msg(Arc::INFO, "%s: Data staging finished", jobid);
 
   // wake up GM thread
   if (kicker_func)
@@ -650,14 +709,22 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
 
   if (files.empty()) {
     // nothing to do so wake up GM thread and return
+    lock.lock();
+    finished_jobs[jobid] = "";
+    lock.unlock();
     if (kicker_func) (*kicker_func)(kicker_arg);
     return true;
   }
 
+  // flag to say whether at least one file needs to be staged
+  bool staging = false;
+  Arc::LogLevel log_level = Arc::Logger::getRootLogger().getThreshold();
+
   for (std::list<FileData>::iterator i = files.begin(); i != files.end(); ++i) {
     if (i->lfn.find(":") == std::string::npos)
       continue; // user down/uploadable file
 
+    staging = true;
     std::string source;
     std::string destination;
     if (job.get_state() == JOB_STATE_PREPARING) {
@@ -700,11 +767,11 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
     active_dtrs.insert(std::pair<std::string, std::string>(jobid, dtr.get_id()));
     lock.unlock();
 
-    // Disconnect from root logger while submitting to Scheduler
-    const std::list<Arc::LogDestination*> log_dests = Arc::Logger::getRootLogger().getDestinations();
-    Arc::Logger::getRootLogger().removeDestinations();
+    // Avoid logging when possible during scheduler submission because it gets
+    // blocked by LFC calls locking the environment
+    Arc::Logger::getRootLogger().setThreshold(Arc::ERROR);
     dtr.push(DataStaging::SCHEDULER);
-    Arc::Logger::getRootLogger().addDestinations(log_dests);
+    Arc::Logger::getRootLogger().setThreshold(log_level);
 
     // update .local with transfer share
     JobLocalDescription *job_desc = new JobLocalDescription;
@@ -719,6 +786,11 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
     }
     delete job_desc;
   }
+  if (!staging) { // nothing needed staged so mark as finished
+    lock.lock();
+    finished_jobs[jobid] = "";
+    lock.unlock();
+  }
   return true;
 }
 
