@@ -18,7 +18,7 @@
 #include <arc/client/Job.h>
 #include <arc/client/JobController.h>
 #include <arc/UserConfig.h>
-#include <arc/data/CheckSum.h>
+#include <arc/CheckSum.h>
 #include <arc/data/FileCache.h>
 #include <arc/data/DataHandle.h>
 #include <arc/data/DataMover.h>
@@ -74,25 +74,27 @@ class FileDataEx : public FileData {
   typedef std::list<FileDataEx>::iterator iterator;
   Arc::DataStatus res;
   PointPair* pair;
-  int size; /* size of the file in bytes */
   /* Times are string to eliminate the need to convert
    * string to time_t while reading from local file
    */
   std::string starttime; /* time of transfer started */
-  std::string endtime; /* time of trnasfer finished */
+  std::string endtime; /* time of transfer finished */
   /* if the file was retrieved from cache: 
    *  - "yes";
    *  - "no"; 
    */
   std::string fromcache;
+  time_t mtime;
   FileDataEx(const FileData& f)
     : FileData(f),
       res(Arc::DataStatus::Success),
-      pair(NULL) {}
+      pair(NULL),
+      mtime((time_t)(-1)) {}
   FileDataEx(const FileData& f, Arc::DataStatus r)
     : FileData(f),
       res(r),
-      pair(NULL) {}
+      pair(NULL),
+      mtime((time_t)(-1)) {}
 };
 
 static std::list<FileData> job_files_;
@@ -114,7 +116,7 @@ class SimpleConditionLock {
   };
 };
 
-int clean_files(std::list<FileData> &job_files,char* session_dir) {
+static int clean_files(std::list<FileData> &job_files,char* session_dir) {
   std::string session(session_dir);
   /* delete only downloadable files, let user manage his/hers files */
   std::list<FileData> tmp;
@@ -128,13 +130,15 @@ int clean_files(std::list<FileData> &job_files,char* session_dir) {
            1 - it is not proper file or other error
            2 - not here yet
 */
-int user_file_exists(FileData &dt,char* session_dir,std::list<std::string>* have_files,std::string* error = NULL) {
+static int user_file_exists(FileDataEx &dt,char* session_dir,std::list<std::string>* have_files,std::string* error = NULL) {
   struct stat st;
   const char *str = dt.lfn.c_str();
   if(strcmp(str,"*.*") == 0) return 0; /* do not wait for this file */
   std::string fname=std::string(session_dir) + '/' + dt.pfn;
   /* check if file does exist at all */
+  // TODO: FileAccess?
   if(lstat(fname.c_str(),&st) != 0) return 2;
+  dt.mtime = st.st_mtime;
   /* check for misconfiguration */
   /* parse files information */
   char *str_;
@@ -199,14 +203,13 @@ int user_file_exists(FileData &dt,char* session_dir,std::list<std::string>* have
     Arc::CRC32Sum crc;
     char buffer[1024];
     ssize_t l;
-    size_t ll = 0;
     for(;;) {
       if((l=read(h,buffer,1024)) == -1) {
         logger.msg(Arc::ERROR, "Error reading file %s", dt.pfn);
         if(error) (*error)="Could not read file to compute checksum.";
         return 1;
       };
-      if(l==0) break; ll+=l;
+      if(l==0) break;
       crc.add(buffer,l);
     };
     close(h);
@@ -224,18 +227,13 @@ int user_file_exists(FileData &dt,char* session_dir,std::list<std::string>* have
 }
 
 class PointPair {
- private:
-  Arc::URL source_url;
-  Arc::URL destination_url;
  public:
   Arc::DataHandle source;
   Arc::DataHandle destination;
   PointPair(const std::string& source_str, const std::string& destination_str,
       const Arc::UserConfig& usercfg)
-    : source_url(source_str),
-      destination_url(destination_str),
-      source(source_url, usercfg),
-      destination(destination_url, usercfg) {};
+    : source(source_str, usercfg),
+      destination(destination_str, usercfg) {};
   ~PointPair(void) {};
   static void callback(Arc::DataMover*,Arc::DataStatus res,void* arg) {
     FileDataEx::iterator &it = *((FileDataEx::iterator*)arg);
@@ -275,7 +273,8 @@ int main(int argc,char** argv) {
   int res=0;
   bool not_uploaded;
   time_t start_time=time(NULL);
-  time_t upload_timeout = 0;
+  time_t files_changed = 0;
+  bool first_loop = true;
   int n_threads = 1;
   int n_files = MAX_DOWNLOADS;
   /* if != 0 - change owner of downloaded files to this user */
@@ -439,7 +438,7 @@ int main(int argc,char** argv) {
   if(file_group != 0) { gid=file_group; }
   else { gid= getgid(); };
   desc.set_uid(uid,gid);
-  JobUser user(env,uid);
+  JobUser user(env,uid,gid);
   user.SetControlDir(control_dir);
   user.SetSessionRoot(session_dir);
 
@@ -473,7 +472,7 @@ int main(int argc,char** argv) {
         exit(1);
       }
     }
-    catch (CacheConfigException e) {
+    catch (CacheConfigException& e) {
       logger.msg(Arc::ERROR, "Error with cache configuration: %s", e.what());
       delete cache;
       exit(1);
@@ -612,12 +611,6 @@ int main(int argc,char** argv) {
 
   // initialize structures to handle download
   /* TODO: add threads=# to all urls if n_threads!=1 */
-  // Compute wait time for user files
-  for(FileDataEx::iterator i=job_files.begin();i!=job_files.end();++i) {
-    if(i->lfn.find(":") == std::string::npos) { /* is it lfn ? */
-      upload_timeout+=MAX_USER_TIME;
-    };
-  };
   // Main download cycle
   if(!userfiles_only) for(;;) {
     // Initiate transfers
@@ -712,6 +705,7 @@ int main(int argc,char** argv) {
       credentials_expired=true;
     transferred=false;
   };
+  files_changed = ::time(NULL);
   // Check if all files have been properly downloaded
   if(!transferred) {
     logger.msg(Arc::INFO, "Some downloads failed"); res=2;
@@ -734,9 +728,11 @@ int main(int argc,char** argv) {
     for(FileDataEx::iterator i=job_files.begin();i!=job_files.end();) {
       if(i->lfn.find(":") == std::string::npos) { /* is it lfn ? */
         /* process user uploadable file */
-        logger.msg(Arc::INFO, "Check user uploadable file: %s", i->pfn);
+        if(first_loop) logger.msg(Arc::INFO, "Checking user uploadable file: %s", i->pfn);
         std::string error;
+        time_t mtime = i->mtime;
         int err=user_file_exists(*i,session_dir,uploaded_files_,&error);
+        if(mtime != i->mtime) files_changed = ::time(NULL);
         if(err == 0) { /* file is uploaded */
           logger.msg(Arc::INFO, "User has uploaded file %s", i->pfn);
           i=job_files.erase(i);
@@ -759,14 +755,12 @@ int main(int argc,char** argv) {
         ++i;
       };
     };
+    first_loop = false;
     if(!not_uploaded) break;
     // check for timeout
-    if((time(NULL)-start_time) > upload_timeout) {
-      for(FileDataEx::iterator i=job_files.begin();i!=job_files.end();++i) {
-        if(i->lfn.find(":") == std::string::npos) { /* is it lfn ? */
-          failure_reason+="User file: "+i->pfn+" - Timeout waiting\n";
-        };
-      };
+    unsigned int time_passed = (unsigned int)(time(NULL) - files_changed);
+    if(time_passed > max_inactivity_time) {
+      logger.msg(Arc::INFO, "No changes in uploadable files for %u seconds",time_passed);
       logger.msg(Arc::ERROR, "Uploadable files timed out"); res=2; break;
     };
     if(uploaded_files_) {
