@@ -89,7 +89,7 @@ sub collect($) {
             my $lrmsjob = $lrms_info->{jobs}{$lrmsid};
             unless ((defined $lrmsjob) and $lrmsjob->{status}) {
                 $log->warning("LRMS plugin returned no status for job $jobid (lrmsid: $lrmsid)");
-                die;
+                next;
             }
             if ((defined $lrmsjob) and $lrmsjob->{status} ne 'EXECUTED') {
                 if ($lrmsjob->{status} eq 'R' or $lrmsjob->{status} eq 'S') {
@@ -417,6 +417,7 @@ sub collect($) {
                     $j->{usedmem} = $gmjob->{UsedMem} if $gmjob->{UsedMem};
                     $j->{completiontime} = $gmjob->{completiontime} if $gmjob->{completiontime};
                     $j->{errors} = join "; ", @{$gmjob->{errors}} if $gmjob->{errors};
+                    $j->{errors} =~ s/\r//g; # workaround for BDII bug
                     $j->{exitcode} = $gmjob->{exitcode} if defined $gmjob->{exitcode};
                 }
 
