@@ -42,7 +42,8 @@ our $j = { 'jobID' => {
             runtimeenvironments=> [ '*' ],
             # from .status
             status             => '',
-            statustime         => '',  # seconds since epoch
+            statusmodified     => '',  # seconds since epoch
+            statusread         => '',  # seconds since epoch
             completiontime     => '*', # MDS time format
             localowner         => '',
             # from .failed
@@ -77,18 +78,6 @@ sub switchEffectiveUser {
     };
 }
 
-# extract the lower limit of a RangeValue_Type expression
-sub get_range_minimum($$) {
-    my ($range_str, $jsdl_ns_re) = @_;
-    my $min;
-    my $any_range_re = "$jsdl_ns_re(?:Exact|(?:Lower|Upper)Bound(?:edRange)?)";
-    while ($range_str =~ m!<($any_range_re)\b[^>]*>[\s\n]*(-?[\d.]+)[\s\n]*</\1!mg) {
-        $min = $2 if not defined $min or $min > $2;
-    }
-    return $min;
-}
-
-
 sub collect {
     my ($controls, $remotegmdirs, $nojobs) = @_;
 
@@ -213,7 +202,8 @@ sub get_gmjobs {
                     $log->warning("Job $ID: Cannot determine user name for owner (uid $uid)");
                 }
 
-                $job->{"statustime"} = $file_stat[9];
+                $job->{"statusmodified"} = $file_stat[9];
+                $job->{"statusread"} = time();
                 # completiontime
                 if ($job->{"status"} eq "FINISHED") {
                     my ($s,$m,$h,$D,$M,$Y) = gmtime($file_stat[9]);
