@@ -15,12 +15,12 @@ use IO::Handle;
 sub translator;
 
 # Global variables for translator
-use vars qw($DEFAULT); $DEFAULT = -1;
+use vars qw($DEFAULT); $DEFAULT = 0;
 use vars qw($outbIP $inbIP $glueSubClusterUniqueID $norduBenchmark $norduOpsys $norduNodecput $norduNodecpu);
 use vars qw($glueHostMainMemoryRAMSize $glueHostArchitecturePlatformType $glueSubClusterUniqueID $GlueHostBenchmarkSI00 $GlueHostBenchmarkSF00);
-use vars qw($glueSubClusterName $glueSubClusterPhysicalCPUs $glueSubClusterLogicalCPUs $glueClusterUniqueID $processorOtherDesc);
-use vars qw($AssignedSlots $mappedStatus $waitingJobs $totalJobs $waitingJobs $freeSlots $estRespTime $worstRespTime);
-use vars qw(@envs);
+use vars qw($glueSubClusterName $glueSubClusterPhysicalCPUs $glueSubClusterLogicalCPUs $glueClusterUniqueID $processorOtherDesc $smpSize);
+use vars qw($AssignedSlots $mappedStatus $waitingJobs $totalJobs $waitingJobs $freeSlots $estRespTime $worstRespTime $vo $ce_unique_id);
+use vars qw(@envs @vos);
 
 #Create nordugrid ldif
 #TODO read this from optarg
@@ -161,9 +161,11 @@ sub translator(){
 	$glueSubClusterUniqueID=$cluster_attributes{'nordugrid-cluster-name'};
 	$glueSubClusterName=$glue_site_unique_id;
 	if ( $processorOtherDesc =~ m/Cores=(\d+)/ ){
-	    $glueSubClusterPhysicalCPUs=int($cluster_attributes{'nordugrid-cluster-totalcpus'}/$1);
+	    $smpSize=$1;
+	    $glueSubClusterPhysicalCPUs=int($cluster_attributes{'nordugrid-cluster-totalcpus'}/$smpSize);
 	}
 	else {
+	    $smpSize=1;
 	    $glueSubClusterPhysicalCPUs=$cluster_attributes{'nordugrid-cluster-totalcpus'};
 	}
 	$glueSubClusterLogicalCPUs=$cluster_attributes{'nordugrid-cluster-totalcpus'};
@@ -212,7 +214,7 @@ objectClass: GlueKey
 	print "GlueHostApplicationSoftwareRunTimeEnvironment:  $_\n"
     }
  
-    print "GlueHostArchitectureSMPSize: 2
+    print "GlueHostArchitectureSMPSize: $smpSize
 GlueHostNetworkAdapterInboundIP: $inbIP
 GlueHostNetworkAdapterOutboundIP: $outbIP";
  
@@ -365,7 +367,14 @@ sub write_gluece_entries(){
 		$glueHostArchitecturePlatformType=$queue_attributes{'nordugrid-queue-architecture'}; ##XX
 		$glueSubClusterUniqueID=$queue_attributes{'nordugrid-queue-name'};  ##XX
 		$glueSubClusterName=$queue_attributes{'nordugrid-queue-name'};  ##XX
-		$glueSubClusterPhysicalCPUs=$queue_attributes{'nordugrid-queue-totalcpus'};  ##XX
+		if ( $processorOtherDesc =~ m/Cores=(\d+)/ ){
+		    $smpSize=$1;
+		    $glueSubClusterPhysicalCPUs=int($queue_attributes{'nordugrid-queue-totalcpus'}/$smpSize);
+		}
+		else {
+		    $smpSize=1;
+		    $glueSubClusterPhysicalCPUs=$queue_attributes{'nordugrid-queue-totalcpus'};
+		}
 		$glueSubClusterLogicalCPUs=$queue_attributes{'nordugrid-queue-totalcpus'};  ##XX
 		$glueClusterUniqueID=$cluster_attributes{'nordugrid-cluster-name'};  ##XX
 
@@ -413,6 +422,7 @@ sub write_gluece_entries(){
 	    # T1 services
 	    $estRespTime = int(600 + ($queue_attributes{'nordugrid-queue-running'} /$queue_attributes{'nordugrid-queue-maxrunning'}) *3600 + ($waitingJobs /$queue_attributes{'nordugrid-queue-maxrunning'}) * 600 );
 	    $worstRespTime = $estRespTime + 2000;
+	    my @vos= split / /, $cluster_attributes{'nordugrid-cluster-acl'};
 
 	    # Write CE Entries
 	    
@@ -452,14 +462,48 @@ GlueCEPolicyMaxRunningJobs: $queue_attributes{'nordugrid-queue-maxrunning'}
 GlueCEPolicyMaxTotalJobs: $queue_attributes{'nordugrid-queue-maxqueuable'}
 GlueCEPolicyMaxWallClockTime: $queue_attributes{'nordugrid-queue-maxcputime'}
 GlueCEPolicyPriority: 1
-GlueCEPolicyAssignedJobSlots: $AssignedSlots
-GlueCEAccessControlBaseRule: $cluster_attributes{'nordugrid-cluster-acl'}
-GlueForeignKey: GlueClusterUniqueID=$cluster_attributes{'nordugrid-cluster-name'}
+GlueCEPolicyAssignedJobSlots: $AssignedSlots\n";
+            foreach (@vos){
+                chomp;
+                print "GlueCEAccessControlBaseRule: $_\n";
+            }
+            print "GlueForeignKey: GlueClusterUniqueID=$cluster_attributes{'nordugrid-cluster-name'}
 GlueSchemaVersionMajor: 1
 GlueSchemaVersionMinor: 2
 ";
-	    
-	}
+
+            foreach (@vos){
+                chomp;
+        	$vo = $_;
+        	$vo =~ s/VO:// ;
+
+                print "
+dn: GlueVOViewLocalID=$vo,GlueCEUniqueID=$ce_unique_id,Mds-Vo-name=resource,o=grid
+objectClass: GlueCETop
+objectClass: GlueVOView
+objectClass: GlueCEInfo
+objectClass: GlueCEState
+objectClass: GlueCEAccessControlBase
+objectClass: GlueCEPolicy
+objectClass: GlueKey
+objectClass: GlueSchemaVersion
+GlueSchemaVersionMajor: 1
+GlueSchemaVersionMinor: 2
+GlueCEInfoDefaultSE: $cluster_attributes{'nordugrid-cluster-localse'}
+GlueCEStateTotalJobs: $totalJobs
+GlueCEInfoDataDir: unset
+GlueCEAccessControlBaseRule: VO:$vo
+GlueCEStateRunningJobs: $queue_attributes{'nordugrid-queue-running'}
+GlueChunkKey: GlueCEUniqueID=$ce_unique_id
+GlueVOViewLocalID: $vo
+GlueCEInfoApplicationDir: unset
+GlueCEStateWaitingJobs: $waitingJobs
+GlueCEStateEstimatedResponseTime: $estRespTime
+GlueCEStateWorstResponseTime: $worstRespTime
+GlueCEStateFreeJobSlots: $freeSlots
+";
+            }
+        }
     }
 }
 
