@@ -13,12 +13,13 @@
 
 static Arc::Logger logger(Arc::Logger::getRootLogger(),"userspec_t");
 
-void userspec_t::free(void) {
+void userspec_t::free(void) const {
   // Keep authentication info to preserve proxy (just in case)
 }
 
 //userspec_t::userspec_t(void):user(),map(user),default_map(user),name(NULL),group(NULL),home(NULL),gridmap(false) {
-userspec_t::userspec_t(void):user(),uid(-1),gid(-1),map(user),default_map(user),gridmap(false) {
+userspec_t::userspec_t(void):user(),uid(-1),gid(-1),port(0),map(user),default_map(user),gridmap(false) {
+  host[0] = 0;
 }
 
 userspec_t::~userspec_t(void) {
@@ -65,36 +66,34 @@ bool check_gridmap(const char* dn,char** user,const char* mapfile) {
   return false;
 }
 
-/*
-int fill_user_spec(userspec_t *spec,globus_ftp_control_auth_info_t *auth,globus_ftp_control_handle_t *handle) {
-  if(spec == NULL) return 1;
-  if(auth == NULL) return 1;
-  if(!(spec->fill(auth,handle)
-    )) return 1;
-  return 0;
-}
-*/
-
-bool userspec_t::fill(globus_ftp_control_auth_info_t *auth,globus_ftp_control_handle_t *handle) {
+bool userspec_t::fill(globus_ftp_control_auth_info_t *auth,globus_ftp_control_handle_t *handle, const char* cfg) {
   struct passwd pw_;
   struct group gr_;
-  struct passwd *pw;
-  struct group *gr;
-  char buf[BUFSIZ];
+  struct passwd* pw=NULL;
+  struct group* gr=NULL;
+  char bufp[BUFSIZ];
+  char bufg[BUFSIZ];
+  if(cfg) config_file = cfg;
   if(auth == NULL) return false;
   if(auth->auth_gssapi_subject == NULL) return false;
   std::string subject = auth->auth_gssapi_subject;
   gridftpd::make_unescaped_string(subject);
   char* name=NULL;
+  char* gname=NULL;
   if(!check_gridmap(subject.c_str(),&name)) {
     logger.msg(Arc::WARNING, "There is no local mapping for user");
-    name=NULL;
   } else {
     if((name == NULL) || (name[0] == 0)) {
       logger.msg(Arc::WARNING, "There is no local name for user");
       if(name) { std::free(name); name=NULL; };
     } else {
       gridmap=true;
+      gname = strchr(name,':');
+      if(gname) {
+        *gname = 0;
+        ++gname;
+        if(gname[0] == 0) gname = NULL;
+      };
     };
   };
   // fill host info
@@ -129,23 +128,35 @@ bool userspec_t::fill(globus_ftp_control_auth_info_t *auth,globus_ftp_control_ha
       user.set(auth->auth_gssapi_subject,auth->auth_gssapi_context,
                auth->delegated_credential_handle,abuf);
     };
+  } else {
+    user.set(auth->auth_gssapi_subject,auth->auth_gssapi_context,
+             auth->delegated_credential_handle);
   };
   if((!user.is_proxy()) || (user.proxy() == NULL) || (user.proxy()[0] == 0)) {
     logger.msg(Arc::INFO, "No proxy provided");
   } else {
-    logger.msg(Arc::VERBOSE, "Proxy stored at %s", user.proxy());
+    logger.msg(Arc::VERBOSE, "Proxy/credentials stored at %s", user.proxy());
   };
   if((getuid() == 0) && name) {
     logger.msg(Arc::INFO, "Initially mapped to local user: %s", name);
-    getpwnam_r(name,&pw_,buf,BUFSIZ,&pw);
+    getpwnam_r(name,&pw_,bufp,BUFSIZ,&pw);
     if(pw == NULL) {
-      logger.msg(Arc::ERROR, "Local user does not exist");
+      logger.msg(Arc::ERROR, "Local user %s does not exist",name);
       std::free(name); name=NULL;
       return false;
     };
+    if(gname) {
+      logger.msg(Arc::INFO, "Initially mapped to local group: %s", gname);
+      getgrnam_r(gname,&gr_,bufg,BUFSIZ,&gr);
+      if(gr == NULL) {
+        logger.msg(Arc::ERROR, "Local group %s does not exist",gname);
+        std::free(name); name=NULL;
+        return false;
+      };
+    };
   } else {
-    if(name) std::free(name); name=NULL;
-    getpwuid_r(getuid(),&pw_,buf,BUFSIZ,&pw);
+    if(name) std::free(name); name=NULL; gname=NULL;
+    getpwuid_r(getuid(),&pw_,bufp,BUFSIZ,&pw);
     if(pw == NULL) {
       logger.msg(Arc::WARNING, "Running user has no name");
     } else {
@@ -155,12 +166,18 @@ bool userspec_t::fill(globus_ftp_control_auth_info_t *auth,globus_ftp_control_ha
   };
   if(pw) {
     uid=pw->pw_uid;
-    gid=pw->pw_gid;
-    logger.msg(Arc::INFO, "Mapped to local id: %i", pw->pw_uid);
+    if(gr) {
+      gid=gr->gr_gid;
+    } else {
+      gid=pw->pw_gid;
+    };
+    logger.msg(Arc::INFO, "Mapped to local id: %i", uid);
     home=pw->pw_dir;
-    getgrgid_r(pw->pw_gid,&gr_,buf,BUFSIZ,&gr);
-    if(gr == NULL) {
-      logger.msg(Arc::ERROR, "No group %i for mapped user", gid);
+    if(!gr) {
+      getgrgid_r(gid,&gr_,bufg,BUFSIZ,&gr);
+      if(gr == NULL) {
+        logger.msg(Arc::ERROR, "No group %i for mapped user", gid);
+      };
     };
     std::string mapstr;
     if(name) mapstr+=name;
@@ -168,22 +185,26 @@ bool userspec_t::fill(globus_ftp_control_auth_info_t *auth,globus_ftp_control_ha
     if(gr) mapstr+=gr->gr_name;
     mapstr+=" all";
     default_map.mapname(mapstr.c_str());
-    logger.msg(Arc::INFO, "Mapped to local group id: %i", pw->pw_gid);
+    logger.msg(Arc::INFO, "Mapped to local group id: %i", gid);
     if(gr) logger.msg(Arc::INFO, "Mapped to local group name: %s", gr->gr_name);
     logger.msg(Arc::VERBOSE, "Mapped user's home: %s", home);
   };
   if(name) std::free(name);
+  if(!user) return false;
   return true;
 }
 
-bool userspec_t::fill(AuthUser& u) {
+bool userspec_t::fill(AuthUser& u, const char* cfg) {
   struct passwd pw_;
   struct group gr_;
   struct passwd *pw;
   struct group *gr;
-  char buf[BUFSIZ];
+  char bufp[BUFSIZ];
+  char bufg[BUFSIZ];
   std::string subject = u.DN();
   char* name=NULL;
+  char* gname=NULL;
+  if(cfg) config_file = cfg;
   if(!check_gridmap(subject.c_str(),&name)) {
     logger.msg(Arc::WARNING, "There is no local mapping for user");
     name=NULL;
@@ -193,6 +214,12 @@ bool userspec_t::fill(AuthUser& u) {
       if(name) { std::free(name); name=NULL; };
     } else {
       gridmap=true;
+      gname = strchr(name,':');
+      if(gname) {
+        *gname = 0;
+        ++gname;
+        if(gname[0] == 0) gname = NULL;
+      };
     };
   };
   user=u;
@@ -203,15 +230,24 @@ bool userspec_t::fill(AuthUser& u) {
   };
   if((getuid() == 0) && name) {
     logger.msg(Arc::INFO, "Initially mapped to local user: %s", name);
-    getpwnam_r(name,&pw_,buf,BUFSIZ,&pw);
+    getpwnam_r(name,&pw_,bufp,BUFSIZ,&pw);
     if(pw == NULL) {
       logger.msg(Arc::ERROR, "Local user does not exist");
       std::free(name); name=NULL;
       return false;
     };
+    if(gname) {
+      logger.msg(Arc::INFO, "Initially mapped to local group: %s", gname);
+      getgrnam_r(gname,&gr_,bufg,BUFSIZ,&gr);
+      if(gr == NULL) {
+        logger.msg(Arc::ERROR, "Local group %s does not exist",gname);
+        std::free(name); name=NULL;
+        return false;
+      };
+    };
   } else {
-    if(name) std::free(name); name=NULL;
-    getpwuid_r(getuid(),&pw_,buf,BUFSIZ,&pw);
+    if(name) std::free(name); name=NULL; gname=NULL;
+    getpwuid_r(getuid(),&pw_,bufp,BUFSIZ,&pw);
     if(pw == NULL) {
       logger.msg(Arc::WARNING, "Running user has no name");
     } else {
@@ -221,12 +257,18 @@ bool userspec_t::fill(AuthUser& u) {
   };
   if(pw) {
     uid=pw->pw_uid;
-    gid=pw->pw_gid;
-    logger.msg(Arc::INFO, "Mapped to local id: %i", pw->pw_uid);
+    if(gr) {
+      gid=gr->gr_gid;
+    } else {
+      gid=pw->pw_gid;
+    };
+    logger.msg(Arc::INFO, "Mapped to local id: %i", uid);
     home=pw->pw_dir;
-    getgrgid_r(pw->pw_gid,&gr_,buf,BUFSIZ,&gr);
-    if(gr == NULL) {
-      logger.msg(Arc::INFO, "No group %i for mapped user", gid);
+    if(!gr) {
+      getgrgid_r(gid,&gr_,bufg,BUFSIZ,&gr);
+      if(gr == NULL) {
+        logger.msg(Arc::INFO, "No group %i for mapped user", gid);
+      };
     };
     std::string mapstr;
     if(name) mapstr+=name;
