@@ -20,24 +20,20 @@
 
 #include "isis.h"
 
-#ifdef WIN32
-#include <arc/win32.h>
-#endif
-
 namespace ISIS
 {
 
-std::vector<std::string>::iterator Neighbor_Container::find_element(const std::string value) {
+std::vector<std::string>::iterator Neighbor_Container::find_element(const std::string& value) {
     return find(content.begin(), content.end(), value);
 }
 
-bool Neighbor_Container::contains(const std::string val) {
+bool Neighbor_Container::contains(const std::string& val) {
     Glib::Mutex::Lock lock(container_lock);
     bool ret_val = (content.end() != find(content.begin(), content.end(), val));
     return ret_val;
 }
 
-void Neighbor_Container::push(const std::string val) {
+void Neighbor_Container::push(const std::string& val) {
     Glib::Mutex::Lock lock(container_lock);
     content.push_back(val);
 }
@@ -465,7 +461,7 @@ static void soft_state_thread(void *data) {
         }
 
         // Create ServiceURL hash
-        FileCacheHash md5;
+        Arc::FileCacheHash md5;
         // calculate my hash from the endpoint URL
         my_hash = md5.getHash(endpoint_);
 
@@ -869,7 +865,7 @@ static void soft_state_thread(void *data) {
             // Reposition itself in the peer-to-peer network
             // if disconnected from every neighbors then reconnect to
             // the network
-            FileCacheHash md5;
+            Arc::FileCacheHash md5;
             my_hash = md5.getHash(my_hash);
             {
                 Glib::Mutex::Lock lock(connection_lock, Glib::TRY_LOCK);
@@ -1077,7 +1073,7 @@ static void soft_state_thread(void *data) {
         }
 
         if ( peerid.empty() ){
-            FileCacheHash md5;
+            Arc::FileCacheHash md5;
             // calculate hash from the endpoint URL or serviceID
             if ( bool(regentry["SrcAdv"]["EPR"]["Address"]) ){
                peerid = md5.getHash((std::string)regentry["SrcAdv"]["EPR"]["Address"]);
@@ -1385,7 +1381,7 @@ static void soft_state_thread(void *data) {
 } // namespace
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-    { "isis", "HED:SERVICE", 0, &ISIS::get_service },
-    { NULL, NULL, 0, NULL }
+    { "isis", "HED:SERVICE", NULL, 0, &ISIS::get_service },
+    { NULL, NULL, NULL, 0, NULL }
 };
 
