@@ -113,8 +113,8 @@ static Arc::Plugin* get_service(Arc::PluginArgument* arg) {
 }
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-    { "pythonservice", "HED:SERVICE", 0, &get_service },
-    { NULL, NULL, 0, NULL }
+    { "pythonservice", "HED:SERVICE", NULL, 0, &get_service },
+    { NULL, NULL, NULL, 0, NULL }
 };
 
 namespace Arc {
